@@ -6,6 +6,7 @@
  */
 
 #include <algorithm>
+#include <string>
 #include "../simgraph.h"
 #include "../vehicle/simvehikel.h"
 #include "../player/simplay.h"
@@ -16,6 +17,7 @@
 
 #include "../dataobj/umgebung.h"
 #include "../dataobj/tabfile.h"
+#include "../dataobj/loadsave.h"
 
 #include "../besch/bildliste_besch.h"
 #include "../besch/vehikel_besch.h"
@@ -23,8 +25,6 @@
 #include "../bauer/warenbauer.h"
 #include "../bauer/vehikelbauer.h"
 
-#include "../utils/cstring_t.h"
-
 #include "../tpl/inthashtable_tpl.h"
 #include "../tpl/stringhashtable_tpl.h"
 
@@ -42,6 +42,10 @@ public:
 		year = y*12;
 		speed = kmh;
 	};
+	void rdwr(loadsave_t *file) {
+		file->rdwr_long(year);
+		file->rdwr_long(speed);
+	}
 };
 
 // speed boni
@@ -59,11 +63,11 @@ static sint32 default_speedbonus[8] =
 	60	// narrowgauge
 };
 
-bool vehikelbauer_t::speedbonus_init(cstring_t objfilename)
+bool vehikelbauer_t::speedbonus_init(const std::string &objfilename)
 {
 	tabfile_t bonusconf;
 	// first take user data, then user global data
-	if (!bonusconf.open(objfilename+"config/speedbonus.tab")) {
+	if (!bonusconf.open((objfilename+"config/speedbonus.tab").c_str())) {
 		dbg->error("vehikelbauer_t::speedbonus_init()", "Can't read speedbonus.tab" );
 		return false;
 	}
@@ -91,7 +95,6 @@ bool vehikelbauer_t::speedbonus_init(cstring_t objfilename)
 }
 
 
-
 sint32 vehikelbauer_t::get_speedbonus( sint32 monthyear, waytype_t wt )
 {
 	const int typ = wt==air_wt ? 3 : (wt-1)&7;
@@ -145,6 +148,23 @@ sint32 vehikelbauer_t::get_speedbonus( sint32 monthyear, waytype_t wt )
 }
 
 
+void vehikelbauer_t::rdwr_speedbonus(loadsave_t *file)
+{
+	for(  int j=0;  j<8;  j++  ) {
+		uint32 count = speedbonus[j].get_count();
+		file->rdwr_long(count);
+		if (file->is_loading()) {
+			speedbonus[j].clear();
+			speedbonus[j].resize(count);
+		}
+		for(uint32 i=0; i<count; i++) {
+			if (file->is_loading()) {
+				speedbonus[j].append(bonus_record_t());
+			}
+			speedbonus[j][i].rdwr(file);
+		}
+	}
+}
 
 
 vehikel_t* vehikelbauer_t::baue(koord3d k, spieler_t* sp, convoi_t* cnv, const vehikel_besch_t* vb )
@@ -164,8 +184,8 @@ vehikel_t* vehikelbauer_t::baue(koord3d k, spieler_t* sp, convoi_t* cnv, const v
 			dbg->fatal("vehikelbauer_t::baue()", "cannot built a vehicle with waytype %i", vb->get_waytype());
 	}
 
-	sp->buche(-(sint32)vb->get_preis(), k.get_2d(), COST_NEW_VEHICLE );
-	sp->buche( (sint32)vb->get_preis(), COST_ASSETS );
+	sp->buche(-(sint64)vb->get_preis(), k.get_2d(), COST_NEW_VEHICLE );
+	sp->buche( (sint64)vb->get_preis(), COST_ASSETS );
 
 	return v;
 }
@@ -176,6 +196,11 @@ bool vehikelbauer_t::register_besch(const vehikel_besch_t *besch)
 {
 	// printf("N=%s T=%d V=%d P=%d\n", besch->get_name(), besch->get_typ(), besch->get_geschw(), besch->get_leistung());
 
+	const vehikel_besch_t *old_besch = name_fahrzeuge.get( besch->get_name() );
+	if(  old_besch  ) {
+		dbg->warning( "vehikelbauer_t::register_besch()", "Object %s was overlaid by addon!", besch->get_name() );
+		name_fahrzeuge.remove( besch->get_name() );
+	}
 	name_fahrzeuge.put(besch->get_name(), besch);
 
 	// register waytype liste
@@ -185,6 +210,7 @@ bool vehikelbauer_t::register_besch(const vehikel_besch_t *besch)
 		typ_fahrzeuge.put(typ, slist_tpl<const vehikel_besch_t *>());
 		typ_liste = typ_fahrzeuge.access(typ);
 	}
+	typ_liste->remove(old_besch);
 	typ_liste->append(besch);
 
 	// correct for driving on left side
@@ -203,7 +229,7 @@ bool vehikelbauer_t::register_besch(const vehikel_besch_t *besch)
 		display_set_base_image_offset( besch->get_bild_nr(7,NULL), -XOFF-YOFF, 0 );
 
 		if(besch->freight_image_type>0) {
-			const bildliste2d_besch_t *liste2d = static_cast<const bildliste2d_besch_t *>(besch->get_child(5));
+			bildliste2d_besch_t const* const liste2d = besch->get_child<bildliste2d_besch_t>(5);
 			for(int i=0; i<besch->freight_image_type; i++) {
 				display_set_base_image_offset(liste2d->get_bild(0, i)->get_nummer(),        +XOFF, +YOFF);
 				display_set_base_image_offset(liste2d->get_bild(1, i)->get_nummer(),        -XOFF, +YOFF);
@@ -214,21 +240,16 @@ bool vehikelbauer_t::register_besch(const vehikel_besch_t *besch)
 				display_set_base_image_offset(liste2d->get_bild(6, i)->get_nummer(),            0, -YOFF);
 				display_set_base_image_offset(liste2d->get_bild(7, i)->get_nummer(), -XOFF - YOFF,     0);
 			}
+		} else if (bildliste_besch_t const* const liste = besch->get_child<bildliste_besch_t>(5)) {
+			display_set_base_image_offset(liste->get_bild(0)->get_nummer(),        +XOFF, +YOFF);
+			display_set_base_image_offset(liste->get_bild(1)->get_nummer(),        -XOFF, +YOFF);
+			display_set_base_image_offset(liste->get_bild(2)->get_nummer(),            0, +YOFF);
+			display_set_base_image_offset(liste->get_bild(3)->get_nummer(),        +XOFF,     0);
+			display_set_base_image_offset(liste->get_bild(4)->get_nummer(),        -XOFF, -YOFF);
+			display_set_base_image_offset(liste->get_bild(5)->get_nummer(),        +XOFF, -YOFF);
+			display_set_base_image_offset(liste->get_bild(6)->get_nummer(),            0, -YOFF);
+			display_set_base_image_offset(liste->get_bild(7)->get_nummer(), -XOFF - YOFF,     0);
 		}
-		else {
-			const bildliste_besch_t *liste = static_cast<const bildliste_besch_t *>(besch->get_child(5));
-			if(liste) {
-				display_set_base_image_offset(liste->get_bild(0)->get_nummer(),        +XOFF, +YOFF);
-				display_set_base_image_offset(liste->get_bild(1)->get_nummer(),        -XOFF, +YOFF);
-				display_set_base_image_offset(liste->get_bild(2)->get_nummer(),            0, +YOFF);
-				display_set_base_image_offset(liste->get_bild(3)->get_nummer(),        +XOFF,     0);
-				display_set_base_image_offset(liste->get_bild(4)->get_nummer(),        -XOFF, -YOFF);
-				display_set_base_image_offset(liste->get_bild(5)->get_nummer(),        +XOFF, -YOFF);
-				display_set_base_image_offset(liste->get_bild(6)->get_nummer(),            0, -YOFF);
-				display_set_base_image_offset(liste->get_bild(7)->get_nummer(), -XOFF - YOFF,     0);
-			}
-		}
-
 	}
 
 	return true;
@@ -287,7 +308,9 @@ bool vehikelbauer_t::alles_geladen()
 	while (typ_iter.next()) {
 		slist_tpl<const vehikel_besch_t*>& typ_liste = typ_iter.access_current_value();
 		uint count = typ_liste.get_count();
-		if (count == 0) continue;
+		if (count == 0) {
+			continue;
+		}
 		const vehikel_besch_t** const tmp     = new const vehikel_besch_t*[count];
 		const vehikel_besch_t** const tmp_end = tmp + count;
 		for (const vehikel_besch_t** i = tmp; i != tmp_end; i++) {
@@ -323,7 +346,7 @@ slist_tpl<const vehikel_besch_t*>* vehikelbauer_t::get_info(waytype_t typ)
  * tries to get best with but adds a little random action
  * @author prissi
  */
-const vehikel_besch_t *vehikelbauer_t::vehikel_search( waytype_t wt, const uint16 month_now, const uint32 target_weight, const uint32 target_speed, const ware_besch_t * target_freight, bool include_electric, bool not_obsolete )
+const vehikel_besch_t *vehikelbauer_t::vehikel_search( waytype_t wt, const uint16 month_now, const uint32 target_weight, const sint32 target_speed, const ware_besch_t * target_freight, bool include_electric, bool not_obsolete )
 {
 	const vehikel_besch_t *besch = NULL;
 	long besch_index=-100000;
@@ -342,10 +365,14 @@ const vehikel_besch_t *vehikelbauer_t::vehikel_search( waytype_t wt, const uint1
 			if(wt==track_wt  &&  !test_besch->can_follow_any()  ) {
 				continue;
 			}
+			// do not buy incomplete vehicles
+			if(wt==road_wt && !test_besch->can_lead(NULL)) {
+				continue;
+			}
 
 			// engine, but not allowed to lead a convoi, or no power at all or no electrics allowed
 			if(target_weight) {
-				if(test_besch->get_leistung()==0  ||  !test_besch->can_lead()  ||  (!include_electric  &&  test_besch->get_engine_type()==vehikel_besch_t::electric) ) {
+				if(test_besch->get_leistung()==0  ||  !test_besch->can_follow(NULL)  ||  (!include_electric  &&  test_besch->get_engine_type()==vehikel_besch_t::electric) ) {
 					continue;
 				}
 			}
@@ -371,7 +398,7 @@ const vehikel_besch_t *vehikelbauer_t::vehikel_search( waytype_t wt, const uint1
 				// assign this vehicle, if we have none found one yet, or we found only a too week one
 				if(  besch!=NULL  ) {
 					// it is cheaper to run? (this is most important)
-					difference += (besch->get_zuladung()*1000)/besch->get_betriebskosten() < (test_besch->get_zuladung()*1000)/test_besch->get_betriebskosten() ? -20 : 20;
+					difference += (besch->get_zuladung()*1000)/(1+besch->get_betriebskosten()) < (test_besch->get_zuladung()*1000)/(1+test_besch->get_betriebskosten()) ? -20 : 20;
 					if(  target_weight>0  ) {
 						// it is strongerer?
 						difference += (besch->get_leistung()*besch->get_gear())/64 < power ? -10 : 10;
@@ -395,15 +422,15 @@ const vehikel_besch_t *vehikelbauer_t::vehikel_search( waytype_t wt, const uint1
 
 			else {
 				// engine/tugboat/truck for trailer
-				if(  test_besch->get_zuladung()!=0  ||  !test_besch->can_lead()  ) {
+				if(  test_besch->get_zuladung()!=0  ||  !test_besch->can_follow(NULL)  ) {
 					continue;
 				}
 				// finally, we might be able to use this vehicle
-				uint32 speed = test_besch->get_geschw();
+				sint32 speed = test_besch->get_geschw();
 				uint32 max_weight = power/( (speed*speed)/2500 + 1 );
 
 				// we found a useful engine
-				long current_index = (power*100)/test_besch->get_betriebskosten() + test_besch->get_geschw() - (sint16)test_besch->get_gewicht() - (sint32)(test_besch->get_preis()/25000);
+				long current_index = (power*100)/(1+test_besch->get_betriebskosten()) + test_besch->get_geschw() - (sint16)test_besch->get_gewicht() - (sint32)(test_besch->get_preis()/25000);
 				// too slow?
 				if(speed < target_speed) {
 					current_index -= 250;
@@ -436,7 +463,7 @@ const vehikel_besch_t *vehikelbauer_t::vehikel_search( waytype_t wt, const uint1
  * if prev_besch==NULL, then the convoi must be able to lead a convoi
  * @author prissi
  */
-const vehikel_besch_t *vehikelbauer_t::get_best_matching( waytype_t wt, const uint16 month_now, const uint32 target_weight, const uint32 target_power, const uint32 target_speed, const ware_besch_t * target_freight, bool include_electric, bool not_obsolete, const vehikel_besch_t *prev_veh, bool is_last )
+const vehikel_besch_t *vehikelbauer_t::get_best_matching( waytype_t wt, const uint16 month_now, const uint32 target_weight, const uint32 target_power, const sint32 target_speed, const ware_besch_t * target_freight, bool not_obsolete, const vehikel_besch_t *prev_veh, bool is_last )
 {
 	const vehikel_besch_t *besch = NULL;
 	long besch_index=-100000;
@@ -470,6 +497,11 @@ const vehikel_besch_t *vehikelbauer_t::get_best_matching( waytype_t wt, const ui
 				continue;
 			}
 
+			// ignore vehicles that need electrification
+			if(test_besch->get_leistung()>0  &&  test_besch->get_engine_type()==vehikel_besch_t::electric) {
+				continue;
+			}
+
 			// likely tender => replace with some engine ...
 			if(target_freight==0  &&  target_weight==0) {
 				if(  test_besch->get_zuladung()!=0  ) {
@@ -493,7 +525,7 @@ const vehikel_besch_t *vehikelbauer_t::get_best_matching( waytype_t wt, const ui
 				// assign this vehicle, if we have none found one yet, or we found only a too week one
 				if(  besch!=NULL  ) {
 					// it is cheaper to run? (this is most important)
-					difference += (besch->get_zuladung()*1000)/besch->get_betriebskosten() < (test_besch->get_zuladung()*1000)/test_besch->get_betriebskosten() ? -20 : 20;
+					difference += (besch->get_zuladung()*1000)/(1+besch->get_betriebskosten()) < (test_besch->get_zuladung()*1000)/(1+test_besch->get_betriebskosten()) ? -20 : 20;
 					if(  target_weight>0  ) {
 						// it is strongerer?
 						difference += (besch->get_leistung()*besch->get_gear())/64 < power ? -10 : 10;
@@ -516,11 +548,11 @@ const vehikel_besch_t *vehikelbauer_t::get_best_matching( waytype_t wt, const ui
 			}
 			else {
 				// finally, we might be able to use this vehicle
-				uint32 speed = test_besch->get_geschw();
+				sint32 speed = test_besch->get_geschw();
 				uint32 max_weight = power/( (speed*speed)/2500 + 1 );
 
 				// we found a useful engine
-				long current_index = (power*100)/test_besch->get_betriebskosten() + test_besch->get_geschw() - (sint16)test_besch->get_gewicht() - (sint32)(test_besch->get_preis()/25000);
+				long current_index = (power*100)/(1+test_besch->get_betriebskosten()) + test_besch->get_geschw() - (sint16)test_besch->get_gewicht() - (sint32)(test_besch->get_preis()/25000);
 				// too slow?
 				if(speed < target_speed) {
 					current_index -= 250;
