@@ -6,6 +6,7 @@
 #include "../simdebug.h"
 
 #include "haus_besch.h"
+#include "../utils/checksum.h"
 
 
 
@@ -89,7 +90,7 @@ const haus_tile_besch_t *haus_besch_t::get_tile(int layout, int x, int y) const
 	koord dims = get_groesse(layout);
 
 	if(layout < 0  ||  x < 0  ||  y < 0  ||  layout >= layouts  ||  x >= get_b(layout)  ||  y >= get_h(layout)) {
-	dbg->fatal("hausbauer_t::get_tile()",
+	dbg->fatal("haus_tile_besch_t::get_tile()",
 			   "invalid request for l=%d, x=%d, y=%d on building %s (l=%d, x=%d, y=%d)",
 		   layout, x, y, get_name(), layouts, groesse.x, groesse.y);
 	}
@@ -107,6 +108,9 @@ const haus_tile_besch_t *haus_besch_t::get_tile(int layout, int x, int y) const
  */
 int haus_besch_t::layout_anpassen(int layout) const
 {
+	if(layout >= 4 && layouts <= 4) {
+		layout -= 4;
+	}
 	if(layout >= 2 && layouts <= 2) {
 		// Sind Layout C und D nicht definiert, nehemen wir ersatzweise A und B
 		layout -= 2;
@@ -119,3 +123,35 @@ int haus_besch_t::layout_anpassen(int layout) const
 	}
 	return layout;
 }
+
+
+void haus_besch_t::calc_checksum(checksum_t *chk) const
+{
+	chk->input((uint8)gtyp);
+	chk->input((uint8)utype);
+	chk->input(animation_time);
+	chk->input(extra_data);
+	chk->input(groesse.x);
+	chk->input(groesse.y);
+	chk->input((uint8)flags);
+	chk->input(level);
+	chk->input(layouts);
+	chk->input(enables);
+	chk->input(chance);
+	chk->input((uint8)allowed_climates);
+	chk->input(intro_date);
+	chk->input(obsolete_date);
+	// now check the layout
+	for(uint8 i=0; i<layouts; i++) {
+		sint16 b=get_b(i);
+		for(sint16 x=0; x<b; x++) {
+			sint16 h=get_h(i);
+			for(sint16 y=0; y<h; y++) {
+				if (get_tile(i,x,y)) {
+					chk->input((sint16)(x+y+i));
+				}
+			}
+		}
+	}
+
+}
