@@ -1,3 +1,4 @@
+#include <string>
 #include "../../dataobj/tabfile.h"
 #include "obj_node.h"
 #include "../grund_besch.h"
@@ -8,24 +9,22 @@
 
 void ground_writer_t::write_obj(FILE* fp, obj_node_t& parent, tabfileobj_t& obj)
 {
-	grund_besch_t besch;
-
 	obj_node_t node(this, 0, &parent);
 
 	write_head(fp, node, obj);
 
-	slist_tpl<slist_tpl<cstring_t> > keys;
+	slist_tpl<slist_tpl<std::string> > keys;
 	// summer images
 	for (int hangtyp = 0; hangtyp < 128; hangtyp++) {
-		keys.append(slist_tpl<cstring_t>());
+		keys.append(slist_tpl<std::string>());
 
 		for (int phase = 0; ; phase++) {
 			char buf[40];
 
 			sprintf(buf, "image[%d][%d]", hangtyp, phase);
 
-			cstring_t str = obj.get(buf);
-			if (str.len() == 0) {
+			std::string str = obj.get(buf);
+			if (str.size() == 0) {
 				break;
 			}
 			keys.at(hangtyp).append(str);
