@@ -5,6 +5,7 @@
 #include <stdio.h>
 
 #include "../simdebug.h"
+#include "../simgraph.h"
 #include "dingliste.h"
 #include "../bauer/hausbauer.h"
 #include "../dings/dummy.h"
@@ -56,21 +57,24 @@
 
 #define baum_pri (50)
 
+// priority of moving things: should be smaller than the priority of powerlines
+#define moving_obj_pri (100)
+
 // priority for entering into dingliste
 // unused entries have 255
-static uint8 type_to_pri[32]=
+static uint8 type_to_pri[256]=
 {
 	255, //
 	baum_pri, // baum
-	100, // zeiger
-	90, 90, 90,	// wolke
+	254, // zeiger
+	200, 200, 200,	// wolke
 	3, 3, // buildings
 	6, // signal
 	2, 2, // bridge/tunnel
 	255,
 	1, 1, 1, // depots
 	5, // smoke generator (not used any more)
-	75, 4, 4, // powerlines
+	150, 4, 4, // powerlines
 	6, // roadsign
 	6, // pillar
 	1, 1, 1, 1, // depots (must be before tunnel!)
@@ -80,7 +84,44 @@ static uint8 type_to_pri[32]=
 	3, // field (factory extension)
 	1, // crossings, treated like bridges or tunnels
 	1, // groundobjs, overlays over bare ground like lakes etc.
-	1  // narrowgaugedepot
+	1,  // narrowgaugedepot
+	255, 255, 255, 255, 255, 255, 255, 255,	// 32-63 left empty (old numbers)
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,
+	moving_obj_pri,	// pedestrians
+	moving_obj_pri,	// city cars
+	moving_obj_pri,	// road vehilce
+	moving_obj_pri,	// rail vehicle
+	moving_obj_pri,	// monorail
+	moving_obj_pri,	// maglev
+	moving_obj_pri,	// narrowgauge
+	255, 255, 255, 255, 255, 255, 255, 255, 255,
+	moving_obj_pri,	// ship
+	moving_obj_pri+1,	// aircraft (no trailer, could be handled by normal method)
+	moving_obj_pri+1,	// movingobject (no trailer, could be handled by normal method)
+	255, 255, 255, 255, 255, 255, 255, 255,	// 83-95 left empty (for other moving stuff) 95, is reserved for old choosesignals
+	255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,	// 96-128 left empty (old numbers)
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,	// 128-255 left empty
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255,
+	255, 255, 255, 255, 255, 255, 255, 255
 };
 
 
@@ -89,7 +130,8 @@ static void dl_free(ding_t** p, uint8 size)
 	assert(size > 1);
 	if (size <= 16) {
 		freelist_t::putback_node(sizeof(*p) * size, p);
-	} else {
+	}
+	else {
 		guarded_free(p);
 	}
 }
@@ -101,7 +143,8 @@ static ding_t** dl_alloc(uint8 size)
 	ding_t** p;
 	if (size <= 16) {
 		p = static_cast<ding_t**>(freelist_t::gimme_node(size * sizeof(*p)));
-	} else {
+	}
+	else {
 		p = MALLOCN(ding_t*, size);
 	}
 	return p;
@@ -110,9 +153,9 @@ static ding_t** dl_alloc(uint8 size)
 
 dingliste_t::dingliste_t()
 {
-    obj.one = NULL;
-    capacity = 0;
-    top = 0;
+	obj.one = NULL;
+	capacity = 0;
+	top = 0;
 }
 
 
@@ -126,8 +169,7 @@ dingliste_t::~dingliste_t()
 }
 
 
-void
-dingliste_t::set_capacity(uint8 new_cap)
+void dingliste_t::set_capacity(uint8 new_cap)
 {
 	// DBG_MESSAGE("dingliste_t::set_capacity()", "old cap=%d, new cap=%d", capacity, new_cap);
 
@@ -161,7 +203,7 @@ dingliste_t::set_capacity(uint8 new_cap)
 	else if(capacity<=1  &&  new_cap>1) {
 		ding_t *tmp=obj.one;
 		obj.some = dl_alloc(new_cap);
-		memset( obj.some, 0, sizeof(ding_t*)*new_cap );
+		MEMZERON(obj.some, new_cap);
 		obj.some[0] = tmp;
 		capacity = new_cap;
 		assert(top<=1);
@@ -208,11 +250,10 @@ bool dingliste_t::grow_capacity()
 
 
 
-void
-dingliste_t::shrink_capacity(uint8 o_top)
+void dingliste_t::shrink_capacity(uint8 o_top)
 {
-    // strategy: avoid free'ing mem if not neccesary. Only if we hold lots
-    // of memory then free it.
+	// strategy: avoid free'ing mem if not neccesary. Only if we hold lots
+	// of memory then free it.
 	if(capacity > 16 && o_top <= 4) {
 		set_capacity(o_top);
 	}
@@ -237,27 +278,33 @@ bool dingliste_t::intern_add_moving(ding_t* ding)
 {
 	// we are more than one object, thus we exclusively use obj.some here!
 	// it would be nice, if also the objects are inserted according to their priorities as
-	// vehicles types (number returned by gib_typ()). However, this would increase
+	// vehicles types (number returned by get_typ()). However, this would increase
 	// the calculation even further. :(
 
-	if(top==0  ||  obj.some[0]->gib_typ()==ding_t::baum) {
-		// airplane or movingobj
-		intern_insert_at( ding, ((vehikel_basis_t*)ding)->gib_waytype()==air_wt ? top : 0 );
-		return true;
-	}
-
 	// find out about the first car etc. moving thing.
-	// We can start to insert things after this index.
+	// We can start to insert between (start) and (end)
 	uint8 start=0;
-	while(start<top  &&   !obj.some[start]->is_moving()) {
+	while(start<top  &&  !obj.some[start]->is_moving()  ) {
 		start ++;
 	}
+/*	uint8 end=start;
+	while(end<top  && obj.some[end]->is_moving()) {
+		end ++;
+	}*/
+	uint8 end = top;
+	while(  end>start  &&  !obj.some[end-1]->is_moving()  ) {
+		end--;
+	}
+	if(start==end) {
+		intern_insert_at(ding, start);
+		return true;
+	}
 
 	// if we have two ways, the way at index 0 is ALWAYS the road!
 	// however ships and planes may be where not way is below ...
-	if(start!=0  &&  obj.some[0]->is_way()  &&  ((weg_t *)obj.some[0])->gib_waytype()==road_wt) {
+	if(start!=0  &&  obj.some[0]->is_way()  &&  ((weg_t *)obj.some[0])->get_waytype()==road_wt) {
 
-		const uint8 fahrtrichtung = ((vehikel_t*)ding)->gib_fahrtrichtung();
+		const uint8 fahrtrichtung = ((vehikel_basis_t*)ding)->get_fahrtrichtung();
 
 		// this is very complicated:
 		// we may have many objects in two lanes (actually five with tram and pedestrians)
@@ -273,14 +320,14 @@ bool dingliste_t::intern_add_moving(ding_t* ding)
 				}
 				else {
 					// we must be drawn before south or west (thus insert after)
-					for(uint8 i=start;  i<top;  i++  ) {
-						if((((const vehikel_t*)obj.some[i])->gib_fahrtrichtung()&ribi_t::suedwest)!=0) {
+					for(uint8 i=start;  i<end;  i++  ) {
+						if((((const vehikel_t*)obj.some[i])->get_fahrtrichtung()&ribi_t::suedwest)!=0) {
 							intern_insert_at(ding, i);
 							return true;
 						}
 					}
 					// nothing going southwest
-					obj.some[top++] = ding;
+					intern_insert_at(ding, end);
 					return true;
 				}
 
@@ -289,16 +336,19 @@ bool dingliste_t::intern_add_moving(ding_t* ding)
 				// going south, west or the rest
 				if((fahrtrichtung&(~ribi_t::suedost))==0) {
 					// if we are going south or southeast we must be drawn as the first in east direction (after nord and nordeast)
-					for(uint8 i=start;  i<top;  i++  ) {
-						const ding_t *dt = obj.some[i];
-						if(dt  &&  dt->is_moving()  &&  (((const vehikel_t*)dt)->gib_fahrtrichtung()&ribi_t::suedwest)!=0) {
-							intern_insert_at(ding, i);
-							return true;
+					for(uint8 i=start;  i<end;  i++  ) {
+						if (ding_t const* const dt = obj.some[i]) {
+							if (vehikel_basis_t const* const v = ding_cast<vehikel_basis_t>(dt)) {
+								if ((v->get_fahrtrichtung() & ribi_t::suedwest) != 0) {
+									intern_insert_at(ding, i);
+									return true;
+								}
+							}
 						}
 					}
 				}
 				// nothing going southeast
-				obj.some[top++] = ding;
+				intern_insert_at(ding, end);
 				return true;
 			}
 		}
@@ -309,8 +359,8 @@ bool dingliste_t::intern_add_moving(ding_t* ding)
 				if((fahrtrichtung&(~ribi_t::suedost))==0) {
 
 					// if we are going east we must be drawn as the first in east direction (after nord and nordeast)
-					for(uint8 i=start;  i<top;  i++  ) {
-						if( (((const vehikel_t*)obj.some[i])->gib_fahrtrichtung()&ribi_t::nordost)!=0) {
+					for(uint8 i=start;  i<end;  i++  ) {
+						if( (((const vehikel_t*)obj.some[i])->get_fahrtrichtung()&ribi_t::nordost)!=0) {
 							intern_insert_at(ding, i);
 							return true;
 						}
@@ -318,7 +368,7 @@ bool dingliste_t::intern_add_moving(ding_t* ding)
 					// nothing going to the east
 				}
 				// we must be drawn before south or west (thus append after)
-				obj.some[top++] = ding;
+				intern_insert_at(ding, end);
 				return true;
 
 			}
@@ -331,15 +381,15 @@ bool dingliste_t::intern_add_moving(ding_t* ding)
 					return true;
 				}
 				else {
-					for(uint8 i=start;  i<top;  i++  ) {
+					for(uint8 i=start;  i<end;  i++  ) {
 						// west or northwest: append after all westwards
-						if((((const vehikel_t*)obj.some[i])->gib_fahrtrichtung()&ribi_t::suedwest)==0) {
+						if((((const vehikel_t*)obj.some[i])->get_fahrtrichtung()&ribi_t::suedwest)==0) {
 							intern_insert_at(ding, i);
 							return true;
 						}
 					}
 					// nothing going to nordeast
-					obj.some[top++] = ding;
+					intern_insert_at(ding, end);
 					return true;
 				}
 			}
@@ -352,14 +402,14 @@ bool dingliste_t::intern_add_moving(ding_t* ding)
 		// but all vehicles are of the same typ, since this is track/channel etc. ONLY!
 
 		// => much simpler to handle
-		if((((vehikel_t*)ding)->gib_fahrtrichtung()&(~ribi_t::suedost))==0) {
+		if((((vehikel_t*)ding)->get_fahrtrichtung()&(~ribi_t::suedost))==0) {
 			// if we are going east or south, we must be drawn before (i.e. put first)
 			intern_insert_at(ding, start);
 			return true;
 		}
 		else {
 			// for north east we must be draw last
-			obj.some[top++] = ding;
+			intern_insert_at(ding, end);
 			return true;
 		}
 	}
@@ -382,28 +432,31 @@ bool dingliste_t::add(ding_t* ding)
 		// memory exceeded
 		return false;
 	}
-
-	// vehicles need a special order
-	if(ding->is_moving()) {
-		return intern_add_moving(ding);
+	if(top==0) {
+		intern_insert_at( ding, 0);
+		return true;
 	}
+	// now top>0
 
 	// now insert it a the correct place
-	uint8 pri=type_to_pri[ding->gib_typ()];
+	const uint8 pri=type_to_pri[ding->get_typ()];
+
+	// vehicles need a special order
+	if(pri==moving_obj_pri) {
+		return intern_add_moving(ding);
+	}
 
 	// roads must be first!
 	if(pri==0) {
 		// check for other ways to keep order! (maximum is two ways per tile at the moment)
-		if(top>0  &&  obj.some[0]->gib_typ()==ding_t::way  &&  ((weg_t *)ding)->gib_waytype()>((weg_t *)obj.some[0])->gib_waytype()) {
-			intern_insert_at(ding, 1);
-		} else {
-			intern_insert_at(ding, 0);
-		}
+		weg_t const* const w   = ding_cast<weg_t>(obj.some[0]);
+		uint8        const pos = w  &&  w->get_waytype() < static_cast<weg_t*>(ding)->get_waytype() ? 1 : 0;
+		intern_insert_at(ding, pos);
 		return true;
 	}
 
 	uint8 i;
-	for(  i=0;  i<top  &&  pri>type_to_pri[obj.some[i]->gib_typ()];  i++  )
+	for(  i=0;  i<top  &&  pri>type_to_pri[obj.some[i]->get_typ()];  i++  )
 		;
 	// now i contains the position, where we either insert of just add ...
 	if(i==top) {
@@ -415,10 +468,11 @@ bool dingliste_t::add(ding_t* ding)
 			/* trees are a little tricky, since they cast a shadow
 			 * therefore the y-order must be correct!
 			 */
-			const sint8 offset = ding->gib_yoff() + ding->gib_xoff();
+			const sint8 offset = ding->get_yoff() + ding->get_xoff();
 
 			for(  ;  i<top;  i++) {
-				if(obj.some[i]->gib_typ()!=ding_t::baum  ||  obj.some[i]->gib_yoff()+obj.some[i]->gib_xoff()>offset) {
+				baum_t const* const tree = ding_cast<baum_t>(obj.some[i]);
+				if (!tree || tree->get_yoff() + tree->get_xoff() > offset) {
 					break;
 				}
 			}
@@ -437,8 +491,7 @@ bool dingliste_t::add(ding_t* ding)
 // take the thing out from the list
 // use this only for temperary removing
 // since it does not shrink list or checks for ownership
-ding_t *
-dingliste_t::remove_last()
+ding_t *dingliste_t::remove_last()
 {
 	ding_t *d=NULL;
 	if(capacity==0) {
@@ -494,9 +547,29 @@ bool dingliste_t::remove(const ding_t* ding)
 }
 
 
+/**
+ * removes object from map
+ * deletes object if it is not a zeiger_t
+ */
+void local_delete_object(ding_t *ding, spieler_t *sp)
+{
+	vehikel_basis_t* const v = ding_cast<vehikel_basis_t>(ding);
+	if (v  &&  ding->get_typ() != ding_t::fussgaenger  &&  ding->get_typ() != ding_t::verkehr  &&  ding->get_typ() != ding_t::movingobj) {
+		v->verlasse_feld();
+	}
+	else {
+		ding->entferne(sp);
+		ding->set_flag(ding_t::not_on_map);
+		// all objects except zeiger (pointer) are destroyed here
+		// zeiger's will be deleted if their associated werkzeug_t (tool) terminates
+		if (ding->get_typ() != ding_t::zeiger) {
+			delete ding;
+		}
+	}
+}
 
-bool
-dingliste_t::loesche_alle(spieler_t *sp, uint8 offset)
+
+bool dingliste_t::loesche_alle(spieler_t *sp, uint8 offset)
 {
 	if(top<=offset) {
 		return false;
@@ -508,31 +581,14 @@ dingliste_t::loesche_alle(spieler_t *sp, uint8 offset)
 	if(capacity>1) {
 		while(  top>offset  ) {
 			top --;
-			ding_t *dt = obj.some[top];
-			if(dt->is_moving()  &&  !(dt->gib_typ()==ding_t::fussgaenger  ||  dt->gib_typ()==ding_t::verkehr  ||  dt->gib_typ()==ding_t::movingobj)) {
-				((vehikel_t *)dt)->verlasse_feld();
-				assert(0);
-			}
-			else {
-				dt->entferne(sp);
-				dt->set_flag(ding_t::not_on_map);
-				delete dt;
-			}
+			local_delete_object(obj.some[top], sp);
 			obj.some[top] = NULL;
 			ok = true;
 		}
 	}
 	else {
 		if(capacity==1) {
-			ding_t *dt = obj.one;
-			if(dt->is_moving()) {
-				((vehikel_t *)dt)->verlasse_feld();
-			}
-			else {
-				dt->entferne(sp);
-				dt->set_flag(ding_t::not_on_map);
-				delete dt;
-			}
+			local_delete_object(obj.one, sp);
 			ok = true;
 			obj.one = NULL;
 			capacity = top = 0;
@@ -546,8 +602,7 @@ dingliste_t::loesche_alle(spieler_t *sp, uint8 offset)
 
 
 /* returns the text of an error message, if obj could not be removed */
-const char *
-dingliste_t::kann_alle_entfernen(const spieler_t *sp, uint8 offset) const
+const char *dingliste_t::kann_alle_entfernen(const spieler_t *sp, uint8 offset) const
 {
 	if(top<=offset) {
 		return NULL;
@@ -573,8 +628,7 @@ dingliste_t::kann_alle_entfernen(const spieler_t *sp, uint8 offset) const
 
 /* recalculates all images
  */
-void
-dingliste_t::calc_bild()
+void dingliste_t::calc_bild()
 {
 	if(capacity==0) {
 		// nothing
@@ -609,20 +663,19 @@ bool dingliste_t::ist_da(const ding_t* ding) const
 
 
 
-ding_t *
-dingliste_t::suche(ding_t::typ typ,uint8 start) const
+ding_t *dingliste_t::suche(ding_t::typ typ,uint8 start) const
 {
 	if(capacity==0) {
 		return NULL;
 	}
 	else if(capacity==1) {
-		return obj.one->gib_typ()!=typ ? NULL : obj.one;
+		return obj.one->get_typ()!=typ ? NULL : obj.one;
 	}
 	else if(start<top) {
 		// else we have to search the list
 		for(uint8 i=start; i<top; i++) {
 			ding_t * tmp = obj.some[i];
-			if(tmp->gib_typ()==typ) {
+			if(tmp->get_typ()==typ) {
 				return tmp;
 			}
 		}
@@ -632,21 +685,20 @@ dingliste_t::suche(ding_t::typ typ,uint8 start) const
 
 
 
-ding_t *
-dingliste_t::get_leitung() const
+ding_t *dingliste_t::get_leitung() const
 {
 	if(capacity==0) {
 		return NULL;
 	}
 	else if(capacity==1) {
-		if(obj.one->gib_typ()>=ding_t::leitung  &&  obj.one->gib_typ()<=ding_t::senke) {
+		if(obj.one->get_typ()>=ding_t::leitung  &&  obj.one->get_typ()<=ding_t::senke) {
 			return obj.one;
 		}
 	}
 	else if(top>0) {
 		// else we have to search the list
 		for(uint8 i=0; i<top; i++) {
-			uint8 typ = obj.some[i]->gib_typ();
+			uint8 typ = obj.some[i]->get_typ();
 			if(typ>=ding_t::leitung  &&  typ<=ding_t::senke) {
 				return obj.some[i];
 			}
@@ -657,14 +709,39 @@ dingliste_t::get_leitung() const
 
 
 
-void
-dingliste_t::rdwr(karte_t *welt, loadsave_t *file, koord3d current_pos)
+ding_t *dingliste_t::get_convoi_vehicle() const
+{
+	if(capacity==0) {
+		return NULL;
+	}
+	else if(capacity==1) {
+		// could
+		uint8 t = obj.one->get_typ();
+		if(  t==ding_t::aircraft  ||  t==ding_t::schiff  ) {
+			return obj.one;
+		}
+	}
+	else if(top>0) {
+		// else we have to search the list
+		for(uint8 i=0; i<top; i++) {
+			uint8 typ = obj.some[i]->get_typ();
+			if(  typ>=ding_t::automobil  &&  typ<=ding_t::aircraft  ) {
+				return obj.some[i];
+			}
+		}
+	}
+	return NULL;
+}
+
+
+
+void dingliste_t::rdwr(karte_t *welt, loadsave_t *file, koord3d current_pos)
 {
 	sint32 max_object_index;
 	if(file->is_saving()) {
 		max_object_index = top-1;
 	}
-	file->rdwr_long(max_object_index, "\n");
+	file->rdwr_long(max_object_index);
 
 	if(max_object_index>254) {
 		dbg->error("dingliste_t::laden()","Too many objects (%i) at (%i,%i), some vehicle may not appear immediately.",max_object_index,current_pos.x,current_pos.y);
@@ -695,12 +772,13 @@ dingliste_t::rdwr(karte_t *welt, loadsave_t *file, koord3d current_pos)
 				case ding_t::wayobj:
 				{
 					wayobj_t* const wo = new wayobj_t(welt, file);
-					if (wo->gib_besch() == NULL) {
+					if (wo->get_besch() == NULL) {
 						// ignore missing wayobjs
 						wo->set_flag(ding_t::not_on_map);
 						delete wo;
 						d = NULL;
-					} else {
+					}
+					else {
 						d = wo;
 					}
 					break;
@@ -712,12 +790,13 @@ dingliste_t::rdwr(karte_t *welt, loadsave_t *file, koord3d current_pos)
 				case ding_t::fussgaenger:
 				{
 					fussgaenger_t* const pedestrian = new fussgaenger_t(welt, file);
-					if (pedestrian->gib_besch() == NULL) {
+					if (pedestrian->get_besch() == NULL) {
 						// no pedestrians ... delete this
 						pedestrian->set_flag(ding_t::not_on_map);
 						delete pedestrian;
 						d = NULL;
-					} else {
+					}
+					else {
 						d = pedestrian;
 					}
 					break;
@@ -728,12 +807,12 @@ dingliste_t::rdwr(karte_t *welt, loadsave_t *file, koord3d current_pos)
 				case ding_t::verkehr:
 				{
 					stadtauto_t* const car = new stadtauto_t(welt, file);
-					if (car->gib_besch() == NULL) {
+					if (car->get_besch() == NULL) {
 						// no citycars ... delete this
 						car->set_flag(ding_t::not_on_map);
 						delete car;
-						d = NULL;
-					} else {
+					}
+					else {
 						d = car;
 					}
 					break;
@@ -770,29 +849,24 @@ dingliste_t::rdwr(karte_t *welt, loadsave_t *file, koord3d current_pos)
 				case ding_t::bahndepot:
 				{
 					// for compatibilty reasons we may have to convert them to tram and monorail depots
-					gebaeude_t *gb = new gebaeude_t(welt, file);
-					if(gb->gib_tile()->gib_besch()->gib_extra()==monorail_wt) {
-						monoraildepot_t *md = new monoraildepot_t(welt,gb->gib_pos(),(spieler_t *)NULL,gb->gib_tile());
-						md->rdwr_vehicles(file);
-						d = md;
-					}
-					else if(gb->gib_tile()->gib_besch()->gib_extra()==tram_wt) {
-						tramdepot_t *td = new tramdepot_t(welt,gb->gib_pos(),(spieler_t *)NULL,gb->gib_tile());
-						td->rdwr_vehicles(file);
-						d = td;
+					bahndepot_t*                   bd;
+					gebaeude_t                     gb(welt, file);
+					haus_tile_besch_t const* const tile = gb.get_tile();
+					if(  tile  ) {
+						switch (tile->get_besch()->get_extra()) {
+							case monorail_wt: bd = new monoraildepot_t(welt, gb.get_pos(), gb.get_besitzer(), tile); break;
+							case tram_wt:     bd = new tramdepot_t(    welt, gb.get_pos(), gb.get_besitzer(), tile); break;
+							default:          bd = new bahndepot_t(    welt, gb.get_pos(), gb.get_besitzer(), tile); break;
+						}
 					}
 					else {
-						bahndepot_t *bd = new bahndepot_t(welt,gb->gib_pos(),(spieler_t *)NULL,gb->gib_tile());
-						bd->rdwr_vehicles(file);
-						d = bd;
+						bd = new bahndepot_t( welt, gb.get_pos(), gb.get_besitzer(), NULL );
 					}
-					d->setze_besitzer( gb->gib_besitzer() );
-					spieler_t::add_maintenance( gb->gib_besitzer(), welt->gib_einstellungen()->maint_building );
-					typ = d->gib_typ();
-
+					bd->rdwr_vehicles(file);
+					d   = bd;
+					typ = d->get_typ();
 					// do not remove from this position, since there will be nothing
-					gb->set_flag(ding_t::not_on_map);
-					delete gb;
+					gb.set_flag(ding_t::not_on_map);
 				}
 				break;
 
@@ -802,7 +876,7 @@ dingliste_t::rdwr(karte_t *welt, loadsave_t *file, koord3d current_pos)
 				case ding_t::pillar:
 				{
 					pillar_t *p = new pillar_t(welt, file);
-					if(p->gib_besch()!=NULL  &&  p->gib_besch()->gib_pillar()!=0) {
+					if(p->get_besch()!=NULL  &&  p->get_besch()->get_pillar()!=0) {
 						d = p;
 					}
 					else {
@@ -817,25 +891,28 @@ dingliste_t::rdwr(karte_t *welt, loadsave_t *file, koord3d current_pos)
 				case ding_t::baum:
 				{
 					baum_t *b = new baum_t(welt, file);
-					if(!b->gib_besch()) {
+					if(!b->get_besch()) {
 						// do not remove from this position, since there will be nothing
 						b->set_flag(ding_t::not_on_map);
 						delete b;
+						b = NULL;
+					}
+					else {
+						d = b;
 					}
-					d = b;
 				}
 				break;
 
 				case ding_t::groundobj:
 				{
 					groundobj_t* const groundobj = new groundobj_t(welt, file);
-					if (groundobj->gib_besch() == NULL) {
+					if(groundobj->get_besch() == NULL) {
 						// do not remove from this position, since there will be nothing
 						groundobj->set_flag(ding_t::not_on_map);
 						// not use entferne, since it would try to lookup besch
 						delete groundobj;
-						d = NULL;
-					} else {
+					}
+					else {
 						d = groundobj;
 					}
 					break;
@@ -844,12 +921,12 @@ dingliste_t::rdwr(karte_t *welt, loadsave_t *file, koord3d current_pos)
 				case ding_t::movingobj:
 				{
 					movingobj_t* const movingobj = new movingobj_t(welt, file);
-					if (movingobj->gib_besch() == NULL) {
+					if (movingobj->get_besch() == NULL) {
 						// no citycars ... delete this
 						movingobj->set_flag(ding_t::not_on_map);
 						delete movingobj;
-						d = NULL;
-					} else {
+					}
+					else {
 						d = movingobj;
 					}
 					break;
@@ -857,17 +934,16 @@ dingliste_t::rdwr(karte_t *welt, loadsave_t *file, koord3d current_pos)
 
 				case ding_t::gebaeude:
 				{
-					// Wenn es ein Gebude nicht mehr gibt, genderte Tabfiles
-					// lassen wir es einfach weg. Das gibt zwar Fundamente ohne
-					// Aufbau, strzt aber nicht ab.
 					gebaeude_t *gb = new gebaeude_t (welt, file);
-					if(gb->gib_tile()==NULL) {
+					if(gb->get_tile()==NULL) {
 						// do not remove from this position, since there will be nothing
 						gb->set_flag(ding_t::not_on_map);
 						delete gb;
-						gb  = 0;
+						gb = NULL;
+					}
+					else {
+						d = gb;
 					}
-					d = gb;
 				}
 				break;
 
@@ -876,7 +952,7 @@ dingliste_t::rdwr(karte_t *welt, loadsave_t *file, koord3d current_pos)
 				case ding_t::roadsign:
 				{
 					roadsign_t *rs = new roadsign_t (welt, file);
-					if(rs->gib_besch()==NULL) {
+					if(rs->get_besch()==NULL) {
 						// roadsign_t without description => ignore
 						rs->set_flag(ding_t::not_on_map);
 						delete rs;
@@ -901,17 +977,17 @@ dingliste_t::rdwr(karte_t *welt, loadsave_t *file, koord3d current_pos)
 					dbg->fatal("dingliste_t::laden()", "During loading: Unknown object type '%d'", typ);
 			}
 
-			if(d  &&  d->gib_typ()!=typ) {
-				dbg->warning( "dingliste_t::rdwr()","typ error : %i instead %i on %i,%i, object ignored!", d->gib_typ(), typ, d->gib_pos().x, d->gib_pos().y );
+			if(d  &&  d->get_typ()!=typ) {
+				dbg->warning( "dingliste_t::rdwr()","typ error : %i instead %i on %i,%i, object ignored!", d->get_typ(), typ, d->get_pos().x, d->get_pos().y );
 				d = NULL;
 			}
 
-			if(d  &&  d->gib_pos()==koord3d::invalid) {
-				d->setze_pos( current_pos );
+			if(d  &&  d->get_pos()==koord3d::invalid) {
+				d->set_pos( current_pos );
 			}
 
-			if(d  &&  d->gib_pos()!=current_pos) {
-				dbg->warning("dingliste_t::rdwr()","position error: %i,%i,%i instead %i,%i,%i (object will be ignored)",d->gib_pos().x,d->gib_pos().y,d->gib_pos().z,current_pos.x,current_pos.y,current_pos.z);
+			if(d  &&  d->get_pos()!=current_pos) {
+				dbg->warning("dingliste_t::rdwr()","position error: %i,%i,%i instead %i,%i,%i (object will be ignored)",d->get_pos().x,d->get_pos().y,d->get_pos().z,current_pos.x,current_pos.y,current_pos.z);
 				d = NULL;
 			}
 
@@ -924,34 +1000,34 @@ dingliste_t::rdwr(karte_t *welt, loadsave_t *file, koord3d current_pos)
 			ding_t *d=bei(i);
 			if(d->is_way()
 				// do not save smoke
-				||  d->gib_typ()==ding_t::raucher
-				||  d->gib_typ()==ding_t::sync_wolke
-				||  d->gib_typ()==ding_t::async_wolke
+				||  d->get_typ()==ding_t::raucher
+				||  d->get_typ()==ding_t::sync_wolke
+				||  d->get_typ()==ding_t::async_wolke
 				// fields will be built by factory
-				||  d->gib_typ()==ding_t::field
+				||  d->get_typ()==ding_t::field
 				// do not save factory buildings => factory will reconstruct them
-				||  (d->gib_typ()==ding_t::gebaeude  &&  ((gebaeude_t *)d)->get_fabrik())
+				||  (d->get_typ()==ding_t::gebaeude  &&  ((gebaeude_t *)d)->get_fabrik())
 				// things with convoi will not be saved
-				||  (d->gib_typ()>=66  &&  d->gib_typ()<82)
+				||  (d->get_typ()>=66  &&  d->get_typ()<82)
 			) {
 				// these objects are simply not saved
 				file->wr_obj_id(-1);
 			}
 			else {
 				// on old versions
-				if(d->gib_pos()==current_pos) {
-					file->wr_obj_id(d->gib_typ());
-					bei(i)->rdwr(file);
+				if(d->get_pos()==current_pos) {
+					file->wr_obj_id(d->get_typ());
+					d->rdwr(file);
 				}
-				else if(bei(i)->gib_pos().gib_2d()==current_pos.gib_2d()) {
+				else if (d->get_pos().get_2d() == current_pos.get_2d()) {
 					// ok, just error in z direction => we will correct it
-					dbg->warning( "dingliste_t::rdwr()","position error: z pos corrected on %i,%i from %i to %i",bei(i)->gib_pos().x,bei(i)->gib_pos().y,bei(i)->gib_pos().z,current_pos.z);
-					file->wr_obj_id(d->gib_typ());
-					bei(i)->setze_pos( current_pos );
-					bei(i)->rdwr(file);
+					dbg->warning( "dingliste_t::rdwr()","position error: z pos corrected on %i,%i from %i to %i", d->get_pos().x, d->get_pos().y, d->get_pos().z, current_pos.z);
+					file->wr_obj_id(d->get_typ());
+					d->set_pos(current_pos);
+					d->rdwr(file);
 				}
 				else {
-					dbg->error( "dingliste_t::rdwr()","unresolvable position error: %i,%i instead %i,%i (object type %i will be not saved!)", bei(i)->gib_pos().x, bei(i)->gib_pos().y, current_pos.x, current_pos.y, bei(i)->gib_typ() );
+					dbg->error("dingliste_t::rdwr()","unresolvable position error: %i,%i instead %i,%i (object type %i will be not saved!)", d->get_pos().x, d->get_pos().y, current_pos.x, current_pos.y, d->get_typ());
 					file->wr_obj_id(-1);
 				}
 			}
@@ -971,23 +1047,125 @@ void dingliste_t::dump() const
 		return;
 	}
 	else if(capacity==1) {
-		DBG_MESSAGE("dingliste_t::dump()","one object \'%s\' owned by sp %p", obj.one->gib_name(), obj.one->gib_besitzer() );
+		DBG_MESSAGE("dingliste_t::dump()","one object \'%s\' owned by sp %p", obj.one->get_name(), obj.one->get_besitzer() );
 		return;
 	}
 
 	DBG_MESSAGE("dingliste_t::dump()","%i objects", top );
 	for(uint8 n=0; n<top; n++) {
-		DBG_MESSAGE( obj.some[n]->gib_name(), "at %i owned by sp %p", n, obj.some[n]->gib_besitzer() );
+		DBG_MESSAGE( obj.some[n]->get_name(), "at %i owned by sp %p", n, obj.some[n]->get_besitzer() );
 	}
 }
 
 
-
-/* display all things, much faster to do it here ...
- * reset_dirty will be only true for the main display; all miniworld windows should still reset main window ...
- *  @author prissi
+/**
+ * Routine to display background images of non-moving things
+ * powerlines have to be drawn after vehicles (and thus are in the obj-array inserted after vehicles)
+ * @param reset_dirty will be only true for the main display; all miniworld windows should still reset main window
+ * @return the index of the first moving thing (or powerline)
+ *
+ * dingliste_t::display_dinge_bg() .. called by the methods in grund_t
+ * local_display_dinge_bg()        .. local function to avoid code duplication, returns false if the first non-valid obj is reached
+ * @author Dwachs
  */
-void dingliste_t::display_dinge( const sint16 xpos, const sint16 ypos, const uint8 start_offset, const bool reset_dirty ) const
+inline bool local_display_dinge_bg(const ding_t *ding, const sint16 xpos, const sint16 ypos, const bool reset_dirty )
+{
+	const bool display_ding = !ding->is_moving();
+	if (display_ding) {
+		ding->display(xpos, ypos, reset_dirty );
+	}
+	return display_ding;
+}
+
+
+uint8 dingliste_t::display_dinge_bg( const sint16 xpos, const sint16 ypos, const uint8 start_offset, const bool reset_dirty ) const
+{
+	if(start_offset>=top) {
+		return start_offset;
+	}
+
+	if(capacity==1) {
+		if(local_display_dinge_bg(obj.one, xpos, ypos, reset_dirty)) {
+			return 1;
+		}
+		return 0;
+	}
+
+	for(uint8 n=start_offset; n<top; n++) {
+		if (!local_display_dinge_bg(obj.some[n], xpos, ypos, reset_dirty)) {
+			return n;
+		}
+	}
+	return top;
+}
+
+
+/**
+ * Routine to draw vehicles
+ * .. vehicles are draws if driving in direction ribi (with special treatment of flying aircrafts)
+ * .. clips vehicle only along relevant edges (depends on ribi and vehicle direction)
+ * @param ontile if true then vehicles are on the tile that defines the clipping
+ * @param reset_dirty will be only true for the main display; all miniworld windows should still reset main window
+ * @return the index of the first non-moving thing
+ *
+ * dingliste_t::display_dinge_vh() .. called by the methods in grund_t
+ * local_display_dinge_vh()        .. local function to avoid code duplication, returns false if the first non-valid obj is reached
+ * @author Dwachs
+ */
+inline bool local_display_dinge_vh(const ding_t *ding, const sint16 xpos, const sint16 ypos, const bool reset_dirty, const ribi_t::ribi ribi, const bool ontile)
+{
+	vehikel_basis_t const* const v = ding_cast<vehikel_basis_t>(ding);
+	aircraft_t      const*       a;
+	if (v && (ontile || !(a = ding_cast<aircraft_t>(v)) || a->is_on_ground())) {
+		const ribi_t::ribi veh_ribi = v->get_fahrtrichtung();
+		if (ontile || (veh_ribi & ribi)==ribi  ||  (ribi_t::rueckwaerts(veh_ribi) & ribi)==ribi  ||  ding->get_typ()==ding_t::aircraft) {
+			activate_ribi_clip((veh_ribi|ribi_t::rueckwaerts(veh_ribi))&ribi);
+			ding->display(xpos, ypos, reset_dirty );
+		}
+		return true;
+	}
+	else {
+		// if !ontile starting_offset is not correct, hence continue searching till the end
+		return !ontile;
+	}
+}
+
+
+uint8 dingliste_t::display_dinge_vh( const sint16 xpos, const sint16 ypos, const uint8 start_offset, const bool reset_dirty, const ribi_t::ribi ribi, const bool ontile ) const
+{
+	if(start_offset>=top) {
+		return start_offset;
+	}
+
+	if(capacity==1) {
+		if(local_display_dinge_vh(obj.one, xpos, ypos, reset_dirty, ribi, ontile)) {
+			return 1;
+		}
+		else {
+			return 0;
+		}
+	}
+
+	uint8 nr_v = start_offset;
+	for(uint8 n=start_offset; n<top; n++) {
+		if (local_display_dinge_vh(obj.some[n], xpos, ypos, reset_dirty, ribi, ontile)) {
+			nr_v = n;
+		}
+		else {
+			break;
+		}
+	}
+	activate_ribi_clip();
+	return nr_v+1;
+}
+
+
+/**
+ * Routine to draw foreground images of everything on the tile (no clipping) and powerlines
+ * @param start_offset .. draws also background images of all objects with index>=start_offset
+ * @param reset_dirty will be only true for the main display; all miniworld windows should still reset main window
+ */
+void dingliste_t::display_dinge_fg( const sint16 xpos, const sint16 ypos, const uint8 start_offset, const bool reset_dirty ) const
 {
 	if(capacity==0) {
 		return;
@@ -995,10 +1173,10 @@ void dingliste_t::display_dinge( const sint16 xpos, const sint16 ypos, const uin
 	else if(capacity==1) {
 		if(start_offset==0) {
 			obj.one->display(xpos, ypos, reset_dirty );
-			obj.one->display_after(xpos, ypos, reset_dirty );
-			if(reset_dirty) {
-				obj.one->clear_flag(ding_t::dirty);
-			}
+		}
+		obj.one->display_after(xpos, ypos, reset_dirty );
+		if(reset_dirty) {
+			obj.one->clear_flag(ding_t::dirty);
 		}
 		return;
 	}
@@ -1010,15 +1188,16 @@ void dingliste_t::display_dinge( const sint16 xpos, const sint16 ypos, const uin
 	// foreground (needs to be done backwards!
 	for(int n=top-1; n>=0;  n--) {
 		obj.some[n]->display_after(xpos, ypos, reset_dirty );
-		obj.some[n]->clear_flag(ding_t::dirty);
+		if(reset_dirty) {
+			obj.some[n]->clear_flag(ding_t::dirty);
+		}
 	}
+	return;
 }
 
 
-
 // start next month (good for toogling a seasons)
-void
-dingliste_t::check_season(const long month)
+void dingliste_t::check_season(const long month)
 {
 	slist_tpl<ding_t *>loeschen;
 
