@@ -9,6 +9,7 @@
 #include "../simhalt.h"
 #include "../simimg.h"
 
+#include "../utils/cbuffer_t.h"
 #include "../gui/messagebox.h"
 #include "../bauer/hausbauer.h"
 #include "../besch/haus_besch.h"
@@ -25,18 +26,10 @@
 
 struct linieneintrag_t schedule_t::dummy_eintrag = { koord3d::invalid, 0, 0 };
 
-void schedule_t::init()
+
+
+schedule_t::schedule_t(loadsave_t* const file)
 {
-	aktuell = 0;
-	abgeschlossen = false;
-	type = schedule_t::fahrplan;
-}
-
-
-
-schedule_t::schedule_t(loadsave_t *file)
-{
-	type = schedule_t::fahrplan;
 	rdwr(file);
 	if(file->is_loading()) {
 		cleanup();
@@ -57,9 +50,8 @@ void schedule_t::copy_from(const schedule_t *src)
 	for(  uint8 i=0;  i<src->eintrag.get_count();  i++  ) {
 		eintrag.append(src->eintrag[i]);
 	}
-	if(  aktuell>=eintrag.get_count()  ) {
-		aktuell = max(1,eintrag.get_count())-1;
-	}
+	set_aktuell( src->get_aktuell() );
+
 	abgeschlossen = src->ist_abgeschlossen();
 }
 
@@ -68,6 +60,7 @@ void schedule_t::copy_from(const schedule_t *src)
 bool schedule_t::ist_halt_erlaubt(const grund_t *gr) const
 {
 	// first: check, if we can go here
+	waytype_t const my_waytype = get_waytype();
 	bool ok = gr->hat_weg(my_waytype);
 	if(  !ok  ) {
 		if(  my_waytype==air_wt  ) {
@@ -183,42 +176,34 @@ void schedule_t::cleanup()
 			lastpos = eintrag[i].pos;
 		}
 	}
-	if(  aktuell>=eintrag.get_count()  ) {
-		aktuell = max(1,eintrag.get_count())-1;
-	}
+	make_aktuell_valid();
 }
 
 
 
-bool
-schedule_t::remove()
+bool schedule_t::remove()
 {
 	bool ok = eintrag.remove_at(aktuell);
-	if(  aktuell>=eintrag.get_count()  ) {
-		aktuell = max(1,eintrag.get_count())-1;
-	}
+	make_aktuell_valid();
 	return ok;
 }
 
 
 
-void
-schedule_t::rdwr(loadsave_t *file)
+void schedule_t::rdwr(loadsave_t *file)
 {
 	xml_tag_t f( file, "fahrplan_t" );
 
-	if(  aktuell>=eintrag.get_count()  ) {
-		aktuell = max(1,eintrag.get_count())-1;
-	}
+	make_aktuell_valid();
 
 	uint8 size = eintrag.get_count();
 	if(  file->get_version()<=101000  ) {
 		uint32 dummy=aktuell;
-		file->rdwr_long(dummy, " ");
+		file->rdwr_long(dummy);
 		aktuell = (uint8)dummy;
 
 		sint32 maxi=size;
-		file->rdwr_long(maxi, " ");
+		file->rdwr_long(maxi);
 		DBG_MESSAGE("fahrplan_t::rdwr()","read schedule %p with %i entries",this,maxi);
 		if(file->get_version()<86010) {
 			// old array had different maxi-counter
@@ -227,8 +212,8 @@ schedule_t::rdwr(loadsave_t *file)
 		size = (uint8)max(0,maxi);
 	}
 	else {
-		file->rdwr_byte(aktuell, " ");
-		file->rdwr_byte(size, " ");
+		file->rdwr_byte(aktuell);
+		file->rdwr_byte(size);
 	}
 	eintrag.resize(size);
 
@@ -237,7 +222,7 @@ schedule_t::rdwr(loadsave_t *file)
 			koord3d pos;
 			uint32 dummy;
 			pos.rdwr(file);
-			file->rdwr_long(dummy, "\n");
+			file->rdwr_long(dummy);
 
 			struct linieneintrag_t stop;
 			stop.pos = pos;
@@ -254,17 +239,19 @@ schedule_t::rdwr(loadsave_t *file)
 				eintrag[i] .waiting_time_shift = 0;
 			}
 			eintrag[i].pos.rdwr(file);
-			file->rdwr_byte(eintrag[i].ladegrad, "\n");
+			file->rdwr_byte(eintrag[i].ladegrad);
 			if(file->get_version()>=99018) {
-				file->rdwr_byte( eintrag[i].waiting_time_shift, "w" );
+				file->rdwr_byte(eintrag[i].waiting_time_shift);
 			}
 		}
 	}
 	if(file->is_loading()) {
 		abgeschlossen = true;
 	}
-	if(aktuell>=eintrag.get_count()) {
-		dbg->error("fahrplan_t::rdwr()","aktuell %i >count %i => aktuell = 0", aktuell, eintrag.get_count() );
+	if(aktuell>=eintrag.get_count()  ) {
+		if(  eintrag.get_count()>0  ) {
+			dbg->error("fahrplan_t::rdwr()","aktuell %i >count %i => aktuell = 0", aktuell, eintrag.get_count() );
+		}
 		aktuell = 0;
 	}
 }
@@ -273,7 +260,7 @@ schedule_t::rdwr(loadsave_t *file)
 
 void schedule_t::rotate90( sint16 y_size )
 {
- 	// now we have to rotate all entries ...
+	// now we have to rotate all entries ...
 	for(  uint8 i = 0;  i<eintrag.get_count();  i++  ) {
 		eintrag[i].pos.rotate90(y_size);
 	}
@@ -285,8 +272,7 @@ void schedule_t::rotate90( sint16 y_size )
  * compare this fahrplan with another, passed in fahrplan
  * @author hsiegeln
  */
-bool
-schedule_t::matches(karte_t *welt, const schedule_t *fpl)
+bool schedule_t::matches(karte_t *welt, const schedule_t *fpl)
 {
 	if(fpl == NULL) {
 		return false;
@@ -301,7 +287,7 @@ schedule_t::matches(karte_t *welt, const schedule_t *fpl)
 		return false;
 	}
 	// now we have to check all entries ...
-	// we need to do this that complicated, because they last stop may make the difference
+	// we need to do this that complicated, because the last stop may make the difference
 	uint16 f1=0, f2=0;
 	while(  f1+f2<eintrag.get_count()+fpl->eintrag.get_count()  ) {
 		if(f1<eintrag.get_count()  &&  f2<fpl->eintrag.get_count()  &&  fpl->eintrag[f2].pos == eintrag[f1].pos) {
@@ -313,7 +299,7 @@ schedule_t::matches(karte_t *welt, const schedule_t *fpl)
 			bool ok = false;
 			if(  f1<eintrag.get_count()  ) {
 				grund_t *gr1 = welt->lookup(eintrag[f1].pos);
-				if(  gr1->get_depot()  ) {
+				if(  gr1  &&  gr1->get_depot()  ) {
 					// skip depot
 					f1++;
 					ok = true;
@@ -321,7 +307,7 @@ schedule_t::matches(karte_t *welt, const schedule_t *fpl)
 			}
 			if(  f2<fpl->eintrag.get_count()  ) {
 				grund_t *gr2 = welt->lookup(fpl->eintrag[f2].pos);
-				if(  gr2->get_depot()  ) {
+				if(  gr2  &&  gr2->get_depot()  ) {
 					ok = true;
 					f2++;
 				}
@@ -348,3 +334,81 @@ void schedule_t::add_return_way()
 		}
 	}
 }
+
+
+
+void schedule_t::sprintf_schedule( cbuffer_t &buf ) const
+{
+	buf.append( aktuell );
+	buf.append( "|" );
+	buf.append( (int)get_type() );
+	buf.append( "|" );
+	for(  uint8 i = 0;  i<eintrag.get_count();  i++  ) {
+		buf.printf( "%s,%i,%i|", eintrag[i].pos.get_str(), (int)eintrag[i].ladegrad, (int)eintrag[i].waiting_time_shift );
+	}
+}
+
+
+bool schedule_t::sscanf_schedule( const char *ptr )
+{
+	const char *p = ptr;
+	// first: clear current schedule
+	while(  eintrag.get_count()>0  ) {
+		remove();
+	}
+	//  first get aktuell pointer
+	aktuell = atoi( p );
+	while(  *p  &&  *p!='|'  ) {
+		p++;
+	}
+	if(  *p!='|'  ) {
+		dbg->error( "schedule_t::sscanf_schedule()","incomplete entry termination!" );
+		return false;
+	}
+	p++;
+	//  then schedule type
+	int type = atoi( p );
+	//  .. check for correct type
+	if(  type != (int)get_type()) {
+		dbg->error( "schedule_t::sscanf_schedule()","schedule has wrong type (%d)! should have been %d.", type, get_type() );
+		return false;
+	}
+	while(  *p  &&  *p!='|'  ) {
+		p++;
+	}
+	if(  *p!='|'  ) {
+		dbg->error( "schedule_t::sscanf_schedule()","incomplete entry termination!" );
+		return false;
+	}
+	p++;
+	// now scan the entries
+	while(  *p>0  ) {
+		sint16 values[5];
+		for(  sint8 i=0;  i<5;  i++  ) {
+			values[i] = atoi( p );
+			while(  *p  &&  (*p!=','  &&  *p!='|')  ) {
+				p++;
+			}
+			if(  i<4  &&  *p!=','  ) {
+				dbg->error( "schedule_t::sscanf_schedule()","incomplete string!" );
+				return false;
+			}
+			if(  i==4  &&  *p!='|'  ) {
+				dbg->error( "schedule_t::sscanf_schedule()","incomplete entry termination!" );
+				return false;
+			}
+			p++;
+		}
+		// ok, now we have a complete entry
+#ifndef _MSC_VER
+		struct linieneintrag_t stop = { koord3d(values[0],values[1],values[2]), values[3], values[4] };
+#else
+		struct linieneintrag_t stop;
+		stop.pos = koord3d(values[0],values[1],values[2]);
+		stop.ladegrad = values[3];
+		stop.waiting_time_shift = values[4];
+#endif
+		eintrag.append( stop );
+	}
+	return true;
+}
