@@ -19,7 +19,7 @@ class vehikel_basis_t;
  * road sign for traffic (one way minimum speed, traffic lights)
  * @author Hj. Malthaner
  */
-class crossing_t : public ding_t
+class crossing_t : public ding_no_info_t
 {
 protected:
 	image_id after_bild, bild;
@@ -29,9 +29,16 @@ protected:
 	const kreuzung_besch_t *besch;
 
 public:
-	enum ding_t::typ get_typ() const { return crossing; }
+	typ get_typ() const { return crossing; }
 	const char* get_name() const { return "Kreuzung"; }
 
+	/**
+	 * waytype associated with this object
+	 * for crossings: return invalid_wt since they do not need a way
+	 * if the way is deleted the crossing will be deleted, too
+	 */
+	waytype_t get_waytype() const { return invalid_wt; }
+
 	crossing_t(karte_t *welt, loadsave_t *file);
 	crossing_t(karte_t *welt, spieler_t *sp, koord3d pos, const kreuzung_besch_t *besch, uint8 ns = 0);
 
@@ -47,8 +54,11 @@ public:
 	 */
 	void info(cbuffer_t & buf) const { logic->info(buf); }
 
-	// no info
-	void zeige_info() {}
+	/**
+	 * @return NULL wenn OK, ansonsten eine Fehlermeldung
+	 * @author Hj. Malthaner
+	 */
+	virtual const char *ist_entfernbar(const spieler_t *sp);
 
 	// returns true, if the crossing can be passed by this vehicle
 	bool request_crossing( const vehikel_basis_t *v ) { return logic->request_crossing( v ); }
@@ -69,6 +79,13 @@ public:
 	void set_logic( crossing_logic_t *l ) { logic = l; }
 	crossing_logic_t *get_logic() { return logic; }
 
+	/*
+	* called whenever the snowline height changes
+	* return false and the ding_t will be deleted
+	* @author prissi
+	*/
+	virtual bool check_season(const long) { calc_bild(); return true; }
+
 	/**
 	 * Dient zur Neuberechnung des Bildes
 	 * @author Hj. Malthaner
