@@ -15,6 +15,7 @@
 #include "components/gui_label.h"
 #include "components/action_listener.h"                                // 28-Dec-2001  Markus Weber    Added
 #include "components/gui_button.h"
+#include "../tpl/vector_tpl.h"
 
 class spieler_t;
 class ware_besch_t;
@@ -34,7 +35,7 @@ public:
 	enum filter_flag_t { any_filter=1, name_filter=2, typ_filter=4, ware_filter=8, spezial_filter=16,
 		sub_filter=32,	// Ab hier beginnen die Unterfilter!
 		lkws_filter=32, zuege_filter=64, schiffe_filter=128, aircraft_filter=256,
-		noroute_filter=512, nofpl_filter=1024, noincome_filter=2048, indepot_filter=4096, noline_filter=8192, stucked_filter=16384, monorail_filter=32768, maglev_filter=65536, narrowgauge_filter=131072 };
+		noroute_filter=512, nofpl_filter=1024, noincome_filter=2048, indepot_filter=4096, noline_filter=8192, stucked_filter=16384, monorail_filter=32768, maglev_filter=65536, narrowgauge_filter=131072, tram_filter=131072*2 };
 
 private:
 	spieler_t *owner;
@@ -61,7 +62,7 @@ private:
 	/*
 	 * Child window, if open
 	 */
-	gui_fenster_t *filter_frame;
+	gui_frame_t *filter_frame;
 
 	/*
 	 * All filter settings are static, so they are not reset each
@@ -76,7 +77,7 @@ private:
 
 	static slist_tpl<const ware_besch_t *> waren_filter;
 
-	static int compare_convois(const void *a, const void *b);
+	static bool compare_convois(convoihandle_t, convoihandle_t);
 
 	/**
 	 * Check all filters for one convoi.
@@ -100,7 +101,7 @@ public:
 	 * gemeldet
 	 * @author V. Meyer
 	 */
-	void infowin_event(const event_t *ev);
+	bool infowin_event(const event_t *ev);
 
 	/**
 	 * The filter frame tells us when it is closed.
