@@ -16,20 +16,20 @@
 #include "../simintr.h"
 #include "../simgraph.h"
 #include "../simwin.h"
+#include "../player/simplay.h"
 
 #include "../besch/reader/obj_reader.h"
 #include "../simskin.h"
 #include "../besch/skin_besch.h"
 
 
-gui_frame_t::gui_frame_t(const char* name, const spieler_t* sp) :
-	opaque(true)
+gui_frame_t::gui_frame_t(char const* const name, spieler_t const* const sp)
 {
 	this->name = name;
 	groesse = koord(200, 100);
 	owner = sp;
-	container.set_pos(koord(0,16));
-	set_resizemode (no_resize); //25-may-02	markus weber	added
+	container.set_pos(koord(0,TITLEBAR_HEIGHT));
+	set_resizemode(no_resize); //25-may-02	markus weber	added
 	dirty = true;
 }
 
@@ -62,31 +62,41 @@ void gui_frame_t::set_fenstergroesse(koord groesse)
 }
 
 
+/**
+ * gibt farbinformationen fuer Fenstertitel, -rnder und -krper
+ * zurck
+ * @author Hj. Malthaner
+ */
+PLAYER_COLOR_VAL gui_frame_t::get_titelcolor() const
+{
+	return owner ? PLAYER_FLAG|(owner->get_player_color1()+1) : WIN_TITEL;
+}
+
 
 /**
  * Events werden hiermit an die GUI-Komponenten
  * gemeldet
  * @author Hj. Malthaner
  */
-void gui_frame_t::infowin_event(const event_t *ev)
+bool gui_frame_t::infowin_event(const event_t *ev)
 {
 	// %DB0 printf( "\nMessage: gui_frame_t::infowin_event( event_t const * ev ) : Fenster|Window %p : Event is %d", (void*)this, ev->ev_class );
 	if(IS_WINDOW_RESIZE(ev)) {
 		koord delta (ev->mx - ev->cx, ev->my - ev->cy);
 		resize(delta);
-		return;	// not pass to childs!
+		return true;	// not pass to childs!
 	} else if(IS_WINDOW_MAKE_MIN_SIZE(ev)) {
 		set_fenstergroesse( get_min_windowsize() ) ;
 		resize( koord(0,0) ) ;
-		return;	// not pass to childs!
+		return true;	// not pass to childs!
 	}
 	else if(ev->ev_class==INFOWIN  &&  (ev->ev_code==WIN_CLOSE  ||  ev->ev_code==WIN_OPEN  ||  ev->ev_code==WIN_TOP)) {
 		dirty = true;
 		container.clear_dirty();
 	}
 	event_t ev2 = *ev;
-	translate_event(&ev2, 0, -16);
-	container.infowin_event(&ev2);
+	translate_event(&ev2, 0, -TITLEBAR_HEIGHT);
+	return container.infowin_event(&ev2);
 }
 
 
@@ -103,14 +113,14 @@ void gui_frame_t::resize(const koord delta)
 
 	// resize window to the minimal width
 	if (new_size.x < min_windowsize.x) {
+		size_change.x = min_windowsize.x - groesse.x;
 		new_size.x = min_windowsize.x;
-		size_change.x = 0;
 	}
 
 	// resize window to the minimal heigth
 	if (new_size.y < min_windowsize.y) {
+		size_change.y = min_windowsize.y - groesse.y;
 		new_size.y = min_windowsize.y;
-		size_change.y = 0;
 	}
 
 	// resize window
@@ -135,33 +145,33 @@ void gui_frame_t::zeichnen(koord pos, koord gr)
 		dirty = false;
 	}
 
-	if(opaque) {
-		// Hajo: skinned windows code
-		if(obj_reader_t::has_been_init) {
-			// draw background
-			PUSH_CLIP(pos.x+1,pos.y+16,gr.x-2,gr.y-16);
-			const int img = skinverwaltung_t::window_skin->get_bild(0)->get_nummer();
+	// draw background
+	PUSH_CLIP(pos.x+1,pos.y+16,gr.x-2,gr.y-16);
 
-			for(int j=0; j<gr.y; j+=64) {
-				for(int i=0; i<gr.x; i+=64) {
-					// the background will not trigger a redraw!
-					display_color_img(img, pos.x+1 + i, pos.y+16 + j, 0, false, false);
-				}
+	// Hajo: skinned windows code
+	if(skinverwaltung_t::window_skin!=NULL) {
+		const int img = skinverwaltung_t::window_skin->get_bild_nr(0);
+
+		for(int j=0; j<gr.y; j+=64) {
+			for(int i=0; i<gr.x; i+=64) {
+				// the background will not trigger a redraw!
+				display_color_img(img, pos.x+1 + i, pos.y+16 + j, 0, false, false);
 			}
-			POP_CLIP();
 		}
-		else {
-			// empty box
-			display_fillbox_wh(pos.x, pos.y, gr.x, gr.y, MN_GREY1, false);
-		}
-
-		// Hajo: left, right
-		display_vline_wh(pos.x, pos.y+16, gr.y-16, MN_GREY4, false);
-		display_vline_wh(pos.x+gr.x-1, pos.y+16, gr.y-16, MN_GREY0, false);
-
-		// Hajo: bottom line
-		display_fillbox_wh(pos.x, pos.y+gr.y-1, gr.x, 1, MN_GREY0, false);
+	}
+	else {
+		// empty box
+		display_fillbox_wh(pos.x+1, pos.y+16, gr.x-2, gr.y-16, MN_GREY1, false);
 	}
 
+	// Hajo: left, right
+	display_vline_wh(pos.x, pos.y+16, gr.y-16, MN_GREY4, false);
+	display_vline_wh(pos.x+gr.x-1, pos.y+16, gr.y-16, MN_GREY0, false);
+
+	// Hajo: bottom line
+	display_fillbox_wh(pos.x, pos.y+gr.y-1, gr.x, 1, MN_GREY0, false);
+
 	container.zeichnen(pos);
+
+	POP_CLIP();
 }
