@@ -12,11 +12,16 @@
 #ifndef gui_gui_frame_h
 #define gui_gui_frame_h
 
-#include "../ifc/gui_fenster.h"
+#include "../dataobj/koord.h"
+#include "../simgraph.h"
 #include "gui_container.h"
-#include "../player/simplay.h"
 #include "../simcolor.h"
 
+// height of titlebar
+#define TITLEBAR_HEIGHT (16)
+
+class loadsave_t;
+class spieler_t;
 
 /**
  * Eine Klasse fr Fenster mit Komponenten.
@@ -26,8 +31,18 @@
  *
  * @author Hj. Malthaner
  */
-class gui_frame_t : virtual public gui_fenster_t
+class gui_frame_t
 {
+public:
+	/**
+	 * Resize modes
+	 * @author Markus Weber
+	 * @date   11-May-2002
+	 */
+	enum resize_modes {
+		no_resize = 0, vertical_resize = 1, horizonal_resize = 2, diagonal_resize = 3
+	};
+
 private:
 	gui_container_t container;
 
@@ -41,11 +56,9 @@ private:
 	 */
 	koord min_windowsize;
 
-	enum resize_modes resize_mode ;      //25-may-02	markus weber added
+	resize_modes resize_mode; // 25-may-02  markus weber added
 	const spieler_t *owner;
 
-	bool opaque:1;
-
 	// set true for total redraw
 	bool dirty:1;
 
@@ -69,6 +82,8 @@ public:
 	 */
 	gui_frame_t(const char *name, const spieler_t *sp=NULL);
 
+	virtual ~gui_frame_t() {}
+
 	/**
 	 * Fgt eine Komponente zum Fenster hinzu.
 	 * @author Hj. Malthaner
@@ -86,7 +101,7 @@ public:
 	 * @return den nicht uebersetzten Namen der Komponente
 	 * @author Hj. Malthaner
 	 */
-	const char * get_name() const { return name; }
+	const char *get_name() const { return name; }
 
 	/**
 	 * setzt den Namen (Fenstertitel)
@@ -94,18 +109,19 @@ public:
 	 */
 	void set_name(const char *name) { this->name=name; }
 
-	/**
-	 * setzt die Transparenz
-	 * @author Hj. Malthaner
+	/* this returns an unique id, if the dialoge can be saved
+	 * if this is defined, you better define a matching constructor with karte_t * and loadsave_t *
 	 */
-	void set_opaque(bool janein) { opaque = janein; }
+	virtual uint32 get_rdwr_id() { return 0; }
+
+	virtual void rdwr( loadsave_t * ) {}
 
 	/**
 	 * gibt farbinformationen fuer Fenstertitel, -rnder und -krper
 	 * zurck
 	 * @author Hj. Malthaner
 	 */
-	virtual PLAYER_COLOR_VAL get_titelcolor() const { return owner ? PLAYER_FLAG|(owner->get_player_color1()+1) : WIN_TITEL; }
+	virtual PLAYER_COLOR_VAL get_titelcolor() const;
 
 	/**
 	 * @return gibt wunschgroesse fr das Darstellungsfenster zurueck
@@ -140,12 +156,69 @@ public:
 	*/
 	koord get_client_windowsize() const {return groesse-koord(0,16); }
 
+	/**
+	 * Manche Fenster haben einen Hilfetext assoziiert.
+	 * @return den Dateinamen fr die Hilfe, oder NULL
+	 * @author Hj. Malthaner
+	 */
+	virtual const char * get_hilfe_datei() const {return NULL;}
+
+	/**
+	 * Does this window need a min size button in the title bar?
+	 * @return true if such a button is needed
+	 * @author Hj. Malthaner
+	 */
+	virtual bool has_min_sizer() const {return false;}
+
+	/**
+	 * Does this window need a next button in the title bar?
+	 * @return true if such a button is needed
+	 * @author Volker Meyer
+	 */
+	virtual bool has_next() const {return false;}
+
+	/**
+	 * Does this window need a prev button in the title bar?
+	 * @return true if such a button is needed
+	 * @author Volker Meyer
+	 */
+	virtual bool has_prev() const {return has_next();}
+
+	virtual bool has_sticky() const { return true; }
+
+	// if false, title and all gadgets will be not drawn
+	virtual bool has_title() const { return true; }
+
+	/**
+	 * Set resize mode
+	 * @author Markus Weber
+	 * @date   11-May-2002
+	 */
+	void set_resizemode(resize_modes mode) { resize_mode = mode; }
+
+	/**
+	 * Get resize mode
+	 * @author Markus Weber
+	 * @date   25-May-2002
+	 */
+	resize_modes get_resizemode(void) { return resize_mode; }
+
+	/**
+	 * Prft, ob eine Position innerhalb der Komponente liegt.
+	 * @author Hj. Malthaner
+	 */
+	virtual bool getroffen(int x, int y)
+	{
+		koord groesse = get_fenstergroesse();
+		return (  x>=0  &&  y>=0  &&  x<groesse.x  &&  y<groesse.y  );
+	}
+
 	/**
 	 * Events werden hiermit an die GUI-Komponenten
 	 * gemeldet
 	 * @author Hj. Malthaner
 	 */
-	virtual void infowin_event(const event_t *ev);
+	virtual bool infowin_event(const event_t *ev);
 
 	/**
 	 * komponente neu zeichnen. Die bergebenen Werte beziehen sich auf
@@ -155,22 +228,11 @@ public:
 	 */
 	virtual void zeichnen(koord pos, koord gr);
 
-	/**
-	 * Set resize mode
-	 * @author Markus Weber
-	 * @date   11-May-2002
-	 */
-	void set_resizemode(enum resize_modes mode) {resize_mode = mode;}
-
-	/**
-	 * Get resize mode
-	 * @author Markus Weber
-	 * @date   25-May-2002
-	 */
-	enum resize_modes get_resizemode(void) {return resize_mode;}
-
+	// called, when the map is rotated
+	virtual void map_rotate90( sint16 /*new_ysize*/ ) { }
 
 	void set_focus( gui_komponente_t *k ) { container.set_focus(k); }
+	virtual gui_komponente_t *get_focus() { return container.get_focus(); }
 };
 
 #endif
