@@ -13,11 +13,16 @@
 #include "../simcolor.h"
 #include "../simgraph.h"
 #include "../simwin.h"
+#include "../simwerkz.h"
 #include "../simskin.h"
-#include "../utils/simstring.h"
+
 #include "../freight_list_sorter.h"
+
 #include "../dataobj/umgebung.h"
 #include "../dataobj/translator.h"
+#include "../dataobj/loadsave.h"
+
+#include "../utils/simstring.h"
 #include "components/list_button.h"
 #include "../besch/skin_besch.h"
 
@@ -65,75 +70,44 @@ const int cost_type_color[MAX_HALT_COST] =
 	COL_VEHICLE_ASSETS
 };
 
-halt_info_t::halt_info_t(karte_t *welt, halthandle_t halt)
-	: gui_frame_t(edit_name, halt->get_besitzer()),
+halt_info_t::halt_info_t(karte_t *welt, halthandle_t halt) :
+		gui_frame_t(halt->get_name(), halt->get_besitzer()),
 		scrolly(&text),
 		text("                                                                                     "
 			" \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n"
 			" \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n"
 		),
 		sort_label(translator::translate("Hier warten/lagern:")),
-		view(welt, halt->get_basis_pos3d()),
-		freight_info(16384),
+		view(welt, halt->get_basis_pos3d(), koord( max(64, get_base_tile_raster_width()), max(56, (get_base_tile_raster_width()*7)/8) )),
+		freight_info(4096),
 		info_buf(256)
 {
 	this->halt = halt;
 	this->welt = welt;
 	halt->set_sortby( umgebung_t::default_sortmode );
 
-	input.set_pos(koord(11,4));
+	const sint16 offset_below_viewport = 21 + view.get_groesse().y;
+	const sint16 total_width = 3*(BUTTON_WIDTH + BUTTON_SPACER) + max(BUTTON_WIDTH + 2*BUTTON_SPACER, view.get_groesse().x + 32);
+
+	input.set_pos(koord(10,4));
 	tstrncpy(edit_name, halt->get_name(), lengthof(edit_name));
 	input.set_text(edit_name, lengthof(edit_name));
-
-	add_komponente(&sort_label);
-
-	// hsiegeln: added sort_button
-	sort_button.set_groesse(koord(BUTTON_WIDTH, BUTTON_HEIGHT));
-	sort_button.set_pos(koord(BUTTON1_X, 78));
-	sort_button.set_text(sort_text[umgebung_t::default_sortmode]);
-	sort_button.set_typ(button_t::roundbox);
-	sort_button.set_tooltip("Sort waiting list by");
-
-	toggler.set_groesse(koord(BUTTON_WIDTH, BUTTON_HEIGHT));
-	toggler.set_pos(koord(BUTTON3_X, 78));
-	toggler.set_text("Chart");
-	toggler.set_typ(button_t::roundbox_state);
-	toggler.set_tooltip("Show/hide statistics");
-	toggler.add_listener(this);
-	toggler.pressed = false;
-	add_komponente(&toggler);
-
-	button.set_groesse(koord(BUTTON_WIDTH, BUTTON_HEIGHT));
-	button.set_pos(koord(BUTTON4_X, 78));
-	button.set_text("Details");
-	button.set_typ(button_t::roundbox);
-
-	scrolly.set_pos(koord(1, 78+BUTTON_HEIGHT+4));
-
-	add_komponente(&sort_button);
-	add_komponente(&button);
 	add_komponente(&input);
+	input.add_listener(this);
 
-	set_fenstergroesse(koord(BUTTON4_X+BUTTON_WIDTH+2, 264));
-	set_min_windowsize(koord(BUTTON4_X+BUTTON_WIDTH+2, 194));
-	set_resizemode(diagonal_resize);     // 31-May-02	markus weber	added
-	resize(koord(0,0));
-
-	button.add_listener(this);
-//	input.add_listener(this);
-	sort_button.add_listener(this);
+	add_komponente(&view);
 
 	// chart
-	chart.set_pos(koord(46,80));
-	chart.set_groesse(koord(BUTTON4_X+BUTTON_WIDTH-50, 100));
+	chart.set_pos(koord(66,view.get_groesse().y+24));
+	chart.set_groesse(koord(BUTTON4_X+BUTTON_WIDTH-70, 100));
 	chart.set_dimension(12, 10000);
 	chart.set_visible(false);
 	chart.set_background(MN_GREY1);
 
 	for (int cost = 0; cost<MAX_HALT_COST; cost++) {
-		chart.add_curve(cost_type_color[cost], halt->get_finance_history(), MAX_HALT_COST, index_of_haltinfo[cost], MAX_MONTHS, 0, false, true);
+		chart.add_curve(cost_type_color[cost], halt->get_finance_history(), MAX_HALT_COST, index_of_haltinfo[cost], MAX_MONTHS, 0, false, true, 0);
 		filterButtons[cost].init(button_t::box_state, cost_type[cost],
-			koord(BUTTON1_X+(BUTTON_WIDTH+BUTTON_SPACER)*(cost%4), 198+(BUTTON_HEIGHT+2)*(cost/4) ),
+			koord(BUTTON1_X+(BUTTON_WIDTH+BUTTON_SPACER)*(cost%4), view.get_groesse().y+142+(BUTTON_HEIGHT+2)*(cost/4) ),
 			koord(BUTTON_WIDTH, BUTTON_HEIGHT));
 		filterButtons[cost].add_listener(this);
 		filterButtons[cost].background = cost_type_color[cost];
@@ -143,11 +117,60 @@ halt_info_t::halt_info_t(karte_t *welt, halthandle_t halt)
 	}
 	add_komponente(&chart);
 
-	add_komponente(&view);
+	add_komponente(&sort_label);
+
+	// hsiegeln: added sort_button
+	sort_button.set_groesse(koord(BUTTON_WIDTH, BUTTON_HEIGHT));
+	sort_button.set_pos(koord(BUTTON1_X, offset_below_viewport));
+	sort_button.set_text(sort_text[umgebung_t::default_sortmode]);
+	sort_button.set_typ(button_t::roundbox);
+	sort_button.set_tooltip("Sort waiting list by");
+	add_komponente(&sort_button);
+
+	toggler.set_groesse(koord(BUTTON_WIDTH, BUTTON_HEIGHT));
+	toggler.set_pos(koord(BUTTON3_X, offset_below_viewport));
+	toggler.set_text("Chart");
+	toggler.set_typ(button_t::roundbox_state);
+	toggler.set_tooltip("Show/hide statistics");
+	toggler.add_listener(this);
+	toggler.pressed = false;
+	add_komponente(&toggler);
+
+	button.set_groesse(koord(BUTTON_WIDTH, BUTTON_HEIGHT));
+	button.set_pos(koord(BUTTON4_X, offset_below_viewport));
+	button.set_text("Details");
+	button.set_typ(button_t::roundbox);
+
+	scrolly.set_pos(koord(0, offset_below_viewport+BUTTON_HEIGHT+4));
+
+	add_komponente(&button);
+
+	set_fenstergroesse(koord(total_width, view.get_groesse().y+208));
+	set_min_windowsize(koord(total_width, view.get_groesse().y+138));
+	set_resizemode(diagonal_resize);     // 31-May-02	markus weber	added
+	resize(koord(0,0));
+
+	button.add_listener(this);
+	sort_button.add_listener(this);
+
 	add_komponente(&scrolly);
 }
 
 
+halt_info_t::~halt_info_t()
+{
+	if(  halt.is_bound()  &&  strcmp(halt->get_name(),edit_name)  &&  edit_name[0]  ) {
+		// text changed => call tool
+		cbuffer_t buf(300);
+		buf.printf( "h%u,%s", halt.get_id(), edit_name );
+		werkzeug_t *w = create_tool( WKZ_RENAME_TOOL | SIMPLE_TOOL );
+		w->set_default_param( buf );
+		halt->get_welt()->set_werkzeug( w, halt->get_besitzer() );
+		// since init always returns false, it is save to delete immediately
+		delete w;
+	}
+}
+
 
 /**
  * Komponente neu zeichnen. Die bergebenen Werte beziehen sich auf
@@ -155,19 +178,15 @@ halt_info_t::halt_info_t(karte_t *welt, halthandle_t halt)
  * in dem die Komponente dargestellt wird.
  * @author Hj. Malthaner
  */
-void
-halt_info_t::zeichnen(koord pos, koord gr)
+void halt_info_t::zeichnen(koord pos, koord gr)
 {
 	if(halt.is_bound()) {
-		if(strcmp(edit_name,halt->get_name())) {
-			halt->set_name( edit_name );
-		}
 
 		// buffer update now only when needed by halt itself => dedicated buffer for this
 		int old_len=freight_info.len();
 		halt->get_freight_info(freight_info);
-		text.set_text(freight_info);
 		if(old_len!=freight_info.len()) {
+			text.set_text(freight_info);
 			text.recalc_size();
 		}
 
@@ -175,10 +194,10 @@ halt_info_t::zeichnen(koord pos, koord gr)
 
 		sint16 top = pos.y+36;
 		COLOR_VAL indikatorfarbe = halt->get_status_farbe();
-		display_fillbox_wh_clip(pos.x+11, top+2, INDICATOR_WIDTH, INDICATOR_HEIGHT, indikatorfarbe, true);
+		display_fillbox_wh_clip(pos.x+10, top+2, INDICATOR_WIDTH, INDICATOR_HEIGHT, indikatorfarbe, true);
 
 		// now what do we accept here?
-		int left = 11+INDICATOR_WIDTH+2;
+		int left = 10+INDICATOR_WIDTH+2;
 		if (halt->get_pax_enabled()) {
 			display_color_img(skinverwaltung_t::passagiere->get_bild_nr(0), pos.x+left, top, 0, false, false);
 			left += 10;
@@ -220,6 +239,10 @@ halt_info_t::zeichnen(koord pos, koord gr)
 			display_color_img(skinverwaltung_t::monorailhaltsymbol->get_bild_nr(0), pos.x+left, top, 0, false, false);
 			left += 23;
 		}
+		if (halttype & haltestelle_t::tramstop) {
+			display_color_img(skinverwaltung_t::tramhaltsymbol->get_bild_nr(0), pos.x+left, top, 0, false, false);
+			left += 23;
+		}
 		if (halttype & haltestelle_t::maglevstop) {
 			display_color_img(skinverwaltung_t::maglevhaltsymbol->get_bild_nr(0), pos.x+left, top, 0, false, false);
 			left += 23;
@@ -233,7 +256,7 @@ halt_info_t::zeichnen(koord pos, koord gr)
 		info_buf.clear();
 		info_buf.append(translator::translate("Storage capacity"));
 		info_buf.append(": ");
-		left = pos.x+11;
+		left = pos.x+10;
 		// passagiere
 		info_buf.append(halt->get_capacity(0));
 		left += display_proportional(left, top, info_buf, ALIGN_LEFT, COL_BLACK, true);
@@ -261,7 +284,23 @@ halt_info_t::zeichnen(koord pos, koord gr)
 		// information about the convoi itself
 		info_buf.clear();
 		halt->info(info_buf);
-		display_multiline_text(pos.x+11, pos.y+53+LINESPACE, info_buf, COL_BLACK);
+		display_multiline_text(pos.x+10, pos.y+53+LINESPACE, info_buf, COL_BLACK);
+	}
+}
+
+
+// activate the statistic
+void halt_info_t::show_hide_statistics( bool show )
+{
+	toggler.pressed = show;
+	const koord offset = show ? koord(0, 170) : koord(0, -170);
+	set_min_windowsize(get_min_windowsize() + offset);
+	scrolly.set_pos(scrolly.get_pos() + offset);
+	chart.set_visible(show);
+	set_fenstergroesse(get_fenstergroesse() + offset);
+	resize(koord(0,0));
+	for (int i=0;i<MAX_HALT_COST;i++) {
+		filterButtons[i].set_visible(toggler.pressed);
 	}
 }
 
@@ -273,30 +312,34 @@ halt_info_t::zeichnen(koord pos, koord gr)
 bool halt_info_t::action_triggered( gui_action_creator_t *comp,value_t /* */)
 {
 	if (comp == &button) { 			// details button pressed
-		create_win( new halt_detail_t(halt), w_info, (long)this);
+		create_win( new halt_detail_t(halt), w_info, magic_halt_detail + halt.get_id() );
 	} else if (comp == &sort_button) { 	// @author hsiegeln sort button pressed
 		umgebung_t::default_sortmode = ((int)(halt->get_sortby())+1)%4;
 		halt->set_sortby((freight_list_sorter_t::sort_mode_t) umgebung_t::default_sortmode);
 		sort_button.set_text(sort_text[umgebung_t::default_sortmode]);
 	} else  if (comp == &toggler) {
-		toggler.pressed ^= 1;
-		const koord offset = toggler.pressed ? koord(0, 165) : koord(0, -165);
-		set_min_windowsize(koord(BUTTON4_X+BUTTON_WIDTH+2, toggler.pressed ?372:194));
-		scrolly.set_pos( scrolly.get_pos() + koord(0,offset.y) );
-		// toggle visibility of components
-		chart.set_visible(toggler.pressed);
-		set_fenstergroesse(get_fenstergroesse() + offset);
-		resize(koord(0,0));
-		for (int i=0;i<MAX_HALT_COST;i++) {
-			filterButtons[i].set_visible(toggler.pressed);
+		show_hide_statistics( toggler.pressed^1 );
+	}
+	else if(  comp == &input  ) {
+		if(  strcmp(halt->get_name(),edit_name)  ) {
+			// text changed => call tool
+			cbuffer_t buf(300);
+			buf.printf( "h%u,%s", halt.get_id(), edit_name );
+			werkzeug_t *w = create_tool( WKZ_RENAME_TOOL | SIMPLE_TOOL );
+			w->set_default_param( buf );
+			halt->get_welt()->set_werkzeug( w, halt->get_besitzer() );
+			// since init always returns false, it is save to delete immediately
+			delete w;
 		}
-	} else {
+	}
+	else {
 		for( int i = 0; i<MAX_HALT_COST; i++) {
 			if (comp == &filterButtons[i]) {
 				filterButtons[i].pressed = !filterButtons[i].pressed;
 				if(filterButtons[i].pressed) {
 					chart.show_curve(i);
-				} else {
+				}
+				else {
 					chart.hide_curve(i);
 				}
 				break;
@@ -309,21 +352,20 @@ bool halt_info_t::action_triggered( gui_action_creator_t *comp,value_t /* */)
 
 
 /**
- * Resize the contents of the window
+ * Set window size and adjust component sizes and/or positions accordingly
  * @author Markus Weber
  */
-void halt_info_t::resize(const koord delta)
+void halt_info_t::set_fenstergroesse(koord groesse)
 {
-	gui_frame_t::resize(delta);
+	gui_frame_t::set_fenstergroesse(groesse);
 
-	const sint16 button_offset_y = toggler.pressed?245:80;
-
-	input.set_groesse(koord(get_fenstergroesse().x-23, 13));
-	toggler.set_pos(koord(BUTTON3_X,button_offset_y));
-	button.set_pos(koord(BUTTON4_X,button_offset_y));
-	sort_button.set_pos(koord(BUTTON1_X,button_offset_y));
-	sort_label.set_pos(koord(BUTTON1_X,button_offset_y-LINESPACE));
-	view.set_pos(koord(get_fenstergroesse().x - 64 - 16 , 21));
+	const sint16 yoff = scrolly.get_pos().y-BUTTON_HEIGHT-2;
+	input.set_groesse(koord(get_fenstergroesse().x-20, 13));
+	toggler.set_pos(koord(BUTTON3_X,yoff));
+	button.set_pos(koord(BUTTON4_X,yoff));
+	sort_button.set_pos(koord(BUTTON1_X,yoff));
+	sort_label.set_pos(koord(BUTTON1_X,yoff-LINESPACE));
+	view.set_pos(koord(get_fenstergroesse().x - view.get_groesse().x - 10 , 21));
 	scrolly.set_groesse(get_client_windowsize()-scrolly.get_pos());
 }
 
@@ -331,7 +373,73 @@ void halt_info_t::resize(const koord delta)
 
 void halt_info_t::map_rotate90( sint16 new_ysize )
 {
-	koord3d l = view.get_location();
-	l.rotate90( new_ysize );
-	view.set_location( l );
+	view.map_rotate90(new_ysize);
+}
+
+
+halt_info_t::halt_info_t(karte_t *welt):
+	gui_frame_t("", NULL),
+	scrolly(&text),
+	text(""),
+	sort_label(NULL),
+	view(welt, koord3d::invalid, koord(64,64) ),
+	freight_info(0),
+	info_buf(0)
+{
+	// just a dummy
+	this->welt = welt;
+}
+
+
+void halt_info_t::rdwr(loadsave_t *file)
+{
+	koord3d halt_pos;
+	koord gr = get_fenstergroesse();
+	uint32 flags = 0;
+	bool stats = toggler.pressed;
+	sint32 xoff = scrolly.get_scroll_x();
+	sint32 yoff = scrolly.get_scroll_y();
+	if(  file->is_saving()  ) {
+		halt_pos = halt->get_basis_pos3d();
+		for( int i = 0; i<MAX_HALT_COST; i++) {
+			if(  filterButtons[i].pressed  ) {
+				flags |= (1<<i);
+			}
+		}
+	}
+	halt_pos.rdwr( file );
+	gr.rdwr( file );
+	file->rdwr_long( flags );
+	file->rdwr_byte( umgebung_t::default_sortmode );
+	file->rdwr_bool( stats );
+	file->rdwr_long( xoff );
+	file->rdwr_long( yoff );
+	if(  file->is_loading()  ) {
+		halt = welt->lookup( halt_pos )->get_halt();
+		// now we can open the window ...
+		KOORD_VAL xpos = win_get_posx( this );
+		KOORD_VAL ypos = win_get_posy( this );
+		halt_info_t *w = new halt_info_t(welt,halt);
+		create_win( xpos, ypos, w, w_info, magic_halt_info+halt.get_id() );
+		if(  stats  ) {
+			gr.y -= 170;
+		}
+		w->set_fenstergroesse( gr );
+		for( int i = 0; i<MAX_HALT_COST; i++) {
+			w->filterButtons[i].pressed = (flags>>i)&1;
+			if(w->filterButtons[i].pressed) {
+				w->chart.show_curve(i);
+			}
+		}
+		if(  stats  ) {
+			w->show_hide_statistics( true );
+		}
+		halt->get_freight_info(w->freight_info);
+		w->text.set_text(w->freight_info);
+		w->text.recalc_size();
+		w->scrolly.set_scroll_position( xoff, yoff );
+		// we must invalidate halthandle
+		halt = halthandle_t();
+		destroy_win( this );
+	}
 }
