@@ -9,6 +9,10 @@
 
 class pakselector_t : public savegame_frame_t
 {
+private:
+	button_t load_addons;
+	bool at_least_one_add;
+
 protected:
 	/**
 	* Aktion, die nach Knopfdruck gestartet wird.
@@ -20,7 +24,7 @@ protected:
 	* Aktion, die nach X-Knopfdruck gestartet wird.
 	* @author V. Meyer
 	*/
-	virtual void del_action(const char *filename);
+	virtual bool del_action(const char *filename);
 
 	// returns extra file info
 	virtual const char *get_info(const char *fname);
@@ -29,6 +33,10 @@ protected:
 	virtual bool check_file( const char *filename, const char *suffix );
 
 public:
+	void fill_list();	// do the search ...
+
+	virtual bool has_title() const { return false; }
+
 	bool has_pak() const { return !entries.empty(); }
 
 	const char * get_hilfe_datei() const { return ""; }
@@ -36,6 +44,16 @@ public:
 	// since we only want to see the frames ...
 	void zeichnen(koord pos, koord gr);
 
+	/**
+	 * This method is called if an action is triggered
+	 * @author Hj. Malthaner
+	 *
+	 * Returns true, if action is done and no more
+	 * components should be triggered.
+	 * V.Meyer
+	 */
+	virtual bool action_triggered( gui_action_creator_t *komp, value_t extra);
+
 	pakselector_t();
 };
 
