@@ -27,7 +27,7 @@ class schedule_list_gui_t : public gui_frame_t, public action_listener_t
 private:
 	spieler_t *sp;
 
-	button_t bt_new_line, bt_change_line, bt_delete_line;
+	button_t bt_new_line, bt_change_line, bt_delete_line, bt_withdraw_line;;
 	gui_container_t cont, cont_haltestellen;
 	gui_scrollpane_t scrolly, scrolly_haltestellen;
 	gui_scrolled_list_t scl;
@@ -37,13 +37,23 @@ private:
 	button_t filterButtons[MAX_LINE_COST];
 	gui_tab_panel_t tabs;
 
-	static const char cost_type[MAX_LINE_COST][64];
-	static const int cost_type_color[MAX_LINE_COST];
-	static uint8 statistic[MAX_LINE_COST];
-	static uint8 statistic_type[MAX_LINE_COST];
-
 	sint32 selection, capacity, load, loadfactor;
 
+	uint32 old_line_count;
+	sint32 last_schedule_count;
+	uint32 last_vehicle_count;
+
+	// so even japanese can have long enough names ...
+	char line_name[512], old_line_name[512];
+
+	// resets textinput to current line name
+	// necessary after line was renamed
+	void reset_line_name();
+
+	// rename selected line
+	// checks if possible / necessary
+	void rename_line();
+
 	void display(koord pos);
 
 	void update_lineinfo(linehandle_t new_line);
@@ -56,7 +66,7 @@ private:
 
 public:
 	schedule_list_gui_t(spieler_t* sp);
-
+	~schedule_list_gui_t();
 	/**
 	* in top-level fenstern wird der Name in der Titelzeile dargestellt
 	* @return den nicht uebersetzten Namen der Komponente
@@ -88,16 +98,16 @@ public:
 	void zeichnen(koord pos, koord gr);
 
 	/**
-	* resize window in response to a resize event
+	* Set window size and adjust component sizes and/or positions accordingly
 	* @author Hj. Malthaner
 	*/
-	void resize(const koord delta);
+	virtual void set_fenstergroesse(koord groesse);
 
    /**
    * Mausklicks werden hiermit an die GUI-Komponenten
    * gemeldet
    */
-   void infowin_event(const event_t *ev);
+   bool infowin_event(const event_t *ev);
 
 	/**
 	* This method is called if an action is triggered
@@ -114,6 +124,15 @@ public:
 	 * @author isidoro
 	 */
 	void show_lineinfo(linehandle_t line);
+
+	/**
+	 * called after renaming of line
+	 */
+	void update_data(linehandle_t changed_line);
+
+	// follwoing: rdwr stuff
+	void rdwr( loadsave_t *file );
+	uint32 get_rdwr_id();
 };
 
 #endif
