@@ -19,6 +19,7 @@
 #include "components/gui_tab_panel.h"
 
 class stadt_t;
+template <class T> class sparse_tpl;
 
 /**
  * Dies stellt ein Fenster mit den Informationen
@@ -29,10 +30,12 @@ class stadt_t;
 class stadt_info_t : public gui_frame_t, private action_listener_t
 {
 private:
-	char name[256];
+	char name[256], old_name[256];
 
 	stadt_t *stadt;
 
+    button_t allow_growth;
+
 	gui_textinput_t name_input;
 
 	gui_tab_panel_t year_month_tabs;
@@ -42,15 +45,29 @@ private:
 	button_t filterButtons[MAX_CITY_HISTORY];
 	bool bFilterIsActive[MAX_CITY_HISTORY];
 
+	uint8* pax_dest_old;
+	uint8* pax_dest_new;
+
+	unsigned long pax_destinations_last_change;
+
+	void init_pax_dest( uint8* pax_dest );
+	void add_pax_dest( uint8* pax_dest, const sparse_tpl< uint8 >* city_pax_dest );
+
+	void rename_city();
+
+	void reset_city_name();
+
 public:
 	stadt_info_t(stadt_t *stadt);
 
+	~stadt_info_t();
+
 	/**
 	 * Manche Fenster haben einen Hilfetext assoziiert.
 	 * @return den Dateinamen fr die Hilfe, oder NULL
 	 * @author Hj. Malthaner
 	 */
-	const char * get_hilfe_datei() const {return "citywindow.txt";}
+	const char *get_hilfe_datei() const {return "citywindow.txt";}
 
 	/**
 	* komponente neu zeichnen. Die bergebenen Werte beziehen sich auf
@@ -59,15 +76,22 @@ public:
 	*/
 	void zeichnen(koord pos, koord gr);
 
-    /**
-     * This method is called if an action is triggered
-     * @author Hj. Malthaner
-     *
-     * Returns true, if action is done and no more
-     * components should be triggered.
-     * V.Meyer
-     */
-    bool action_triggered( gui_action_creator_t *komp, value_t extra);
+	/**
+	 * This method is called if an action is triggered
+	 * @author Hj. Malthaner
+	 *
+	 * Returns true, if action is done and no more
+	 * components should be triggered.
+	 * V.Meyer
+	 */
+	bool action_triggered( gui_action_creator_t *komp, value_t extra);
+
+	void map_rotate90( sint16 );
+
+	// since we need to update the city pointer when topped
+	bool infowin_event(const event_t *ev);
+
+	void update_data();
 };
 
 #endif
