@@ -7,18 +7,31 @@
  */
 
 #include <string.h>
+#ifdef _MSC_VER
+#include <new.h> // for _set_new_handler
+#include <direct.h>
+#else
+#include <unistd.h>
+#endif
 
 #include "welt.h"
 #include "karte.h"
 
 #include "../simdebug.h"
-#include "../simio.h"
 #include "../simworld.h"
 #include "../simwin.h"
 #include "../simimg.h"
+#include "../simmesg.h"
 #include "../simtools.h"
+#include "../simversion.h"
+
+#include "../bauer/hausbauer.h"
+#include "../bauer/wegbauer.h"
+
+#include "../besch/haus_besch.h"
 
 #include "../dataobj/einstellungen.h"
+#include "../dataobj/loadsave.h"
 #include "../dataobj/umgebung.h"
 #include "../dataobj/translator.h"
 
@@ -27,6 +40,7 @@
 #include "../dings/baum.h"
 #include "../simcity.h"
 #include "../vehicle/simvehikel.h"
+#include "../player/simplay.h"
 
 #include "../simcolor.h"
 
@@ -37,6 +51,14 @@
 #include "../utils/simstring.h"
 #include "components/list_button.h"
 
+#include "sprachen.h"
+#include "climates.h"
+#include "settings_frame.h"
+#include "loadsave_frame.h"
+#include "load_relief_frame.h"
+#include "messagebox.h"
+#include "scenario_frame.h"
+
 #define START_HEIGHT (28)
 
 #define LEFT_ARROW (110)
@@ -59,50 +81,65 @@ DBG_MESSAGE("","sizeof(stat)=%d, sizeof(tm)=%d",sizeof(struct stat),sizeof(struc
 	this->welt = welt;
 	this->sets = sets;
 	this->old_lang = -1;
-	this->sets->set_beginner_mode(umgebung_t::default_einstellungen.get_beginner_mode());
+	this->sets->beginner_mode = umgebung_t::default_einstellungen.get_beginner_mode();
+
+	// find earliest start and end date ...
+	uint16 game_start = 4999;
+	uint16 game_ends = 0;
+	// first townhalls
+	const vector_tpl<const haus_besch_t *> *s = hausbauer_t::get_list(haus_besch_t::rathaus);
+	for (uint32 i = 0; i<s->get_count(); i++) {
+		const haus_besch_t *besch = (*s)[i];
+		uint16 intro_year = (besch->get_intro_year_month()+11)/12;
+		if(  intro_year<game_start  ) {
+			game_start = intro_year;
+		}
+		uint16 retire_year = (besch->get_retire_year_month()+11)/12;
+		if(  retire_year>game_ends  ) {
+			game_ends = retire_year;
+		}
+	}
+	// then streets
+	game_start = max( game_start, (wegbauer_t::get_earliest_way(road_wt)->get_intro_year_month()+11)/12 );
+	game_ends = min( game_ends, (wegbauer_t::get_latest_way(road_wt)->get_retire_year_month()+11)/12 );
+
 	loaded_heightfield = load_heightfield = false;
 	load = start = close = scenario = quit = false;
 	int intTopOfButton=START_HEIGHT;
 	sets->heightfield = "";
 
 	// select map stuff ..
+	inp_map_number.init( abs(sets->get_karte_nummer()), 0, 0x7FFFFFFF, 1, true );
 	inp_map_number.set_pos(koord(LEFT_ARROW, intTopOfButton));
 	inp_map_number.set_groesse(koord(RIGHT_ARROW-LEFT_ARROW+10, 12));
-	inp_map_number.set_limits(0,0x7FFFFFFF);
-	inp_map_number.set_value(abs(sets->get_karte_nummer())%9999);
 	inp_map_number.add_listener( this );
 	add_komponente( &inp_map_number );
 
 	intTopOfButton += 12;
 	intTopOfButton += 12;
 
+	inp_x_size.init( sets->get_groesse_x(), 8, min(32000,4194304/sets->get_groesse_y()), sets->get_groesse_x()>=512 ? 128 : 64, false );
 	inp_x_size.set_pos(koord(LEFT_ARROW,intTopOfButton) );
 	inp_x_size.set_groesse(koord(RIGHT_ARROW-LEFT_ARROW+10, 12));
 	inp_x_size.add_listener(this);
-	inp_x_size.set_value( sets->get_groesse_x() );
-	inp_x_size.set_limits( 64, min(32766,4194304/sets->get_groesse_y()) );
-	inp_x_size.set_increment_mode( sets->get_groesse_x()>=512 ? 128 : 64 );
-	inp_x_size.wrap_mode( false );
 	add_komponente( &inp_x_size );
 	intTopOfButton += 12;
 
+	inp_y_size.init( sets->get_groesse_y(), 8, min(32000,4194304/sets->get_groesse_x()), sets->get_groesse_y()>=512 ? 128 : 64, false );
 	inp_y_size.set_pos(koord(LEFT_ARROW,intTopOfButton) );
 	inp_y_size.set_groesse(koord(RIGHT_ARROW-LEFT_ARROW+10, 12));
 	inp_y_size.add_listener(this);
-	inp_y_size.set_limits( 64, min(32766,4194304/sets->get_groesse_x()) );
-	inp_y_size.set_value( sets->get_groesse_y() );
-	inp_y_size.set_increment_mode( sets->get_groesse_y()>=512 ? 128 : 64 );
-	inp_y_size.wrap_mode( false );
 	add_komponente( &inp_y_size );
 	intTopOfButton += 12;
 
 	// maps etc.
 	intTopOfButton += 5;
-	random_map.set_pos( koord(11, intTopOfButton) );
+	random_map.set_pos( koord(10, intTopOfButton) );
 	random_map.set_groesse( koord(104, BUTTON_HEIGHT) );
 	random_map.set_typ(button_t::roundbox);
 	random_map.add_listener( this );
 	add_komponente( &random_map );
+
 	load_map.set_pos( koord(104+11+30, intTopOfButton) );
 	load_map.set_groesse( koord(104, BUTTON_HEIGHT) );
 	load_map.set_typ(button_t::roundbox);
@@ -129,7 +166,6 @@ DBG_MESSAGE("","sizeof(stat)=%d, sizeof(tm)=%d",sizeof(struct stat),sizeof(struc
 	add_komponente( &inp_town_size );
 	intTopOfButton += 12;
 
-
 	inp_intercity_road_len.set_pos(koord(RIGHT_COLUMN,intTopOfButton) );
 	inp_intercity_road_len.set_groesse(koord(RIGHT_COLUMN_WIDTH, 12));
 	inp_intercity_road_len.add_listener(this);
@@ -139,16 +175,7 @@ DBG_MESSAGE("","sizeof(stat)=%d, sizeof(tm)=%d",sizeof(struct stat),sizeof(struc
 	add_komponente( &inp_intercity_road_len );
 	intTopOfButton += 12;
 
-	inp_traffic_density.set_pos(koord(RIGHT_COLUMN,intTopOfButton) );
-	inp_traffic_density.set_groesse(koord(RIGHT_COLUMN_WIDTH, 12));
-	inp_traffic_density.add_listener(this);
-	inp_traffic_density.set_limits(0,16);
-	inp_traffic_density.set_value(abs(sets->get_verkehr_level()) );
-	add_komponente( &inp_traffic_density );
-	intTopOfButton += 12;
-
 	// industry stuff
-	intTopOfButton += 5;
 	inp_other_industries.set_pos(koord(RIGHT_COLUMN,intTopOfButton) );
 	inp_other_industries.set_groesse(koord(RIGHT_COLUMN_WIDTH, 12));
 	inp_other_industries.add_listener(this);
@@ -157,14 +184,6 @@ DBG_MESSAGE("","sizeof(stat)=%d, sizeof(tm)=%d",sizeof(struct stat),sizeof(struc
 	add_komponente( &inp_other_industries );
 	intTopOfButton += 12;
 
-	inp_electric_producer.set_pos(koord(RIGHT_COLUMN,intTopOfButton) );
-	inp_electric_producer.set_groesse(koord(RIGHT_COLUMN_WIDTH, 12));
-	inp_electric_producer.add_listener(this);
-	inp_electric_producer.set_limits(0,100);
-	inp_electric_producer.set_value(abs(sets->get_electric_promille()/10) );
-	add_komponente( &inp_electric_producer );
-	intTopOfButton += 12;
-
 	inp_tourist_attractions.set_pos(koord(RIGHT_COLUMN,intTopOfButton) );
 	inp_tourist_attractions.set_groesse(koord(RIGHT_COLUMN_WIDTH, 12));
 	inp_tourist_attractions.add_listener(this);
@@ -184,28 +203,37 @@ DBG_MESSAGE("","sizeof(stat)=%d, sizeof(tm)=%d",sizeof(struct stat),sizeof(struc
 	inp_intro_date.set_pos(koord(RIGHT_COLUMN,intTopOfButton) );
 	inp_intro_date.set_groesse(koord(RIGHT_COLUMN_WIDTH, 12));
 	inp_intro_date.add_listener(this);
-	inp_intro_date.set_limits(1400,2050);
+	inp_intro_date.set_limits(game_start,game_ends);
 	inp_intro_date.set_increment_mode(10);
 	inp_intro_date.set_value(abs(sets->get_starting_year()) );
 	add_komponente( &inp_intro_date );
 	intTopOfButton += 12;
 
-	intTopOfButton += 5;
-	allow_player_change.set_pos( koord(10,intTopOfButton) );
-	allow_player_change.set_typ( button_t::square_state );
-	allow_player_change.add_listener( this );
-	allow_player_change.pressed = sets->get_allow_player_change();
-	add_komponente( &allow_player_change );
-	intTopOfButton += 12;
-
-	intTopOfButton += 5;
 	use_beginner_mode.set_pos( koord(10,intTopOfButton) );
 	use_beginner_mode.set_typ( button_t::square_state );
-	use_beginner_mode.add_listener( this );
 	use_beginner_mode.pressed = sets->get_beginner_mode();
+	use_beginner_mode.add_listener( this );
 	add_komponente( &use_beginner_mode );
 	intTopOfButton += 12;
 
+	intTopOfButton += 10;
+	open_setting_gui.set_pos( koord(10,intTopOfButton) );
+	open_setting_gui.set_groesse( koord(80, 14) );
+	open_setting_gui.set_typ( button_t::roundbox );
+	open_setting_gui.set_text("Setting");
+	open_setting_gui.add_listener( this );
+	open_setting_gui.pressed = win_get_magic( magic_settings_frame_t );
+	add_komponente( &open_setting_gui );
+
+	open_climate_gui.set_pos( koord(80+20,intTopOfButton) );
+	open_climate_gui.set_groesse( koord(150, 14) );
+	open_climate_gui.set_typ( button_t::roundbox );
+	open_climate_gui.add_listener( this );
+	open_climate_gui.set_text("Climate Control");
+	open_climate_gui.pressed = win_get_magic( magic_climate );
+	add_komponente( &open_climate_gui );
+	intTopOfButton += 12;
+
 	// load game
 	intTopOfButton += 10;
 	load_game.set_pos( koord(10, intTopOfButton) );
@@ -248,8 +276,7 @@ DBG_MESSAGE("","sizeof(stat)=%d, sizeof(tm)=%d",sizeof(struct stat),sizeof(struc
  * @param filename name of heightfield file
  * @author Hajo/prissi
  */
-bool
-welt_gui_t::update_from_heightfield(const char *filename)
+bool welt_gui_t::update_from_heightfield(const char *filename)
 {
 	DBG_MESSAGE("welt_gui_t::update_from_heightfield()",filename);
 
@@ -290,8 +317,6 @@ welt_gui_t::update_from_heightfield(const char *filename)
 			}
 		}
 
-		strcpy(map_number_s,translator::translate("file"));
-
 		return true;
 	}
 	return false;
@@ -303,14 +328,13 @@ welt_gui_t::update_from_heightfield(const char *filename)
  * Berechnet Preview-Karte neu. Inititialisiert RNG neu!
  * @author Hj. Malthaner
  */
-void
-welt_gui_t::update_preview()
+void welt_gui_t::update_preview()
 {
 	const int mx = sets->get_groesse_x()/preview_size;
 	const int my = sets->get_groesse_y()/preview_size;
 
 	if(loaded_heightfield) {
-		update_from_heightfield(sets->heightfield);
+		update_from_heightfield(sets->heightfield.c_str());
 	}
 	else {
 		setsimrand( 0xFFFFFFFF, sets->get_karte_nummer() );
@@ -331,22 +355,21 @@ welt_gui_t::update_preview()
  * This method is called if an action is triggered
  * @author Hj. Malthaner
  */
-bool
-welt_gui_t::action_triggered( gui_action_creator_t *komp,value_t v)
+bool welt_gui_t::action_triggered( gui_action_creator_t *komp,value_t v)
 {
 	// check for changed map (update preview for any event)
 	int knr = inp_map_number.get_value(); //
 
 	if(komp==&inp_x_size) {
 		sets->set_groesse_x( v.i );
-		inp_x_size.set_increment_mode( v.i>=512 ? 128 : 64 );
-		inp_y_size.set_limits( 64, min(32766,16777216/sets->get_groesse_x()) );
+		inp_x_size.set_increment_mode( v.i>=64 ? (v.i>=512 ? 128 : 64) : 8 );
+		inp_y_size.set_limits( 8, min(32000,16777216/sets->get_groesse_x()) );
 		update_preview();
 	}
 	else if(komp==&inp_y_size) {
 		sets->set_groesse_y( v.i );
-		inp_y_size.set_increment_mode( v.i>=512 ? 128 : 64 );
-		inp_x_size.set_limits( 64, min(32766,16777216/sets->get_groesse_y()) );
+		inp_y_size.set_increment_mode( v.i>=64 ? (v.i>=512 ? 128 : 64) : 8 );
+		inp_x_size.set_limits( 8, min(32000,16777216/sets->get_groesse_y()) );
 		update_preview();
 	}
 	else if(komp==&inp_number_of_towns) {
@@ -359,15 +382,9 @@ welt_gui_t::action_triggered( gui_action_creator_t *komp,value_t v)
 		umgebung_t::intercity_road_length = v.i;
 		inp_intercity_road_len.set_increment_mode( v.i>=1000 ? 100 : 20 );
 	}
-	else if(komp==&inp_traffic_density) {
-		sets->set_verkehr_level( v.i );
-	}
 	else if(komp==&inp_other_industries) {
 		sets->set_land_industry_chains( v.i );
 	}
-	else if(komp==&inp_electric_producer) {
-		sets->set_electric_promille( v.i*10 );
-	}
 	else if(komp==&inp_tourist_attractions) {
 		sets->set_tourist_attractions( v.i );
 	}
@@ -384,42 +401,88 @@ welt_gui_t::action_triggered( gui_action_creator_t *komp,value_t v)
 		// load relief
 		loaded_heightfield = false;
 		sets->heightfield = "";
-		sets->set_grundwasser(-2);
+		sets->grundwasser = -2;
 		create_win(new load_relief_frame_t(sets), w_info, magic_load_t);
 		knr = sets->get_karte_nummer();	// otherwise using cancel would not show the normal generated map again
 	}
 	else if(komp==&use_intro_dates) {
-		sets->set_use_timeline( use_intro_dates.pressed^1 );
-		use_intro_dates.pressed = sets->get_use_timeline();
-	}
-	else if(komp==&allow_player_change) {
-		sets->set_allow_player_change( allow_player_change.pressed^1 );
-		allow_player_change.pressed = sets->get_allow_player_change();
+		// 0,1 should force setting to new game as well. don't allow to change
+		// 2,3 allow to change
+		if(sets->get_use_timeline()&2) {
+			// don't change bit1. bit1 affects loading saved game
+			sets->set_use_timeline( sets->get_use_timeline()^1 );
+			use_intro_dates.pressed = sets->get_use_timeline()&1;
+		}
 	}
 	else if(komp==&use_beginner_mode) {
-		sets->set_beginner_mode( use_beginner_mode.pressed^1 );
+		sets->beginner_mode = sets->get_beginner_mode()^1;
 		use_beginner_mode.pressed = sets->get_beginner_mode();
 	}
-	else if(komp==&load_game) {
-		load = true;
-	}
-	else if(komp==&load_scenario) {
-		scenario = true;
-	}
-	else if(komp==&start_game) {
-		if(loaded_heightfield) {
-			load_heightfield = true;
+	else if(komp==&open_setting_gui) {
+		gui_frame_t *sg = win_get_magic( magic_settings_frame_t );
+		if(  sg  ) {
+			destroy_win( sg );
+			open_setting_gui.pressed = false;
 		}
 		else {
-			start = true;
+			create_win(10, 40, new settings_frame_t(sets), w_info, magic_settings_frame_t );
+			open_setting_gui.pressed = true;
+		}
+	}
+	else if(komp==&open_climate_gui) {
+		gui_frame_t *climate_gui = win_get_magic( magic_climate );
+		if(  climate_gui  ) {
+			destroy_win( climate_gui );
+			open_climate_gui.pressed = false;
+		}
+		else {
+			climate_gui_t *cg = new climate_gui_t(sets);
+			create_win((display_get_width() - cg->get_fenstergroesse().x-10), 40, cg, w_info, magic_climate );
+			open_climate_gui.pressed = true;
+		}
+	}
+	else if(komp==&load_game) {
+		welt->get_message()->clear();
+		create_win( new loadsave_frame_t(welt, true), w_info, magic_load_t);
+	}
+	else if(komp==&load_scenario) {
+		char path[1024];
+		sprintf( path, "%s%sscenario/", umgebung_t::program_dir, umgebung_t::objfilename.c_str() );
+		chdir( path );
+		destroy_all_win(true);
+		welt->get_message()->clear();
+		create_win( new scenario_frame_t(welt), w_info, magic_load_t );
+		chdir( umgebung_t::user_dir );
+	}
+	else if(komp==&start_game) {
+		destroy_all_win(true);
+		welt->get_message()->clear();
+		create_win(200, 100, new news_img("Erzeuge neue Karte.\n", skinverwaltung_t::neueweltsymbol->get_bild_nr(0)), w_info, magic_none);
+		if(loaded_heightfield) {
+			welt->load_heightfield(&umgebung_t::default_einstellungen);
+		}
+		else {
+			umgebung_t::default_einstellungen.heightfield = "";
+			welt->init( &umgebung_t::default_einstellungen, 0 );
+		}
+		destroy_all_win(true);
+		welt->step_month( umgebung_t::default_einstellungen.get_starting_month() );
+		welt->set_pause(false);
+		// save setting ...
+		loadsave_t file;
+		if(file.wr_open("default.sve",loadsave_t::binary,"settings only",SAVEGAME_VER_NR)) {
+			// save default setting
+			umgebung_t::default_einstellungen.rdwr(&file);
+			file.close();
 		}
 	}
 	else if(komp==&quit_game) {
-		quit = true;
+		destroy_all_win(true);
+		umgebung_t::quit_simutrans = true;
 	}
 
 	if(knr>=0) {
-		sets->set_karte_nummer( knr );
+		sets->nummer = knr;
 		if(!loaded_heightfield) {
 			update_preview();
 		}
@@ -429,21 +492,23 @@ welt_gui_t::action_triggered( gui_action_creator_t *komp,value_t v)
 
 
 
-void welt_gui_t::infowin_event(const event_t *ev)
+bool  welt_gui_t::infowin_event(const event_t *ev)
 {
 	gui_frame_t::infowin_event(ev);
 
 	if(ev->ev_class==INFOWIN  &&  ev->ev_code==WIN_CLOSE) {
 		close = true;
 	}
+
+	return true;
 }
 
 
 
 void welt_gui_t::zeichnen(koord pos, koord gr)
 {
-	if(!loaded_heightfield  && sets->heightfield.len()!=0) {
-		if(update_from_heightfield(sets->heightfield)) {
+	if(!loaded_heightfield  && sets->heightfield.size()!=0) {
+		if(update_from_heightfield(sets->heightfield.c_str())) {
 			loaded_heightfield = true;
 		}
 		else {
@@ -459,8 +524,10 @@ void welt_gui_t::zeichnen(koord pos, koord gr)
 		load_map.set_text("Lade Relief");
 		load_map.set_tooltip("load height data from file");
 		use_intro_dates.set_text("Use timeline start year");
-		allow_player_change.set_text("Allow player change");
-		use_beginner_mode.set_text("Beginner mode");
+		use_beginner_mode.set_text("Use beginner mode");
+		use_beginner_mode.set_tooltip("Higher transport fees, crossconnect all factories");
+		open_setting_gui.set_text("Setting");
+		open_climate_gui.set_text("Climate Control");
 		load_game.set_text("Load game");
 		load_scenario.set_text("Load scenario");
 		start_game.set_text("Starte Spiel");
@@ -469,6 +536,12 @@ void welt_gui_t::zeichnen(koord pos, koord gr)
 		welt->set_dirty();
 	}
 
+	open_climate_gui.pressed = win_get_magic( magic_climate );
+	open_setting_gui.pressed = win_get_magic( magic_settings_frame_t );
+
+	use_intro_dates.pressed = sets->get_use_timeline()&1;
+	use_beginner_mode.pressed = sets->get_beginner_mode();
+
 	gui_frame_t::zeichnen(pos, gr);
 
 	char buf[256];
@@ -512,19 +585,16 @@ void welt_gui_t::zeichnen(koord pos, koord gr)
 	y += 12;
 	display_proportional_clip(x, y, translator::translate("Intercity road len:"), ALIGN_LEFT, COL_BLACK, true);
 	y += 12;
-	display_proportional_clip(x, y, translator::translate("6WORLD_CHOOSE"), ALIGN_LEFT, COL_BLACK, true);
-	y += 12+5;
-
 	display_proportional_clip(x, y, translator::translate("Land industries"), ALIGN_LEFT, COL_BLACK, true);
 	y += 12;
-	display_proportional_clip(x, y, translator::translate("Percent Electricity"), ALIGN_LEFT, COL_BLACK, true);
-	y += 12;
 	display_proportional_clip(x, y, translator::translate("Tourist attractions"), ALIGN_LEFT, COL_BLACK, true);
 	y += 12+5;
 
+	y += 12+12+5;
 	y += 12+5;
-	y += 12+5;
-	y += 12+5;
+	y += 5;
+
+	display_ddd_box_clip(x, y-22, 240, 0, MN_GREY0, MN_GREY4);
 
 	display_ddd_box_clip(x, y, 240, 0, MN_GREY0, MN_GREY4);
 }
