@@ -1,11 +1,17 @@
 
 /* standard good AI code */
 
+#include "../simfab.h"
+#include "../simmenu.h"
 #include "../simtypes.h"
+#include "../simwerkz.h"
 
 #include "simplay.h"
 
 #include "../simhalt.h"
+#include "../simintr.h"
+#include "../simline.h"
+#include "../simmesg.h"
 #include "../simtools.h"
 #include "../simworld.h"
 
@@ -30,6 +36,8 @@ ai_goods_t::ai_goods_t(karte_t *wl, uint8 nr) : ai_t(wl,nr)
 {
 	state = NR_INIT;
 
+	freight = NULL;
+
 	root = NULL;
 	start = NULL;
 	ziel = NULL;
@@ -43,7 +51,7 @@ ai_goods_t::ai_goods_t(karte_t *wl, uint8 nr) : ai_t(wl,nr)
 	ship_vehicle = NULL;
 	road_weg = NULL;
 
-	next_contruction_steps = welt->get_steps()+simrand(400);
+	next_contruction_steps = welt->get_steps()+ 50;
 
 	road_transport = nr<7;
 	rail_transport = nr>2;
@@ -61,10 +69,10 @@ void ai_goods_t::neues_jahr()
 {
 	spieler_t::neues_jahr();
 
-	// AI will reconsider the oldes unbuiltable lines again
-	uint remove = (uint)max(0,(int)forbidden_conections.get_count()-3);
-	while(  remove < forbidden_conections.get_count()  ) {
-		forbidden_conections.remove_first();
+	// AI will reconsider the oldest unbuiltable lines again
+	uint remove = (uint)max(0,(int)forbidden_connections.get_count()-3);
+	while(  remove < forbidden_connections.get_count()  ) {
+		forbidden_connections.remove_first();
 	}
 }
 
@@ -163,7 +171,7 @@ bool ai_goods_t::get_factory_tree_lowest_missing( fabrik_t *fab )
 
 
 /* recursive lookup of a tree and how many factories must be at least connected
- * returns -1, if this tree is incomplete
+ * returns -1, if this tree is can't be completed
  */
 int ai_goods_t::get_factory_tree_missing_count( fabrik_t *fab )
 {
@@ -182,33 +190,45 @@ int ai_goods_t::get_factory_tree_missing_count( fabrik_t *fab )
 		const vector_tpl <koord> & sources = fab->get_suppliers();
 		for( unsigned q=0;  q<sources.get_count();  q++  ) {
 			fabrik_t *qfab = fabrik_t::get_fab(welt,sources[q]);
-			if(!fab) {
+			if(!qfab) {
 				dbg->error( "fabrik_t::get_fab()","fab %s at %s does not find supplier at %s.", fab->get_name(), fab->get_pos().get_str(), sources[q].get_str() );
 				continue;
 			}
-			const fabrik_besch_t* const fb = qfab->get_besch();
-			for (uint qq = 0; qq < fb->get_produkte(); qq++) {
-				if (fb->get_produkt(qq)->get_ware() == ware && !is_forbidden( fabrik_t::get_fab(welt,sources[q]), fab, ware)) {
-					int n = get_factory_tree_missing_count( qfab );
-					if(n>=0) {
-						complete = true;
-						if(  !is_connected( sources[q], fab->get_pos().get_2d(), ware )  ) {
-							numbers += 1;
+			if( !is_forbidden( qfab, fab, ware ) ) {
+				const fabrik_besch_t* const fb = qfab->get_besch();
+				for (uint qq = 0; qq < fb->get_produkte(); qq++) {
+					if (fb->get_produkt(qq)->get_ware() == ware ) {
+						int n = get_factory_tree_missing_count( qfab );
+						if(n>=0) {
+							complete = true;
+							if(  !is_connected( sources[q], fab->get_pos().get_2d(), ware )  ) {
+								numbers += 1;
+							}
+							numbers += n;
 						}
-						numbers += n;
 					}
 				}
 			}
 		}
 		if(!complete) {
-			if(fab->get_besch()->get_lieferanten()==0  ||  numbers==0) {
-				return -1;
-			}
+			return -1;
 		}
 	}
 	return numbers;
 }
 
+void add_neighbourhood( vector_tpl<koord> &list, const uint16 size)
+{
+	uint32 old_size = list.get_count();
+	koord test;
+	for( uint32 i = 0; i < old_size; i++ ) {
+		for( test.x = -size; test.x < size+1; test.x++ ) {
+			for( test.y = -size; test.y < size+1; test.y++ ) {
+				list.append_unique( list[i] + test );
+			}
+		}
+	}
+}
 
 
 bool ai_goods_t::suche_platz1_platz2(fabrik_t *qfab, fabrik_t *zfab, int length )
@@ -222,17 +242,81 @@ bool ai_goods_t::suche_platz1_platz2(fabrik_t *qfab, fabrik_t *zfab, int length
 	koord ziel_size( length, 0 );
 
 	bool ok = false;
+	bool has_ziel = false;
 
 	if(qfab->get_besch()->get_platzierung()!=fabrik_besch_t::Wasser) {
-		ok = suche_platz(start, start_size, ziel, qfab->get_besch()->get_haus()->get_groesse() );
+		if( length == 0 ) {
+			vector_tpl<koord3d> tile_list[2];
+			const uint16 cov = welt->get_einstellungen()->get_station_coverage();
+			koord test;
+			for( uint8 i = 0; i < 2; i++ ) {
+				fabrik_t *fab =  i==0 ? qfab : zfab;
+				vector_tpl<koord> fab_tiles;
+				fab->get_tile_list( fab_tiles );
+				add_neighbourhood( fab_tiles, cov );
+				vector_tpl<koord> one_more( fab_tiles );
+				add_neighbourhood( one_more, 1 );
+				// Any halts here?
+				vector_tpl<koord> halts;
+				for( uint32 j = 0; j < one_more.get_count(); j++ ) {
+					halthandle_t halt = haltestelle_t::get_halt( welt, one_more[j], this );
+					if( halt.is_bound() && !halts.is_contained(halt->get_basis_pos()) ) {
+						bool halt_connected = halt->get_fab_list().is_contained( fab );
+						for( slist_tpl< haltestelle_t::tile_t >::const_iterator iter = halt->get_tiles().begin(); iter != halt->get_tiles().end(); ++iter ) {
+							koord pos = iter->grund->get_pos().get_2d();
+							if( halt_connected || fab_tiles.is_contained(pos) ) {
+								halts.append_unique( pos );
+							}
+						}
+					}
+				}
+				add_neighbourhood( halts, 1 );
+				vector_tpl<koord> *next = &halts;
+				for( uint8 k = 0; k < 2; k++ ) {
+					// On which tiles we can start?
+					for( uint32 j = 0; j < next->get_count(); j++ ) {
+						const grund_t* gr = welt->lookup_kartenboden( next->operator[](j) );
+						if(  gr  &&  gr->get_grund_hang() == hang_t::flach  &&  !gr->hat_wege()  &&  !gr->get_leitung()  ) {
+							tile_list[i].append_unique( gr->get_pos() );
+						}
+					}
+					if( !tile_list[i].empty() ) {
+						// Skip, if found tiles beneath halts.
+						break;
+					}
+					next = &fab_tiles;
+				}
+			}
+			// Test which tiles are the best:
+			wegbauer_t bauigel(welt, this);
+			bauigel.route_fuer( wegbauer_t::strasse, road_weg, tunnelbauer_t::find_tunnel(road_wt,road_weg->get_topspeed(),welt->get_timeline_year_month()), brueckenbauer_t::find_bridge(road_wt,road_weg->get_topspeed(),welt->get_timeline_year_month()) );
+			// we won't destroy cities (and save the money)
+			bauigel.set_keep_existing_faster_ways(true);
+			bauigel.set_keep_city_roads(true);
+			bauigel.set_maximum(10000);
+			bauigel.calc_route(tile_list[0], tile_list[1]);
+			if(  bauigel.get_count() > 2  ) {
+				// Sometimes reverse route is the best, so we have to change the koords.
+				koord3d_vector_t const& r = bauigel.get_route();
+				start = r.front().get_2d();
+				ziel  = r.back().get_2d();
+				if (!tile_list[0].is_contained(r.front())) sim::swap(start, ziel);
+				ok = true;
+				has_ziel = true;
+			}
+		}
+		if( !ok ) {
+			ok = suche_platz(start, start_size, ziel, qfab->get_besch()->get_haus()->get_groesse(qfab->get_rotate()) );
+		}
 	}
 	else {
 		// water factory => find harbour location
 		ok = find_harbour(start, start_size, ziel );
 	}
-	if(ok) {
+
+	if( ok && !has_ziel ) {
 		// found a place, search for target
-		ok = suche_platz(ziel, ziel_size, start, zfab->get_besch()->get_haus()->get_groesse() );
+		ok = suche_platz(ziel, ziel_size, start, zfab->get_besch()->get_haus()->get_groesse(zfab->get_rotate()) );
 	}
 
 	INT_CHECK("simplay 1729");
@@ -281,6 +365,11 @@ bool ai_goods_t::create_ship_transport_vehikel(fabrik_t *qfab, int anz_vehikel)
 	}
 	DBG_MESSAGE( "ai_goods_t::create_ship_transport_vehikel()", "for %i ships", anz_vehikel );
 
+	if(  convoihandle_t::is_exhausted()  ) {
+		// too many convois => cannot do anything about this ...
+		return false;
+	}
+
 	// must remove marker
 	grund_t* gr = welt->lookup_kartenboden(platz1);
 	if (gr) gr->obj_loesche_alle(this);
@@ -291,20 +380,15 @@ bool ai_goods_t::create_ship_transport_vehikel(fabrik_t *qfab, int anz_vehikel)
 	}
 
 	// sea pos (and not on harbour ... )
-	halthandle_t halt = haltestelle_t::get_halt(welt,platz1);
+	halthandle_t halt = haltestelle_t::get_halt(welt,platz1,this);
 	koord pos1 = platz1 - koord(gr->get_grund_hang())*h->get_groesse().y;
 	koord best_pos = pos1;
 	for(  int y = pos1.y-welt->get_einstellungen()->get_station_coverage();  y<=pos1.y+welt->get_einstellungen()->get_station_coverage();  y++  ) {
 		for(  int x = pos1.x-welt->get_einstellungen()->get_station_coverage();  x<=pos1.x+welt->get_einstellungen()->get_station_coverage();  x++  ) {
 			koord p(x,y);
-			const planquadrat_t *plan = welt->lookup(p);
-			if(plan) {
-				grund_t *gr = plan->get_kartenboden();
-				if(  gr->ist_wasser()  &&  !gr->get_halt().is_bound()  ) {
-					if(plan->get_haltlist_count()>=1  &&  plan->get_haltlist()[0]==halt  &&  abs_distance(best_pos,platz2)<abs_distance(p,platz2)) {
-						best_pos = p;
-					}
-				}
+			// in water, the water tiles have no halt flag!
+			if(welt->ist_in_kartengrenzen(p)  &&  !welt->lookup(p)->get_halt().is_bound()  &&  halt == haltestelle_t::get_halt(welt,p,this)  &&  koord_distance(best_pos,platz2)<koord_distance(p,platz2)  ) {
+				best_pos = p;
 			}
 		}
 	}
@@ -319,11 +403,16 @@ bool ai_goods_t::create_ship_transport_vehikel(fabrik_t *qfab, int anz_vehikel)
 	fpl->append( welt->lookup_kartenboden(best_pos), 0 );
 	fpl->append( welt->lookup(qfab->get_pos()), 100 );
 	fpl->set_aktuell( 1 );
+	fpl->eingabe_abschliessen();
 	linehandle_t line=simlinemgmt.create_line(simline_t::shipline,this,fpl);
 	delete fpl;
 
 	// now create all vehicles as convois
 	for(int i=0;  i<anz_vehikel;  i++) {
+		if(  convoihandle_t::is_exhausted()  ) {
+			// too many convois => cannot do anything about this ...
+			return i>0;
+		}
 		vehikel_t* v = vehikelbauer_t::baue( qfab->get_pos(), this, NULL, ship_vehicle);
 		convoi_t* cnv = new convoi_t(this);
 		// V.Meyer: give the new convoi name from first vehicle
@@ -356,8 +445,8 @@ void ai_goods_t::create_road_transport_vehikel(fabrik_t *qfab, int anz_vehikel)
 	const haus_besch_t* fh = hausbauer_t::get_random_station(haus_besch_t::generic_stop, road_wt, welt->get_timeline_year_month(), haltestelle_t::WARE);
 	// succeed in frachthof creation
 	if(fh  &&  call_general_tool(WKZ_STATION, platz1, fh->get_name())  &&  call_general_tool(WKZ_STATION, platz2, fh->get_name())  ) {
-		koord3d pos1 = welt->lookup(platz1)->get_kartenboden()->get_pos();
-		koord3d pos2 = welt->lookup(platz2)->get_kartenboden()->get_pos();
+		koord3d pos1 = welt->lookup_kartenboden(platz1)->get_pos();
+		koord3d pos2 = welt->lookup_kartenboden(platz2)->get_pos();
 
 		int	start_location=0;
 		// sometimes, when factories are very close, we need exakt calculation
@@ -371,18 +460,23 @@ void ai_goods_t::create_road_transport_vehikel(fabrik_t *qfab, int anz_vehikel)
 		koord3d startpos=(start_location==0)?pos1:pos2;
 		ribi_t::ribi w_ribi = welt->lookup(startpos)->get_weg_ribi_unmasked(road_wt);
 		// now start all vehicle one field before, so they load immediately
-		startpos = welt->lookup(koord(startpos.get_2d())+koord(w_ribi))->get_kartenboden()->get_pos();
+		startpos = welt->lookup_kartenboden(koord(startpos.get_2d())+koord(w_ribi))->get_pos();
 
 		// since 86.01 we use lines for road vehicles ...
 		schedule_t *fpl=new autofahrplan_t();
 		fpl->append(welt->lookup(pos1), start_location == 0 ? 100 : 0);
 		fpl->append(welt->lookup(pos2), start_location == 1 ? 100 : 0);
 		fpl->set_aktuell( start_location );
+		fpl->eingabe_abschliessen();
 		linehandle_t line=simlinemgmt.create_line(simline_t::truckline,this,fpl);
 		delete fpl;
 
 		// now create all vehicles as convois
 		for(int i=0;  i<anz_vehikel;  i++) {
+			if(  convoihandle_t::is_exhausted()  ) {
+				// too many convois => cannot do anything about this ...
+				return;
+			}
 			vehikel_t* v = vehikelbauer_t::baue(startpos, this, NULL, road_vehicle);
 			convoi_t* cnv = new convoi_t(this);
 			// V.Meyer: give the new convoi name from first vehicle
@@ -404,22 +498,28 @@ void ai_goods_t::create_road_transport_vehikel(fabrik_t *qfab, int anz_vehikel)
 void ai_goods_t::create_rail_transport_vehikel(const koord platz1, const koord platz2, int anz_vehikel, int ladegrad)
 {
 	schedule_t *fpl;
+	if(  convoihandle_t::is_exhausted()  ) {
+		// too many convois => cannot do anything about this ...
+		return;
+	}
 	convoi_t* cnv = new convoi_t(this);
-	koord3d pos1= welt->lookup(platz1)->get_kartenboden()->get_pos();
-	koord3d pos2 = welt->lookup(platz2)->get_kartenboden()->get_pos();
+	koord3d pos1= welt->lookup_kartenboden(platz1)->get_pos();
+	koord3d pos2 = welt->lookup_kartenboden(platz2)->get_pos();
 
 	// probably need to electrify the track?
 	if(  rail_engine->get_engine_type()==vehikel_besch_t::electric  ) {
 		// we need overhead wires
 		const way_obj_besch_t *e = wayobj_t::wayobj_search(track_wt,overheadlines_wt,welt->get_timeline_year_month());
 		wkz_wayobj_t wkz;
-		wkz.default_param = e->get_name();
+		wkz.set_default_param(e->get_name());
 		wkz.init( welt, this );
 		wkz.work( welt, this, welt->lookup_kartenboden(platz1)->get_pos() );
 		wkz.work( welt, this, welt->lookup_kartenboden(platz2)->get_pos() );
 		wkz.exit( welt, this );
 	}
-	vehikel_t* v = vehikelbauer_t::baue(pos2, this, NULL, rail_engine);
+
+	koord3d start_pos = welt->lookup_kartenboden(pos1.get_2d() + (abs(size1.x)>abs(size1.y) ? koord(size1.x,0) : koord(0,size1.y)))->get_pos();
+	vehikel_t* v = vehikelbauer_t::baue( start_pos, this, NULL, rail_engine);
 
 	// V.Meyer: give the new convoi name from first vehicle
 	cnv->set_name(rail_engine->get_name());
@@ -432,15 +532,16 @@ void ai_goods_t::create_rail_transport_vehikel(const koord platz1, const koord p
 	 */
 	for(int i = 0; i < anz_vehikel; i++) {
 		// use the vehicle we searched before
-		vehikel_t* v = vehikelbauer_t::baue(pos2, this, NULL, rail_vehicle);
+		vehikel_t* v = vehikelbauer_t::baue(start_pos, this, NULL, rail_vehicle);
 		cnv->add_vehikel( v );
 	}
 
-	fpl = cnv->get_vehikel(0)->erzeuge_neuen_fahrplan();
+	fpl = cnv->front()->erzeuge_neuen_fahrplan();
 
 	fpl->set_aktuell( 0 );
 	fpl->append(welt->lookup(pos1), ladegrad);
 	fpl->append(welt->lookup(pos2), 0);
+	fpl->eingabe_abschliessen();
 
 	cnv->set_schedule(fpl);
 	welt->sync_add( cnv );
@@ -489,7 +590,12 @@ int ai_goods_t::baue_bahnhof(const koord* p, int anz_vehikel)
 	}
 	koord pos;
 	for(  pos=t-zv;  pos!=*p;  pos-=zv ) {
-		if(  make_all_bahnhof  ||  is_my_halt(pos+koord(-1,-1))  ||  is_my_halt(pos+koord(-1,1))  ||  is_my_halt(pos+koord(1,-1))  ||  is_my_halt(pos+koord(1,1))  ) {
+		if(  make_all_bahnhof  ||
+			haltestelle_t::get_halt(welt,pos+koord(-1,-1),this).is_bound()  ||
+			haltestelle_t::get_halt(welt,pos+koord(-1, 1),this).is_bound()  ||
+			haltestelle_t::get_halt(welt,pos+koord( 1,-1),this).is_bound()  ||
+			haltestelle_t::get_halt(welt,pos+koord( 1, 1),this).is_bound()
+		) {
 			// start building, if next to an existing station
 			make_all_bahnhof = true;
 			call_general_tool( WKZ_STATION, pos, besch->get_name() );
@@ -498,7 +604,7 @@ int ai_goods_t::baue_bahnhof(const koord* p, int anz_vehikel)
 	}
 	// now add the other squares (going backwards)
 	for(  pos=*p;  pos!=t;  pos+=zv ) {
-		if(  !is_my_halt(pos)  ) {
+		if(  !haltestelle_t::get_halt(welt,pos,this).is_bound()  ) {
 			call_general_tool( WKZ_STATION, pos, besch->get_name() );
 		}
 	}
@@ -519,24 +625,20 @@ bool ai_goods_t::create_simple_rail_transport()
 	clean_marker(platz1,size1);
 	clean_marker(platz2,size2);
 
+	wegbauer_t bauigel(welt, this);
+	bauigel.route_fuer( (wegbauer_t::bautyp_t)(wegbauer_t::schiene|wegbauer_t::bot_flag), rail_weg, tunnelbauer_t::find_tunnel(track_wt,rail_engine->get_geschw(),welt->get_timeline_year_month()), brueckenbauer_t::find_bridge(track_wt,rail_engine->get_geschw(),welt->get_timeline_year_month()) );
+	bauigel.set_keep_existing_ways(false);
+
 	bool ok=true;
 	// first: make plain stations tiles as intended
 	sint8 z1 = max( welt->get_grundwasser()+Z_TILE_STEP, welt->lookup_kartenboden(platz1)->get_hoehe() );
 	koord k = platz1;
 	koord diff1( sgn(size1.x), sgn(size1.y) );
 	koord perpend( sgn(size1.y), sgn(size1.x) );
-	ribi_t::ribi ribi1 = ribi_typ( diff1 );
 	while(k!=size1+platz1) {
 		if(!welt->ebne_planquadrat( this, k, z1 )) {
-			ok = false;
-			break;
+			return false;
 		}
-		grund_t *gr = welt->lookup_kartenboden(k);
-		weg_t *sch = weg_t::alloc(track_wt);
-		sch->set_besch( rail_weg );
-		int cost = -gr->neuen_weg_bauen(sch, ribi1, this) - rail_weg->get_preis();
-		buche(cost, k, COST_CONSTRUCTION);
-		ribi1 = ribi_t::doppelt( ribi1 );
 		k += diff1;
 	}
 
@@ -545,66 +647,52 @@ bool ai_goods_t::create_simple_rail_transport()
 	k = platz2;
 	perpend = koord( sgn(size2.y), sgn(size2.x) );
 	koord diff2( sgn(size2.x), sgn(size2.y) );
-	ribi_t::ribi ribi2 = ribi_typ( diff2 );
 	while(k!=size2+platz2  &&  ok) {
 		if(!welt->ebne_planquadrat(this,k,z2)) {
-			ok = false;
-			break;
+			return false;
 		}
-		grund_t *gr = welt->lookup_kartenboden(k);
-		weg_t *sch = weg_t::alloc(track_wt);
-		sch->set_besch( rail_weg );
-		int cost = -gr->neuen_weg_bauen(sch, ribi2, this) - rail_weg->get_preis();
-		buche(cost, k, COST_CONSTRUCTION);
-		ribi2 = ribi_t::doppelt( ribi2 );
 		k += diff2;
 	}
 
+	bauigel.calc_route( koord3d(platz1,z1), koord3d(platz1+size1-diff1, z1));
+	bauigel.baue();
+	bauigel.calc_route( koord3d(platz2,z2), koord3d(platz2+size2-diff2, z2));
+	bauigel.baue();
+
+	vector_tpl<koord3d> starttiles, endtiles;
 	// now calc the route
-	wegbauer_t bauigel(welt, this);
 	if(ok) {
-		bauigel.route_fuer( (wegbauer_t::bautyp_t)(wegbauer_t::schiene|wegbauer_t::bot_flag), rail_weg, tunnelbauer_t::find_tunnel(track_wt,rail_engine->get_geschw(),welt->get_timeline_year_month()), brueckenbauer_t::find_bridge(track_wt,rail_engine->get_geschw(),welt->get_timeline_year_month()) );
-		bauigel.set_keep_existing_ways(false);
-		bauigel.calc_route( koord3d(platz2+size2,z2), koord3d(platz1+size1,z1) );
+		starttiles.append(welt->lookup_kartenboden(platz1 + size1)->get_pos());
+		starttiles.append(welt->lookup_kartenboden(platz1 - diff1)->get_pos());
+		endtiles.append(welt->lookup_kartenboden(platz2 + size2)->get_pos());
+		endtiles.append(welt->lookup_kartenboden(platz2 - diff2)->get_pos());
+		bauigel.calc_route( starttiles, endtiles );
 		INT_CHECK("simplay 2478");
 	}
 
-	if(ok  &&  bauigel.max_n > 3) {
-#if 0
-/* FIX THIS! */
-		//just check, if I could not start at the other end of the station ...
-		int start=0, end=bauigel.max_n;
-		for( int j=1;  j<bauigel.max_n-1;  j++  ) {
-			if(bauigel.get_route_bei(j)==platz2-diff2) {
-				start = j;
-				platz2 = platz2+size2-diff2;
-				size2 = size2*(-1);
-				diff2 = diff2*(-1);
-			}
-			if(bauigel.get_route_bei(j)==platz1-diff1) {
-				end = j;
-				platz1 = platz1+size1-diff1;
-				size1 = size1*(-1);
-				diff1 = diff1*(-1);
-			}
-		}
-		// so found shorter route?
-		if(start!=0  ||  end!=bauigel.max_n) {
-			bauigel.calc_route( koord3d(platz2+size2,z2), koord3d(platz1+size1,z1) );
-		}
-#endif
-
+	if(ok  &&  bauigel.get_count() > 4) {
 DBG_MESSAGE("ai_goods_t::create_simple_rail_transport()","building simple track from %d,%d to %d,%d",platz1.x, platz1.y, platz2.x, platz2.y);
 		bauigel.baue();
 		// connect to track
-		ribi1 = ribi_typ(diff1);
-		assert( welt->lookup_kartenboden(platz1+size1-diff1)->weg_erweitern(track_wt, ribi1) );
-		ribi1 = ribi_t::rueckwaerts(ribi1);
-		assert( welt->lookup_kartenboden(platz1+size1)->weg_erweitern(track_wt, ribi1) );
-		ribi2 = ribi_typ(diff2);
-		assert( welt->lookup_kartenboden(platz2+size2-diff2)->weg_erweitern(track_wt, ribi2) );
-		ribi2 = ribi_t::rueckwaerts(ribi2);
-		assert( welt->lookup_kartenboden(platz2+size2)->weg_erweitern(track_wt, ribi2) );
+
+		koord3d_vector_t const& r     = bauigel.get_route();
+		koord3d                 tile1 = r.front();
+		koord3d                 tile2 = r.back();
+		if (!starttiles.is_contained(tile1)) sim::swap(tile1, tile2);
+		// No botflag, since we want to connect with the station.
+		bauigel.route_fuer( wegbauer_t::schiene, rail_weg, tunnelbauer_t::find_tunnel(track_wt,rail_engine->get_geschw(),welt->get_timeline_year_month()), brueckenbauer_t::find_bridge(track_wt,rail_engine->get_geschw(),welt->get_timeline_year_month()) );
+		bauigel.calc_straight_route( koord3d(platz1,z1), tile1);
+		bauigel.baue();
+		bauigel.calc_straight_route( koord3d(platz2,z2), tile2);
+		bauigel.baue();
+		// If connection is built not at platz1/2, we must alter platz1/2, otherwise baue_bahnhof gets confused.
+		if(  tile1.get_2d() != platz1 + size1  ) {
+			platz1 = platz1 + size1 - diff1;
+			size1 = -size1;
+		}
+		if(  tile2.get_2d() != platz2 + size2  ) {
+			platz2 = platz2 + size2 - diff2;
+		}
 		return true;
 	}
 	else {
@@ -651,6 +739,7 @@ void ai_goods_t::step()
 	switch(state) {
 
 		case NR_INIT:
+
 			state = NR_SAMMLE_ROUTEN;
 			count = 0;
 			built_update_headquarter();
@@ -689,7 +778,7 @@ void ai_goods_t::step()
 				}
 				else {
 					// add to impossible connections
-					forbidden_conections.append( fabconnection_t( start, ziel, freight ) );
+					forbidden_connections.append( new fabconnection_t( start, ziel, freight ) );
 					state = CHECK_CONVOI;
 				}
 			}
@@ -714,12 +803,11 @@ void ai_goods_t::step()
 			 * a suitable car (and engine)
 			 * a suitable weg
 			 */
-			koord zv = start->get_pos().get_2d() - ziel->get_pos().get_2d();
-			int dist = abs(zv.x) + abs(zv.y);
+			uint32 dist = koord_distance( start->get_pos(), ziel->get_pos() );
 
 			// guess the "optimum" speed (usually a little too low)
-			uint32 best_rail_speed = 80;// is ok enough for goods, was: min(60+freight->get_speed_bonus()*5, 140 );
-			uint32 best_road_speed = min(60+freight->get_speed_bonus()*5, 130 );
+			sint32 best_rail_speed = 80;// is ok enough for goods, was: min(60+freight->get_speed_bonus()*5, 140 );
+			sint32 best_road_speed = min(60+freight->get_speed_bonus()*5, 130 );
 
 			// obey timeline
 			uint month_now = (welt->use_timeline() ? welt->get_current_month() : 0);
@@ -759,15 +847,15 @@ DBG_MESSAGE("do_ki()","road vehicle %p",road_vehicle);
 				start_ware++;
 			}
 			assert(  start_ware<ausgang.get_count()  );
-			const int prod = min(ziel->get_base_production(),
-			                 ( start->get_base_production() * start->get_besch()->get_produkt(start_ware)->get_faktor() )/256u - start->get_abgabe_letzt(start_ware) );
+			const int prod = min((uint32)ziel->get_base_production(),
+			                 ( start->get_base_production() * start->get_besch()->get_produkt(start_ware)->get_faktor() )/256u - (uint32)start->get_abgabe_letzt(start_ware) );
 
 DBG_MESSAGE("do_ki()","check railway");
 			/* calculate number of cars for railroad */
 			count_rail=255;	// no cars yet
 			if(  rail_vehicle!=NULL  ) {
 				// if our car is faster: well use slower speed to save money
-			 	best_rail_speed = min(51,rail_vehicle->get_geschw());
+				best_rail_speed = min(51, rail_vehicle->get_geschw());
 				// for engine: gues number of cars
 				count_rail = (prod*dist) / (rail_vehicle->get_zuladung()*best_rail_speed)+1;
 				// assume the engine weight 100 tons for power needed calcualtion
@@ -844,7 +932,7 @@ DBG_MESSAGE("ai_goods_t::do_ki()","No roadway possible.");
 			if(  count_road<255  ) {
 				// for short distance: reduce number of cars
 				// calculated here, since the above number was based on production
-				count_road = CLIP( (dist*15)/best_road_speed, 2, count_road );
+				count_road = CLIP( (dist*15u)/best_road_speed, 2, count_road );
 				int freight_price = (freight->get_preis()*road_vehicle->get_zuladung()*count_road)/24*((8000+(best_road_speed-80)*freight->get_speed_bonus())/1000);
 				cost_road = road_weg->get_wartung() + 300/dist + (count_road*road_vehicle->get_betriebskosten()*best_road_speed)/(2*dist+5);
 				income_road = (freight_price*best_road_speed)/(2*dist+5);
@@ -864,17 +952,17 @@ DBG_MESSAGE("ai_goods_t::do_ki()","No roadway possible.");
 					}
 				}
 				// if state is still NR_BAUE_ROUTE1 then there are no sutiable places
-				if(state==NR_BAUE_ROUTE1  &&  suche_platz1_platz2(start, ziel, 1)) {
+				if(state==NR_BAUE_ROUTE1  &&  (count_road != 255)  &&  suche_platz1_platz2(start, ziel, 0)) {
 					// rail was too expensive or not successfull
 					count_rail = 255;
 					state = ship_vehicle ? NR_BAUE_WATER_ROUTE : NR_BAUE_STRASSEN_ROUTE;
-					next_contruction_steps = 80;
+					next_contruction_steps += 80;
 				}
 			}
 			// no success at all?
 			if(state==NR_BAUE_ROUTE1) {
 				// maybe this route is not builtable ... add to forbidden connections
-				forbidden_conections.append( fabconnection_t( start, ziel, freight ) );
+				forbidden_connections.append( new fabconnection_t( start, ziel, freight ) );
 				ziel = NULL;	// otherwise it may always try to built the same route!
 				state = CHECK_CONVOI;
 			}
@@ -900,13 +988,13 @@ DBG_MESSAGE("ai_goods_t::do_ki()","No roadway possible.");
 					state = CHECK_CONVOI;
 					break;
 				}
-				int ships_needed = 1 + (prod*abs_distance(harbour,start->get_pos().get_2d())) / (ship_vehicle->get_zuladung()*max(20,ship_vehicle->get_geschw()));
+				int ships_needed = 1 + (prod*koord_distance(harbour,start->get_pos().get_2d())) / (ship_vehicle->get_zuladung()*max(20,ship_vehicle->get_geschw()));
 				if(create_ship_transport_vehikel(start,ships_needed)) {
 					if(welt->lookup(harbour)->get_halt()->get_fab_list().is_contained(ziel)) {
 						// so close, so we are already connected
 						grund_t *gr = welt->lookup_kartenboden(platz2);
 						if (gr) gr->obj_loesche_alle(this);
-						state = NR_ROAD_SUCCESS;
+						state = (rail_vehicle  &&  count_rail<255) ? NR_RAIL_SUCCESS : NR_ROAD_SUCCESS;
 					}
 					else {
 						// else we need to built the second part of the route
@@ -934,8 +1022,8 @@ DBG_MESSAGE("ai_goods_t::do_ki()","No roadway possible.");
 				if(count_rail>=3) {
 					if(count_rail<org_count_rail) {
 						// rethink engine
-					 	int best_rail_speed = min(51,rail_vehicle->get_geschw());
-					  	// obey timeline
+						int best_rail_speed = min(51, rail_vehicle->get_geschw());
+						// obey timeline
 						uint month_now = (welt->use_timeline() ? welt->get_current_month() : 0);
 						// for engine: gues number of cars
 						long power_needed=(long)(((best_rail_speed*best_rail_speed)/2500.0+1.0)*(100.0+count_rail*(rail_vehicle->get_gewicht()+rail_vehicle->get_zuladung()*freight->get_weight_per_unit()*0.001)));
@@ -953,15 +1041,26 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 					char param[16];
 					sprintf( param, "%i", track_wt );
 					wkz_wayremover_t wkz;
-					wkz.default_param = param;
+					wkz.set_default_param(param);
 					wkz.init( welt, this );
 					wkz.work( welt, this, welt->lookup_kartenboden(platz1)->get_pos() );
 					wkz.work( welt, this, welt->lookup_kartenboden(platz2)->get_pos() );
 					wkz.exit( welt, this );
+					if( (count_road != 255) && suche_platz1_platz2(start, ziel, 0) ) {
+						state = NR_BAUE_STRASSEN_ROUTE;
+					}
+					else {
+						state = NR_BAUE_CLEAN_UP;
+					}
 				}
 			}
 			else {
-				state = NR_BAUE_STRASSEN_ROUTE;
+				if( (count_road != 255) && suche_platz1_platz2(start, ziel, 0) ) {
+					state = NR_BAUE_STRASSEN_ROUTE;
+				}
+				else {
+					state = NR_BAUE_CLEAN_UP;
+				}
 			}
 		break;
 
@@ -982,12 +1081,14 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 		// remove marker etc.
 		case NR_BAUE_CLEAN_UP:
 		{
-			forbidden_conections.append( fabconnection_t( start, ziel, freight ) );
+			if(start!=NULL  &&  ziel!=NULL) {
+				forbidden_connections.append( new fabconnection_t( start, ziel, freight ) );
+			}
 			if(ship_vehicle) {
 				// only here, if we could built ships but no connection
 				halthandle_t start_halt;
 				for( int r=0;  r<4;  r++  ) {
-					start_halt = haltestelle_t::get_halt(welt,platz1+koord::nsow[r]);
+					start_halt = haltestelle_t::get_halt(welt,platz1+koord::nsow[r],this);
 					if(start_halt.is_bound()  &&  (start_halt->get_station_type()&haltestelle_t::dock)!=0) {
 						// delete all ships on this line
 						vector_tpl<linehandle_t> lines;
@@ -995,13 +1096,16 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 						if(!lines.empty()) {
 							linehandle_t line = lines.back();
 							schedule_t *fpl=line->get_schedule();
-							if(fpl->get_count()>1  &&  haltestelle_t::get_halt(welt,fpl->eintrag[0].pos)==start_halt) {
+							if(fpl->get_count()>1  &&  haltestelle_t::get_halt(welt,fpl->eintrag[0].pos,this)==start_halt) {
 								while(line->count_convoys()>0) {
-									line->get_convoy(0)->self_destruct();
-									line->get_convoy(0)->step();
+									convoihandle_t cnv = line->get_convoy(0);
+									cnv->self_destruct();
+									if(cnv.is_bound()) {
+										cnv->step();
+									}
 								}
+								simlinemgmt.delete_line( line );
 							}
-							simlinemgmt.delete_line( line );
 						}
 						// delete harbour
 						call_general_tool( WKZ_REMOVER, platz1+koord::nsow[r], NULL );
@@ -1052,16 +1156,16 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 
 			for( int i = welt->get_convoi_count()-1;  i>=0;  i--  ) {
 				const convoihandle_t cnv = welt->get_convoi(i);
-				if(cnv->get_besitzer()!=this) {
+				if(!cnv.is_bound()  ||  cnv->get_besitzer()!=this) {
 					continue;
 				}
 
-				if(cnv->get_vehikel(0)->get_waytype()==water_wt) {
+				if (cnv->front()->get_waytype() == water_wt) {
 					// ships will be only deleted together with the connecting convoi
 					continue;
 				}
 
-				long gewinn = 0;
+				sint64 gewinn = 0;
 				for( int j=0;  j<12;  j++  ) {
 					gewinn += cnv->get_finance_history( j, CONVOI_PROFIT );
 				}
@@ -1072,7 +1176,7 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 
 				// check for empty vehicles (likely stucked) that are making no plus and remove them ...
 				// take care, that the vehicle is old enough ...
-				if(!delete_this  &&  (welt->get_current_month()-cnv->get_vehikel(0)->get_insta_zeit())>6  &&  gewinn<=0  ){
+				if (!delete_this && (welt->get_current_month() - cnv->front()->get_insta_zeit()) > 6 && gewinn <= 0) {
 					sint64 goods=0;
 					// no goods for six months?
 					for( int i=0;  i<6;  i ++) {
@@ -1083,7 +1187,7 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 
 				// well, then delete this (likely stucked somewhere) or insanely unneeded
 				if(delete_this) {
-					waytype_t wt = cnv->get_vehikel(0)->get_besch()->get_waytype();
+					waytype_t const wt = cnv->front()->get_besch()->get_waytype();
 					linehandle_t line = cnv->get_line();
 					DBG_MESSAGE("ai_goods_t::do_ki()","%s retires convoi %s!", get_name(), cnv->get_name());
 
@@ -1095,12 +1199,14 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 					}
 
 					cnv->self_destruct();
-					cnv->step();	// to really get rid of it
+					if(cnv.is_bound()) {
+						cnv->step();	// to really get rid of it
+					}
 
 					// last vehicle on that connection (no line => railroad)
 					if(  !line.is_bound()  ||  line->count_convoys()==0  ) {
 						// check if a conncetion boat must be removed
-						halthandle_t start_halt = haltestelle_t::get_halt(welt,start_pos);
+						halthandle_t start_halt = haltestelle_t::get_halt(welt,start_pos,this);
 						if(start_halt.is_bound()  &&  (start_halt->get_station_type()&haltestelle_t::dock)!=0) {
 							// delete all ships on this line
 							vector_tpl<linehandle_t> lines;
@@ -1109,14 +1215,17 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 							for (vector_tpl<linehandle_t>::const_iterator iter2 = lines.begin(), end = lines.end(); iter2 != end; iter2++) {
 								linehandle_t line = *iter2;
 								schedule_t *fpl=line->get_schedule();
-								if(fpl->get_count()>1  &&  haltestelle_t::get_halt(welt,fpl->eintrag[0].pos)==start_halt) {
+								if(fpl->get_count()>1  &&  haltestelle_t::get_halt(welt,fpl->eintrag[0].pos,this)==start_halt) {
 									water_stop = koord( (start_pos.x+fpl->eintrag[0].pos.x)/2, (start_pos.y+fpl->eintrag[0].pos.y)/2 );
 									while(line->count_convoys()>0) {
-										line->get_convoy(0)->self_destruct();
-										line->get_convoy(0)->step();
+										convoihandle_t cnv = line->get_convoy(0);
+										cnv->self_destruct();
+										if(cnv.is_bound()) {
+											cnv->step();
+										}
 									}
+									simlinemgmt.delete_line( line );
 								}
-								simlinemgmt.delete_line( line );
 							}
 							// delete harbour
 							call_general_tool( WKZ_REMOVER, water_stop, NULL );
@@ -1127,7 +1236,7 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 						char param[16];
 						sprintf( param, "%i", track_wt );
 						wkz_wayremover_t wkz;
-						wkz.default_param = param;
+						wkz.set_default_param(param);
 						wkz.init( welt, this );
 						wkz.work( welt, this, start_pos );
 						wkz.work( welt, this, end_pos );
@@ -1141,7 +1250,7 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 							char param[16];
 							sprintf( param, "%i", wt );
 							wkz_wayremover_t wkz;
-							wkz.default_param = param;
+							wkz.set_default_param(param);
 							wkz.init( welt, this );
 							wkz.work( welt, this, start_pos );
 							if(wkz.work( welt, this, end_pos )!=NULL) {
@@ -1172,6 +1281,12 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 
 void ai_goods_t::rdwr(loadsave_t *file)
 {
+	if(  file->get_version()<102002  ) {
+		// do to an error the player was never saved correctly
+		spieler_t::rdwr(file);
+		return;
+	}
+
 	xml_tag_t t( file, "ai_goods_t" );
 
 	// first: do all the administration
@@ -1198,22 +1313,22 @@ void ai_goods_t::rdwr(loadsave_t *file)
 	}
 
 	// now save current state ...
-	file->rdwr_enum(state, " ");
+	file->rdwr_enum(state);
 	platz1.rdwr( file );
 	size1.rdwr( file );
 	platz2.rdwr( file );
 	size2.rdwr( file );
-	file->rdwr_long(count_rail, "" );
-	file->rdwr_long(count_road, "" );
-	file->rdwr_long(count, "" );
-	file->rdwr_bool( road_transport, "" );
-	file->rdwr_bool( rail_transport, "" );
-	file->rdwr_bool( ship_transport, "" );
+	file->rdwr_long(count_rail);
+	file->rdwr_long(count_road);
+	file->rdwr_long(count);
+	file->rdwr_bool(road_transport);
+	file->rdwr_bool(rail_transport);
+	file->rdwr_bool(ship_transport);
 
 	if(file->is_saving()) {
 		// save current pointers
 		sint32 delta_steps = next_contruction_steps-welt->get_steps();
-		file->rdwr_long(delta_steps, " ");
+		file->rdwr_long(delta_steps);
 		koord3d k3d = root ? root->get_pos() : koord3d::invalid;
 		k3d.rdwr(file);
 		k3d = start ? start->get_pos() : koord3d::invalid;
@@ -1240,7 +1355,7 @@ void ai_goods_t::rdwr(loadsave_t *file)
 	}
 	else {
 		// since steps in loaded game == 0
-		file->rdwr_long(next_contruction_steps, " ");
+		file->rdwr_long(next_contruction_steps);
 		// reinit current pointers
 		koord3d k3d;
 		k3d.rdwr(file);
@@ -1270,27 +1385,44 @@ void ai_goods_t::rdwr(loadsave_t *file)
 	}
 
 	// finally: forbidden connection list
-	sint32 cnt = forbidden_conections.get_count();
-	file->rdwr_long(cnt,"Fc");
+	sint32 cnt = forbidden_connections.get_count();
+	file->rdwr_long(cnt);
 	if(file->is_saving()) {
-		slist_iterator_tpl<fabconnection_t> iter(forbidden_conections);
+		slist_iterator_tpl<fabconnection_t*> iter(forbidden_connections);
 		while(  iter.next()  ) {
-			fabconnection_t fc = iter.get_current();
-			fc.rdwr(file);
+			fabconnection_t *fc = iter.get_current();
+			fc->rdwr(file);
 		}
 	}
 	else {
-		forbidden_conections.clear();
+		forbidden_connections.clear();
 		while(  cnt-->0  ) {
-			fabconnection_t fc(0,0,0);
-			fc.rdwr(file);
-			forbidden_conections.append(fc);
+			fabconnection_t *fc = new fabconnection_t(0,0,0);
+			fc->rdwr(file);
+			// @author Bernd Gabriel, Jan 01, 2010: Don't add, if fab or ware no longer in the game.
+			if (fc->fab1  &&  fc->fab2  &&  fc->ware)
+			{
+				forbidden_connections.append(fc);
+			}
+			else {
+				delete fc;
+			}
 		}
 	}
 }
 
-
-
+bool ai_goods_t::is_forbidden( fabrik_t *fab1, fabrik_t *fab2, const ware_besch_t *w ) const
+{
+	fabconnection_t fc(fab1, fab2, w);
+	slist_iterator_tpl<fabconnection_t*> iter(forbidden_connections);
+	while(  iter.next()  ) {
+		fabconnection_t *test_fc = iter.get_current();
+		if (fc == (*test_fc)) {
+			return true;
+		}
+	}
+	return false;
+}
 
 void ai_goods_t::fabconnection_t::rdwr(loadsave_t *file)
 {
@@ -1326,31 +1458,52 @@ void ai_goods_t::fabconnection_t::rdwr(loadsave_t *file)
  */
 void ai_goods_t::bescheid_vehikel_problem(convoihandle_t cnv,const koord3d ziel)
 {
-	switch(cnv->get_state()) {
+	if(  cnv->get_state() == convoi_t::NO_ROUTE  &&  this!=welt->get_active_player()  ) {
+			DBG_MESSAGE("ai_passenger_t::bescheid_vehikel_problem","Vehicle %s can't find a route to (%i,%i)!", cnv->get_name(),ziel.x,ziel.y);
+			cnv->self_destruct();
+			return;
+	}
+	spieler_t::bescheid_vehikel_problem( cnv, ziel );
+}
 
-		case convoi_t::NO_ROUTE:
-DBG_MESSAGE("ai_goods_t::bescheid_vehikel_problem","Vehicle %s can't find a route to (%i,%i)!", cnv->get_name(),ziel.x,ziel.y);
-			if(this==welt->get_active_player()) {
-				char buf[256];
-				sprintf(buf,translator::translate("Vehicle %s can't find a route!"), cnv->get_name());
-				welt->get_message()->add_message(buf, cnv->get_pos().get_2d(),message_t::convoi,PLAYER_FLAG|player_nr,cnv->get_vehikel(0)->get_basis_bild());
-			}
-			else {
-				cnv->self_destruct();
+
+/**
+ * Tells the player that a fabrik_t is going to be deleted.
+ * It could also tell, that a fab has been created, but by now the fabrikbauer_t does not.
+ * @author Bernd Gabriel, Jan 01, 2010
+ */
+void ai_goods_t::notify_factory(notification_factory_t flag, const fabrik_t* fab)
+{
+	switch(flag) {
+		// factory is going to be deleted
+		case notify_delete:
+			if (start==fab  ||  ziel==fab  ||  root==fab) {
+				root = NULL;
+				start = NULL;
+				ziel = NULL;
+				// set new state
+				if (state == NR_SAMMLE_ROUTEN  ||  state == NR_BAUE_ROUTE1) {
+					state = NR_INIT;
+				}
+				else if (state == NR_BAUE_SIMPLE_SCHIENEN_ROUTE  ||  state == NR_BAUE_STRASSEN_ROUTE  ||  state == NR_BAUE_WATER_ROUTE) {
+					state = NR_BAUE_CLEAN_UP;
+				}
+				else if (state == NR_RAIL_SUCCESS  ||  state == NR_ROAD_SUCCESS ||  state == NR_WATER_SUCCESS) {
+					state = CHECK_CONVOI;
+				}
+				for(  slist_tpl<fabconnection_t *>::iterator i=forbidden_connections.begin();  i!=forbidden_connections.end();  ) {
+					fabconnection_t *fc = *i;
+					if (fc->fab1 == fab || fc->fab2 == fab)
+					{
+						i = forbidden_connections.erase(i);
+						delete fc;
+					}
+					else {
+						++i;
+					}
+				}
 			}
 			break;
-
-		case convoi_t::WAITING_FOR_CLEARANCE_ONE_MONTH:
-		case convoi_t::CAN_START_ONE_MONTH:
-DBG_MESSAGE("ai_goods_t::bescheid_vehikel_problem","Vehicle %s stucked!", cnv->get_name(),ziel.x,ziel.y);
-			if(this==welt->get_active_player()) {
-				char buf[256];
-				sprintf(buf,translator::translate("Vehicle %s is stucked!"), cnv->get_name());
-				welt->get_message()->add_message(buf, cnv->get_pos().get_2d(),message_t::convoi,PLAYER_FLAG|player_nr,cnv->get_vehikel(0)->get_basis_bild());
-			}
-			break;
-
-		default:
-DBG_MESSAGE("ai_goods_t::bescheid_vehikel_problem","Vehicle %s, state %i!", cnv->get_name(), cnv->get_state());
+		default: ;
 	}
 }
