@@ -1,4 +1,6 @@
+#include "utils/simstring.h"
 #include "dataobj/translator.h"
+#include "dataobj/loadsave.h"
 #include "simtypes.h"
 #include "simline.h"
 #include "simhalt.h"
@@ -10,7 +12,9 @@
 #include "simlinemgmt.h"
 
 
-uint8 simline_t::convoi_to_line_catgory[MAX_CONVOI_COST]={LINE_CAPACITY, LINE_TRANSPORTED_GOODS, LINE_REVENUE, LINE_OPERATIONS, LINE_PROFIT };
+uint8 simline_t::convoi_to_line_catgory[MAX_CONVOI_COST] = {
+	LINE_CAPACITY, LINE_TRANSPORTED_GOODS, LINE_REVENUE, LINE_OPERATIONS, LINE_PROFIT, LINE_DISTANCE
+};
 
 karte_t *simline_t::welt=NULL;
 
@@ -19,13 +23,15 @@ karte_t *simline_t::welt=NULL;
 simline_t::simline_t(karte_t* welt, spieler_t* sp)
 {
 	self = linehandle_t(this);
-	sprintf( name, "(%i) %s", self.get_id(), translator::translate("Line") );
+	char printname[128];
+	sprintf( printname, "(%i) %s", self.get_id(), translator::translate("Line",welt->get_einstellungen()->get_name_language_id()) );
+	name = printname;
 	init_financial_history();
 	this->id = INVALID_LINE_ID;
 	this->welt = welt;
-	this->old_fpl = NULL;
 	this->fpl = NULL;
 	this->sp = sp;
+	withdraw = false;
 	state_color = COL_YELLOW;
 }
 
@@ -34,29 +40,21 @@ simline_t::simline_t(karte_t* welt, spieler_t* sp)
 void simline_t::set_line_id(uint32 id)
 {
 	this->id = id;
-	sprintf( name, "(%i) %s", id, translator::translate("Line") );
+	char printname[128];
+	sprintf( printname, "(%i) %s", id, translator::translate("Line",welt->get_einstellungen()->get_name_language_id()) );
+	name = printname;
 }
 
 
 
 simline_t::~simline_t()
 {
-	sint32 count = count_convoys() - 1;
-	for(  sint32 i = count;  i>=0;  i--  ) 	{
-		DBG_DEBUG("simline_t::~simline_t()", "convoi '%d' removed", i);
-		DBG_DEBUG("simline_t::~simline_t()", "convoi '%d'->fpl=%p", i, get_convoy(i)->get_schedule());
+	DBG_DEBUG("simline_t::~simline_t()", "deleting fpl=%p", fpl);
 
-		// Hajo: take care - this call will do "remove_convoi()"
-		// on our list!
-		get_convoy(i)->unset_line();
-	}
+	assert(count_convoys()==0);
 	unregister_stops();
 
-	DBG_DEBUG("simline_t::~simline_t()", "deleting fpl=%p and old_fpl=%p", fpl, old_fpl);
-
 	delete fpl;
-	delete old_fpl;
-
 	self.detach();
 
 	DBG_MESSAGE("simline_t::~simline_t()", "line %d (%p) destroyed", id, this);
@@ -73,16 +71,14 @@ void simline_t::add_convoy(convoihandle_t cnv)
 
 	// first convoi may change line type
 	if (type == trainline  &&  line_managed_convoys.empty() &&  cnv.is_bound()) {
-		if(cnv->get_vehikel(0)) {
-			// check, if needed to convert to tram line?
-			if(cnv->get_vehikel(0)->get_besch()->get_waytype()==tram_wt) {
-				type = simline_t::tramline;
-			}
-			// check, if needed to convert to monorail line?
-			if(cnv->get_vehikel(0)->get_besch()->get_waytype()==monorail_wt) {
-				type = simline_t::monorailline;
+		// check, if needed to convert to tram/monorail line
+		if (vehikel_t const* const v = cnv->front()) {
+			switch (v->get_besch()->get_waytype()) {
+				case tram_wt:     type = simline_t::tramline;     break;
 				// elevated monorail were saved with wrong coordinates for some versions.
 				// We try to recover here
+				case monorail_wt: type = simline_t::monorailline; break;
+				default:          break;
 			}
 		}
 	}
@@ -92,17 +88,11 @@ void simline_t::add_convoy(convoihandle_t cnv)
 	// what goods can this line transport?
 	bool update_schedules = false;
 	if(  cnv->get_state()!=convoi_t::INITIAL  ) {
-		// already on the road => need to add them
-		for(uint i=0;  i<cnv->get_vehikel_anzahl();  i++  ) {
-			// Only consider vehicles that really transport something
-			// this helps against routing errors through passenger
-			// trains pulling only freight wagons
-			if (cnv->get_vehikel(i)->get_fracht_max() == 0) {
-				continue;
-			}
-			const ware_besch_t *ware=cnv->get_vehikel(i)->get_fracht_typ();
-			if(ware!=warenbauer_t::nichts  &&  !goods_catg_index.is_contained(ware->get_catg_index())) {
-				goods_catg_index.append( ware->get_catg_index(), 1 );
+		const minivec_tpl<uint8> &convoys_goods = cnv->get_goods_catg_index();
+		for(  uint8 i = 0;  i < convoys_goods.get_count();  i++  ) {
+			const uint8 catg_index = convoys_goods[i];
+			if(  !goods_catg_index.is_contained( catg_index )  ) {
+				goods_catg_index.append( catg_index, 1 );
 				update_schedules = true;
 			}
 		}
@@ -141,23 +131,50 @@ void simline_t::rdwr(loadsave_t *file)
 
 	assert(fpl);
 
-	file->rdwr_str(name, sizeof(name));
+	if(  file->is_loading()  ) {
+		char name[1024];
+		file->rdwr_str(name, lengthof(name));
+		this->name = name;
+	}
+	else {
+		char name[1024];
+		tstrncpy( name, this->name.c_str(), lengthof(name) );
+		file->rdwr_str(name, lengthof(name));
+	}
+
 	if(file->get_version()<88003) {
 		sint32 dummy=id;
-		file->rdwr_long(dummy, " ");
+		file->rdwr_long(dummy);
 		id = (uint16)dummy;
 	}
 	else {
-		file->rdwr_short(id, " ");
+		file->rdwr_short(id);
 	}
 	fpl->rdwr(file);
 
 	//financial history
-	for (int j = 0; j<MAX_LINE_COST; j++) {
+	if(  file->get_version()<=102002  ) {
+		for (int j = 0; j<6; j++) {
+			for (int k = MAX_MONTHS-1; k>=0; k--) {
+				file->rdwr_longlong(financial_history[k][j]);
+			}
+		}
 		for (int k = MAX_MONTHS-1; k>=0; k--) {
-			file->rdwr_longlong(financial_history[k][j], " ");
+			financial_history[k][LINE_DISTANCE] = 0;
 		}
 	}
+	else {
+		for (int j = 0; j<MAX_LINE_COST; j++) {
+			for (int k = MAX_MONTHS-1; k>=0; k--) {
+				file->rdwr_longlong(financial_history[k][j]);
+			}
+		}
+	}
+
+	if(file->get_version()>=102002) {
+		file->rdwr_bool(withdraw);
+	}
+
 	// otherwise inintialized to zero if loading ...
 	financial_history[0][LINE_CONVOIS] = count_convoys();
 }
@@ -166,7 +183,9 @@ void simline_t::rdwr(loadsave_t *file)
 
 void simline_t::laden_abschliessen()
 {
-	register_stops(fpl);
+	if(  line_managed_convoys.get_count()>0  ) {
+		register_stops(fpl);
+	}
 	recalc_status();
 }
 
@@ -176,7 +195,7 @@ void simline_t::register_stops(schedule_t * fpl)
 {
 DBG_DEBUG("simline_t::register_stops()", "%d fpl entries in schedule %p", fpl->get_count(),fpl);
 	for (int i = 0; i<fpl->get_count(); i++) {
-		const halthandle_t halt = haltestelle_t::get_halt( welt, fpl->eintrag[i].pos );
+		const halthandle_t halt = haltestelle_t::get_halt( welt, fpl->eintrag[i].pos, sp );
 		if(halt.is_bound()) {
 //DBG_DEBUG("simline_t::register_stops()", "halt not null");
 			halt->add_line(self);
@@ -199,7 +218,7 @@ void simline_t::unregister_stops()
 void simline_t::unregister_stops(schedule_t * fpl)
 {
 	for (int i = 0; i<fpl->get_count(); i++) {
-		halthandle_t halt = haltestelle_t::get_halt( welt, fpl->eintrag[i].pos );
+		halthandle_t halt = haltestelle_t::get_halt( welt, fpl->eintrag[i].pos, sp );
 		if(halt.is_bound()) {
 			halt->remove_line(self);
 		}
@@ -210,11 +229,10 @@ void simline_t::unregister_stops(schedule_t * fpl)
 
 void simline_t::renew_stops()
 {
-	if(  old_fpl  ) {
-		unregister_stops( old_fpl );
+	if(  line_managed_convoys.get_count()>0  ) {
+		register_stops( fpl );
+		DBG_DEBUG("simline_t::renew_stops()", "Line id=%d, name='%s'", id, name.c_str());
 	}
-	register_stops( fpl );
-	DBG_DEBUG("simline_t::renew_stops()", "Line id=%d, name='%s'", id, name);
 }
 
 
@@ -232,22 +250,9 @@ void simline_t::new_month()
 }
 
 
-
-/*
- * called from line_management_gui.cc to prepare line for a change of its schedule
- */
-void simline_t::prepare_for_update()
-{
-	DBG_DEBUG("simline_t::prepare_for_update()", "line %d (%p)", id, this);
-	delete old_fpl;
-	old_fpl = fpl->copy();
-}
-
-
-
 void simline_t::init_financial_history()
 {
-	memset( financial_history, 0, sizeof(financial_history) );
+	MEMZERO(financial_history);
 }
 
 
@@ -261,6 +266,7 @@ void simline_t::recalc_status()
 	if(financial_history[0][LINE_CONVOIS]==0) {
 		// noconvois assigned to this line
 		state_color = COL_WHITE;
+		withdraw = false;
 	}
 	else if(financial_history[0][LINE_PROFIT]<0) {
 		// ok, not performing best
@@ -295,22 +301,18 @@ void simline_t::recalc_catg_index()
 		old_goods_catg_index.append( goods_catg_index[i] );
 	}
 	goods_catg_index.clear();
+	withdraw = line_managed_convoys.get_count()>0;
 	// then recreate current
 	for(unsigned i=0;  i<line_managed_convoys.get_count();  i++ ) {
 		// what goods can this line transport?
-//		const convoihandle_t cnv = line_managed_convoys[i];
+		// const convoihandle_t cnv = line_managed_convoys[i];
 		const convoi_t *cnv = line_managed_convoys[i].get_rep();
-		for(uint i=0;  i<cnv->get_vehikel_anzahl();  i++  ) {
-			// Only consider vehicles that really transport something
-			// this helps against routing errors through passenger
-			// trains pulling only freight wagons
-			if (cnv->get_vehikel(i)->get_fracht_max() == 0) {
-				continue;
-			}
-			const ware_besch_t *ware=cnv->get_vehikel(i)->get_fracht_typ();
-			if(ware!=warenbauer_t::nichts  ) {
-				goods_catg_index.append_unique( ware->get_catg_index(), 1 );
-			}
+		withdraw &= cnv->get_withdraw();
+
+		const minivec_tpl<uint8> &convoys_goods = cnv->get_goods_catg_index();
+		for(  uint8 i = 0;  i < convoys_goods.get_count();  i++  ) {
+			const uint8 catg_index = convoys_goods[i];
+			goods_catg_index.append_unique( catg_index, 1 );
 		}
 	}
 	// if different => schedule need recalculation
@@ -329,3 +331,15 @@ void simline_t::recalc_catg_index()
 		}
 	}
 }
+
+
+
+void simline_t::set_withdraw( bool yes_no )
+{
+	withdraw = yes_no  &&  (line_managed_convoys.get_count()>0);
+	// convois in depots will be immeadiately destroyed, thus we go backwards
+	for( sint32 i=line_managed_convoys.get_count()-1;  i>=0;  i--  ) {
+		line_managed_convoys[i]->set_no_load(yes_no);	// must be first, since set withdraw might destroy convoi if in depot!
+		line_managed_convoys[i]->set_withdraw(yes_no);
+	}
+}
