@@ -1,4 +1,10 @@
+#if defined(_M_X64)  ||  defined(__x86_64__)
+#warning "Simutrans is preferably compiled as 32 bit binary!"
+#endif
+
+
 #include <stdio.h>
+#include <string>
 #include <new>
 #include <time.h>
 
@@ -55,14 +61,15 @@
 #include "dings/baum.h"
 
 #include "utils/simstring.h"
-#include "utils/cstring_t.h"
 #include "utils/searchfolder.h"
 
+#include "dataobj/network.h"	// must be before any "windows.h" is included via bzlib2.h ...
 #include "dataobj/loadsave.h"
 #include "dataobj/umgebung.h"
 #include "dataobj/tabfile.h"
 #include "dataobj/einstellungen.h"
 #include "dataobj/translator.h"
+#include "dataobj/pakset_info.h"
 
 #include "besch/reader/obj_reader.h"
 #include "besch/sound_besch.h"
@@ -70,10 +77,13 @@
 #include "music/music.h"
 #include "sound/sound.h"
 
+#include "utils/cbuffer_t.h"
+
 #include "bauer/vehikelbauer.h"
 #include "vehicle/simvehikel.h"
 #include "vehicle/simverkehr.h"
 
+using std::string;
 
 /* diagnostic routine:
  * show the size of several internal structures
@@ -175,29 +185,80 @@ static void show_times(karte_t *welt, karte_ansicht_t *view)
 
 
 
-/**
- * Show Intro Screen
- */
-static void zeige_banner(karte_t *welt)
+void modal_dialogue( gui_frame_t *gui, long magic, karte_t *welt, bool (*quit)() )
 {
-	banner_t* b = new banner_t();
+	if(  display_get_width()==0  ) {
+		dbg->error( "modal_dialogue()", "called without a display driver => nothing will be shown!" );
+		// cannot handle this!
+		return;
+	}
+
 	event_t ev;
+	create_win( (display_get_width()-gui->get_fenstergroesse().x)/2, (display_get_height()-gui->get_fenstergroesse().y)/2, gui, w_info, magic );
 
-	destroy_all_win();	// since eventually the successful load message is still there ....
+	if(  welt  ) {
+		welt->set_pause( false );
+		welt->reset_interaction();
+		welt->reset_timer();
 
-	create_win(0, -48, b, w_info, magic_none );
-
-	// hide titelbar with this trick
-	win_set_pos( b, 0, -48 );
-
-	do {
-		win_poll_event(&ev);
-		check_pos_win(&ev);
-		INT_CHECK("simmain 189");
-		dr_sleep(10);
-		welt->step();
-	} while(win_is_top(b));
+		long ms_pause = max( 25, 1000/umgebung_t::fps );
+		uint32 last_step = dr_time()+ms_pause;
+		uint step_count = 5;
+		while(  win_is_open(gui)  &&  !umgebung_t::quit_simutrans  &&  !quit()  ) {
+			do {
+				DBG_DEBUG4("zeige_banner", "calling win_poll_event");
+				win_poll_event(&ev);
+				// no toolbar events
+				if(  ev.my < werkzeug_t::toolbar_tool[0]->iconsize.y  ) {
+					ev.my = werkzeug_t::toolbar_tool[0]->iconsize.y;
+				}
+				if(  ev.cy < werkzeug_t::toolbar_tool[0]->iconsize.y  ) {
+					ev.cy = werkzeug_t::toolbar_tool[0]->iconsize.y;
+				}
+				DBG_DEBUG4("zeige_banner", "calling check_pos_win");
+				check_pos_win(&ev);
+				if(  ev.ev_class == EVENT_SYSTEM  &&  ev.ev_code == SYSTEM_QUIT  ) {
+					umgebung_t::quit_simutrans = true;
+					break;
+				}
+				dr_sleep(5);
+			} while(  dr_time()<last_step  );
+			DBG_DEBUG4("zeige_banner", "calling welt->sync_step");
+			welt->sync_step( ms_pause, true, true );
+			DBG_DEBUG4("zeige_banner", "calling welt->step");
+			if(  step_count--==0  ) {
+				welt->step();
+				step_count = 5;
+			}
+			last_step += ms_pause;
+		}
+	}
+	else {
+		display_show_pointer(true);
+		show_pointer(1);
+		set_pointer(0);
+		display_fillbox_wh( 0, 0, display_get_width(), display_get_height(), COL_BLACK, true );
+		while(  win_is_open(gui)  &&  !umgebung_t::quit_simutrans  &&  !quit()  ) {
+			// do not move, do not close it!
+			dr_prepare_flush();
+			gui->zeichnen( koord(win_get_posx(gui),win_get_posy(gui)), gui->get_fenstergroesse() );
+			display_poll_event(&ev);
+			// main window resized
+			check_pos_win(&ev);
+			dr_flush();
+			dr_sleep(50);
+			// main window resized
+			if(ev.ev_class==EVENT_SYSTEM  &&  ev.ev_code==SYSTEM_RESIZE) {
+				// main window resized
+				simgraph_resize( ev.mx, ev.my );
+				display_fillbox_wh( 0, 0, ev.mx, ev.my, COL_BLACK, true );
+			}
+		}
+		set_pointer(1);
+		display_fillbox_wh( 0, 0, display_get_width(), display_get_height(), COL_BLACK, true );
+	}
 
+	// just trigger not another following window => wait for button release
 	if (IS_LEFTCLICK(&ev)) {
 		do {
 			display_get_event(&ev);
@@ -206,46 +267,47 @@ static void zeige_banner(karte_t *welt)
 }
 
 
+// some routines for the modal display
+static bool never_quit() { return false; }
+static bool empty_objfilename() { return !umgebung_t::objfilename.empty(); }
+static bool no_language() { return translator::get_language()!=-1; }
+
+
 
 /**
  * Show pak selector
  */
 static void ask_objfilename()
 {
-	// more than one => show selector box (ugly and without translations ...)
-	set_pointer(0);
 	pakselector_t* sel = new pakselector_t();
 	sel->fill_list();
-	if(!sel->has_pak()) {
-		// nothing there ...
-		set_pointer(1);
+	if(sel->has_pak()) {
+		destroy_all_win(true);	// since eventually the successful load message is still there ....
+		modal_dialogue( sel, magic_none, NULL, empty_objfilename );
+	}
+	else {
 		delete sel;
-		return;
 	}
-	koord xy( display_get_width()/2 - 180, display_get_height()/2 - sel->get_fenstergroesse().y/2 );
-	event_t ev;
+}
 
-	destroy_all_win();	// since eventually the successful load message is still there ....
 
-	create_win( xy.x, xy.y, sel, w_info, magic_none );
 
-	while(umgebung_t::objfilename.empty()) {
-		// do not move, do not close it!
-		dr_prepare_flush();
-		sel->zeichnen( xy, sel->get_fenstergroesse() );
-		display_poll_event(&ev);
-		// main window resized
-		check_pos_win(&ev);
-		dr_flush();
-		dr_sleep(50);
-		// main window resized
-		if(ev.ev_class==EVENT_SYSTEM  &&  ev.ev_code==SYSTEM_RESIZE) {
-			// main window resized
-			simgraph_resize( ev.mx, ev.my );
-			display_fillbox_wh( 0, 0, ev.mx, ev.my, COL_BLACK, true );
-		}
+/**
+ * Show language selector
+ */
+static void ask_language()
+{
+	if(  display_get_width()==0  ) {
+		// only console available ... => choose english for the moment
+		dbg->warning( "ask_language", "No language selected, will use english!" );
+		translator::set_language( "en" );
+	}
+	else {
+		sprachengui_t* sel = new sprachengui_t();
+		destroy_all_win(true);	// since eventually the successful load message is still there ....
+		modal_dialogue( sel, magic_none, NULL, no_language );
+		destroy_win( sel );
 	}
-	set_pointer(1);
 }
 
 
@@ -299,6 +361,7 @@ int simu_main(int argc, char** argv)
 #else
 	std::set_new_handler(sim_new_handler);
 #endif
+	umgebung_t::init();
 
 	// you really want help with this?
 	if (gimme_arg(argc, argv, "-h",     0) ||
@@ -320,44 +383,85 @@ int simu_main(int argc, char** argv)
 			"  by Hansjrg Malthaner et. al.\n"
 			"  <hansjoerg.malthaner@gmx.de>\n"
 			"---------------------------------------\n"
+			"command line parameters available: \n"
+			" -addons             loads also addons (with -objects)\n"
+			" -async              asynchronic images, only for SDL\n"
+			" -debug NUM          enables debuging (1..5)\n"
+			" -freeplay           play with endless money\n"
+			" -fullscreen         starts simutrans in fullscreen mode\n"
+			" -fps COUNT          framerate (from 5 to 100)\n"
+			" -h | -help | --help displays this help\n"
+			" -lang CODE          starts with specified language\n"
+			" -load FILE[.sve]    loads game in file 'save/FILE.sve'\n"
+			" -log                enables logging to file 'simu.log'\n"
+			" -noaddons           does not load any addon (default)\n"
+			" -nomidi             turns off background music\n"
+			" -nosound            turns off ambient sounds\n"
+			" -objects DIR_NAME/  load the pakset in specified directory\n"
+			" -res N              starts in specified resolution: \n"
+			"                      1=640x480, 2=800x600, 3=1024x768, 4=1280x1024\n"
+			" -screensize WxH     set screensize to width W and height H\n"
+			" -server [PORT]      starts program as server (for network game)\n"
+			"                     without port specified uses 13353\n"
+			" -server_id NUM      ID for server announcements\n"
+			" -server_name NAME   name for server announcements\n"
+			" -server_comment TXT comment for server announcements\n"
+			" -singleuser         Save everything in program directory (portable version)\n"
+#ifdef DEBUG
+			" -sizes              Show current size of some structures\n"
+#endif
+			" -startyear N        start in year N\n"
+			" -timeline           enables timeline\n"
+#if defined DEBUG || defined PROFILE
+			" -times              does some simple profiling\n"
+			" -until MONTH        quits when MONTH = (month*12+year-1) starts\n"
+#endif
+			" -use_workdir        use current dir as basedir\n"
 		);
 		return 0;
 	}
 
+#ifdef _WIN32
+#define PATHSEP "\\"
+#else
+#define PATHSEP "/"
+#endif
+	const char* path_sep = PATHSEP;
+
+
 #ifdef __BEOS__
-	if(1) { // since BeOS only supports relative paths ...
+	if (1) // since BeOS only supports relative paths ...
 #else
 	// use current dir as basedir, else use program_dir
-	if (gimme_arg(argc, argv, "-use_workdir",0)) {
+	if (gimme_arg(argc, argv, "-use_workdir", 0))
 #endif
+	{
 		// save the current directories
-		getcwd( umgebung_t::program_dir, 1024 );
-#ifdef _WIN32
-		strcat( umgebung_t::program_dir, "\\" );
-#else
-		strcat( umgebung_t::program_dir, "/" );
-#endif
+		getcwd(umgebung_t::program_dir, lengthof(umgebung_t::program_dir));
+		strcat( umgebung_t::program_dir, path_sep );
 	}
 	else {
 		strcpy( umgebung_t::program_dir, argv[0] );
-#ifdef _WIN32
-		*(strrchr( umgebung_t::program_dir, '\\' )+1) = 0;
-#else
-		*(strrchr( umgebung_t::program_dir, '/' )+1) = 0;
-#endif
+		*(strrchr( umgebung_t::program_dir, path_sep[0] )+1) = 0;
+
 		chdir( umgebung_t::program_dir );
 	}
+	printf("Use work dir %s\n", umgebung_t::program_dir);
 
 	// only the pak specifiy conf should overide this!
 	uint16 pak_diagonal_multiplier = umgebung_t::default_einstellungen.get_pak_diagonal_multiplier();
 
 	// parsing config/simuconf.tab
-	print("Reading low level config data ...\n");
-	bool found_simuconf=false;
+	printf("Reading low level config data ...\n");
+	bool found_settings = false;
+	bool found_simuconf = false;
 	bool multiuser = (gimme_arg(argc, argv, "-singleuser", 0) == NULL);
 
 	tabfile_t simuconf;
-	if(simuconf.open("config/simuconf.tab")) {
+	char path_to_simuconf[24];
+	// was  config/simuconf.tab
+	sprintf(path_to_simuconf, "config%csimuconf.tab", path_sep[0]);
+	if(simuconf.open(path_to_simuconf)) {
 		{
 			tabfileobj_t contents;
 			simuconf.read(contents);
@@ -387,30 +491,33 @@ int simu_main(int argc, char** argv)
 			remove( "settings.xml" );
 		}
 		else {
+			found_settings = true;
 			umgebung_t::rdwr(&file);
 			umgebung_t::default_einstellungen.rdwr(&file);
 			file.close();
-			// reset to false (otherwise freeplay will persist)
+			// reset to false (otherwise these settings will persist)
 			umgebung_t::default_einstellungen.set_freeplay( false );
+			umgebung_t::default_einstellungen.set_allow_player_change( true );
+			umgebung_t::announce_server = 0;
 		}
 	}
 
 	// continue parsing ...
 	chdir( umgebung_t::program_dir );
 	if(  found_simuconf  ) {
-		if(simuconf.open("config/simuconf.tab")) {
-			printf("parse_simuconf() at config/simuconf.tab");
-			umgebung_t::default_einstellungen.parse_simuconf( simuconf, disp_width, disp_height, fullscreen, umgebung_t::objfilename, false );
+		if(simuconf.open(path_to_simuconf)) {
+			printf("parse_simuconf() at config/simuconf.tab: ");
+			umgebung_t::default_einstellungen.parse_simuconf( simuconf, disp_width, disp_height, fullscreen, umgebung_t::objfilename );
 		}
 	}
 
 	// if set for multiuser, then parses the users config (if there)
 	// retrieve everything (but we must do this again once more ... )
 	if(multiuser) {
-		cstring_t obj_conf = umgebung_t::user_dir;
-		if(simuconf.open(obj_conf + "simuconf.tab")) {
-			printf("parse_simuconf() at %ssimuconf.tab", (const char *)obj_conf);
-			umgebung_t::default_einstellungen.parse_simuconf( simuconf, disp_width, disp_height, fullscreen, umgebung_t::objfilename, false );
+		const string obj_conf = string(umgebung_t::user_dir) + "simuconf.tab";
+		if (simuconf.open(obj_conf.c_str())) {
+			printf("parse_simuconf() at %s: ", obj_conf.c_str() );
+			umgebung_t::default_einstellungen.parse_simuconf( simuconf, disp_width, disp_height, fullscreen, umgebung_t::objfilename );
 		}
 	}
 
@@ -430,16 +537,60 @@ int simu_main(int argc, char** argv)
 	// now set the desired objectfilename (overide all previous settings)
 	if (gimme_arg(argc, argv, "-objects", 1)) {
 		umgebung_t::objfilename = gimme_arg(argc, argv, "-objects", 1);
+		// append slash / replace trailing backslash if necessary
+		uint16 len = umgebung_t::objfilename.length();
+		if (len > 0) {
+			if (umgebung_t::objfilename[len-1]=='\\') {
+				umgebung_t::objfilename.erase(len-1);
+				umgebung_t::objfilename += "/";
+			}
+			else if (umgebung_t::objfilename[len-1]!='/') {
+				umgebung_t::objfilename += "/";
+			}
+		}
 	}
 
+#ifdef REVISION
+	const char *version = "Simutrans version " VERSION_NUMBER " from " VERSION_DATE " r" QUOTEME(REVISION) "\n";
+#else
+	const char *version = "Simutrans version " VERSION_NUMBER " from " VERSION_DATE "\n";
+#endif
 	if (gimme_arg(argc, argv, "-log", 0)) {
 		chdir( umgebung_t::user_dir );
-		init_logging("simu.log", true, gimme_arg(argc, argv, "-log", 0) != NULL);
+		char temp_log_name[256];
+		const char *logname = "simu.log";
+		if(  gimme_arg(argc, argv, "-server", 0)  ) {
+			const char *p = gimme_arg(argc, argv, "-server", 1);
+			int portadress = p ? atoi( p ) : 13353;
+			sprintf( temp_log_name, "simu-server%d.log", portadress==0 ? 13353 : portadress );
+			logname = temp_log_name;
+		}
+		init_logging( logname, true, gimme_arg(argc, argv, "-log", 0 ) != NULL, version );
 	} else if (gimme_arg(argc, argv, "-debug", 0) != NULL) {
-		init_logging("stderr", true, gimme_arg(argc, argv, "-debug", 0) != NULL);
+		init_logging( "stderr", true, gimme_arg(argc, argv, "-debug", 0 ) != NULL, version );
 	} else {
-		init_logging(NULL, false, false);
+		init_logging(NULL, false, false, version);
 	}
+
+	// starting a server?
+	if(  gimme_arg(argc, argv, "-server", 0)  ) {
+		const char *p = gimme_arg(argc, argv, "-server", 1);
+		int portadress = p ? atoi( p ) : 13353;
+		if(  portadress==0  ) {
+			portadress = 13353;
+		}
+		// will fail fatal on the opening routine ...
+		dbg->message( "simmain()", "Server started on port %i", portadress );
+		umgebung_t::networkmode = network_init_server( portadress );
+		if(  umgebung_t::networkmode  ) {
+			umgebung_t::server = portadress;
+		}
+	}
+	else {
+		// no announce for clients ...
+		umgebung_t::announce_server = 0;
+	}
+
 	DBG_MESSAGE( "simmain::main()", "Version: " VERSION_NUMBER "  Date: " VERSION_DATE);
 	DBG_MESSAGE( "Debuglevel","%i", umgebung_t::verbose_debug );
 	DBG_MESSAGE( "program_dir", umgebung_t::program_dir );
@@ -450,6 +601,9 @@ int simu_main(int argc, char** argv)
 		show_sizes();
 	}
 #endif
+
+	// prepare skins first
+	obj_reader_t::init();
 	chdir( umgebung_t::program_dir );
 
 	// likely only the programm without graphics was downloaded
@@ -480,12 +634,12 @@ int simu_main(int argc, char** argv)
 
 	fullscreen |= (gimme_arg(argc, argv, "-fullscreen", 0) != NULL);
 
-	if (gimme_arg(argc, argv, "-screensize", 0) != NULL) {
+	if(gimme_arg(argc, argv, "-screensize", 0) != NULL) {
 		const char* res_str = gimme_arg(argc, argv, "-screensize", 1);
 		int n = 0;
 
 		if (res_str != NULL) {
-			n = sscanf(res_str, "%dx%d", &disp_width, &disp_height);
+			n = sscanf(res_str, "%hdx%hd", &disp_width, &disp_height);
 		}
 
 		if (n != 2) {
@@ -516,8 +670,10 @@ int simu_main(int argc, char** argv)
 		}
 	}
 
-	print("Preparing display ...\n");
+	printf("Preparing display ...\n");
+	DBG_MESSAGE("simmain", "simgraph_init disp_width=%d, disp_height=%d, fullscreen=%d", disp_width, disp_height, fullscreen);
 	simgraph_init(disp_width, disp_height, fullscreen);
+	DBG_MESSAGE("simmain", ".. results in disp_width=%d, disp_height=%d", display_get_width(), display_get_height());
 
 	// if no object files given, we ask the user
 	if(  umgebung_t::objfilename.empty()  ) {
@@ -534,37 +690,49 @@ int simu_main(int argc, char** argv)
 	}
 
 	// now find the pak specific tab file ...
-	cstring_t obj_conf = umgebung_t::objfilename + "config/simuconf.tab";
-	cstring_t dummy("");
-	if(simuconf.open((const char *)obj_conf)) {
+	const string obj_conf = umgebung_t::objfilename + path_to_simuconf;
+	string dummy("");
+	if (simuconf.open(obj_conf.c_str())) {
 		sint16 idummy;
-		printf("parse_simuconf() at %sconfig/simuconf.tab", (const char *)obj_conf);
-		umgebung_t::default_einstellungen.parse_simuconf( simuconf, idummy, idummy, idummy, dummy, false );
+		printf("parse_simuconf() at %s: ", obj_conf.c_str());
+		umgebung_t::default_einstellungen.parse_simuconf( simuconf, idummy, idummy, idummy, dummy );
 		pak_diagonal_multiplier = umgebung_t::default_einstellungen.get_pak_diagonal_multiplier();
 		simuconf.close();
 	}
 	// and parse again parse the user settings
 	if(umgebung_t::user_dir!=umgebung_t::program_dir) {
-		cstring_t obj_conf = umgebung_t::user_dir;
-		if(simuconf.open(obj_conf + "simuconf.tab")) {
+		const string obj_conf = string(umgebung_t::user_dir) + "simuconf.tab";
+		if (simuconf.open(obj_conf.c_str())) {
 			sint16 idummy;
-			printf("parse_simuconf() at %ssimuconf.tab", (const char *)obj_conf);
-			umgebung_t::default_einstellungen.parse_simuconf( simuconf, idummy, idummy, idummy, dummy, false );
+			printf("parse_simuconf() at %s: ", obj_conf.c_str());
+			umgebung_t::default_einstellungen.parse_simuconf( simuconf, idummy, idummy, idummy, dummy );
 			simuconf.close();
 		}
+		if(  gimme_arg(argc, argv, "-objects", 1) != NULL  ) {
+			if(gimme_arg(argc, argv, "-addons", 0) != NULL) {
+				umgebung_t::default_einstellungen.set_with_private_paks( true );
+			}
+			if(gimme_arg(argc, argv, "-noaddons", 0) != NULL) {
+				umgebung_t::default_einstellungen.set_with_private_paks( false );
+			}
+		}
+	}
+	else {
+		// not possible for single user
+		umgebung_t::default_einstellungen.set_with_private_paks( false );
 	}
 
 	// now (re)set the correct length from the pak
 	umgebung_t::default_einstellungen.set_pak_diagonal_multiplier( pak_diagonal_multiplier );
 	vehikel_basis_t::set_diagonal_multiplier( pak_diagonal_multiplier, pak_diagonal_multiplier );
 
-	convoihandle_t::init( umgebung_t::max_convoihandles );
-	linehandle_t::init( umgebung_t::max_linehandles );
-	halthandle_t::init( umgebung_t::max_halthandles );
+	convoihandle_t::init( 1024 );
+	linehandle_t::init( 1024 );
+	halthandle_t::init( 1024 );
 
 	// just check before loading objects
 	if (!gimme_arg(argc, argv, "-nosound", 0)  &&  dr_init_sound()) {
-		print("Reading compatibility sound data ...\n");
+		printf("Reading compatibility sound data ...\n");
 		sound_besch_t::init();
 	}
 	else {
@@ -572,57 +740,111 @@ int simu_main(int argc, char** argv)
 	}
 
 	// Adam - Moved away loading from simmain and placed into translator for better modularisation
-	if (!translator::load(umgebung_t::objfilename)) {
+	if(  !translator::load(umgebung_t::objfilename)  ) {
 		// installation error: likely only program started
 		dbg->fatal("simmain::main()", "Unable to load any language files\n*** PLEASE INSTALL PROPER BASE FILES ***\n");
 		exit(11);
 	}
 
+	// use requested language (if available)
+	if(  gimme_arg(argc, argv, "-lang", 1)  ) {
+		const char *iso = gimme_arg(argc, argv, "-lang", 1);
+		if(  strlen(iso)>=2  ) {
+			translator::set_language( iso );
+		}
+		if(  translator::get_language()==-1  ) {
+			dbg->fatal("simmain", "Illegal language defintion \"%s\"", iso );
+		}
+		umgebung_t::language_iso = translator::get_lang()->iso_base;
+	}
+	else if(  found_settings  ) {
+		translator::set_language( umgebung_t::language_iso );
+	}
+
 	// Hajo: simgraph init loads default fonts, now we need to load
 	// the real fonts for the current language
 	sprachengui_t::init_font_from_lang();
 	chdir(umgebung_t::program_dir);
 
-	print("Reading city configuration ...\n");
+	printf("Reading city configuration ...\n");
 	stadt_t::cityrules_init(umgebung_t::objfilename);
 
-	print("Reading speedbonus configuration ...\n");
+	printf("Reading speedbonus configuration ...\n");
 	vehikelbauer_t::speedbonus_init(umgebung_t::objfilename);
 
-	print("Reading forest configuration ...\n");
-	baum_t::forestrules_init(umgebung_t::objfilename);
+	printf("Reading menu configuration ...\n");
+	werkzeug_t::init_menu();
 
 	// loading all paks
-	print("Reading object data from %s...\n", (const char*)umgebung_t::objfilename);
-	if (!obj_reader_t::init(umgebung_t::objfilename)) {
-		fprintf(stderr, "reading object data failed.\n");
-		exit(11);
+	printf("Reading object data from %s...\n", umgebung_t::objfilename.c_str());
+	obj_reader_t::load(umgebung_t::objfilename.c_str(), translator::translate("Loading paks ...") );
+	if(  umgebung_t::default_einstellungen.get_with_private_paks()  ) {
+		// try to read addons from private directory
+		chdir( umgebung_t::user_dir );
+		if(!obj_reader_t::load(umgebung_t::objfilename.c_str(), translator::translate("Loading addon paks ..."))) {
+			fprintf(stderr, "reading addon object data failed (disabling).\n");
+			umgebung_t::default_einstellungen.set_with_private_paks( false );
+		}
+		chdir( umgebung_t::program_dir );
 	}
-	obj_reader_t::has_been_init = true;
+	obj_reader_t::laden_abschliessen();
+	pakset_info_t::calculate_checksum();
+	pakset_info_t::debug();
 
 	// set overtaking offsets
 	vehikel_basis_t::set_overtaking_offsets( umgebung_t::drive_on_left );
 
-	print("Reading menu configuration ...\n");
-	werkzeug_t::init_menu(umgebung_t::objfilename);
+	printf("Reading menu configuration ...\n");
+	werkzeug_t::read_menu(umgebung_t::objfilename);
+
+	if(  translator::get_language()==-1  ) {
+		ask_language();
+	}
 
 	bool new_world = true;
-	cstring_t loadgame = "";
+	std::string loadgame;
 
 	if (gimme_arg(argc, argv, "-load", 0) != NULL) {
-		char buf[256];
+		cbuffer_t buf(1024);
 		chdir( umgebung_t::user_dir );
 		/**
 		 * Added automatic adding of extension
 		 */
-		sprintf(buf, SAVE_PATH_X "%s", (const char*)searchfolder_t::complete(gimme_arg(argc, argv, "-load", 1), "sve"));
+		const char *name = gimme_arg(argc, argv, "-load", 1);
+		if(  strstr(name,"net:")==name  ) {
+			buf.append( name );
+		}
+		else {
+			buf.printf( SAVE_PATH_X "%s", searchfolder_t::complete(name, "sve").c_str() );
+		}
+		printf( "loading savegame \"%s\"\n", name );
 		loadgame = buf;
 		new_world = false;
 	}
-	else {
+
+	// recover last server game
+	if(  new_world  &&  umgebung_t::server  ) {
+		chdir( umgebung_t::user_dir );
+		loadsave_t file;
+		static char servername[128];
+		sprintf( servername, "server%d-network.sve", umgebung_t::server );
+		// try recover with the latest savegame
+		if(  file.rd_open(servername)  ) {
+			// compare pakset (objfilename has trailing path separator, pak_extension not)
+			if(  strncmp( file.get_pak_extension(), umgebung_t::objfilename.c_str(), strlen(file.get_pak_extension() ) )==0  ) {
+				// same pak directory - load this
+				loadgame = servername;
+				new_world = false;
+			}
+			file.close();
+		}
+	}
+
+	// still nothing to be loaded => search for demo games
+	if(  new_world  ) {
 		chdir( umgebung_t::program_dir );
 		char buffer[256];
-		sprintf(buffer, "%s%sdemo.sve", (const char*)umgebung_t::program_dir, (const char*)umgebung_t::objfilename);
+		sprintf(buffer, "%s%sdemo.sve", (const char*)umgebung_t::program_dir, umgebung_t::objfilename.c_str());
 		// access did not work!
 		FILE *f=fopen(buffer,"rb");
 		if(f) {
@@ -652,10 +874,10 @@ DBG_MESSAGE("simmain","loadgame file found at %s",buffer);
 
 	// init midi before loading sounds
 	if(dr_init_midi()) {
-		print("Reading midi data ...\n");
+		printf("Reading midi data ...\n");
 		if(!midi_init(umgebung_t::user_dir)) {
 			if(!midi_init(umgebung_t::program_dir)) {
-				print("Midi disabled ...\n");
+				printf("Midi disabled ...\n");
 			}
 		}
 		if(gimme_arg(argc, argv, "-nomidi", 0)) {
@@ -663,7 +885,7 @@ DBG_MESSAGE("simmain","loadgame file found at %s",buffer);
 		}
 	}
 	else {
-		print("Midi disabled ...\n");
+		printf("Midi disabled ...\n");
 		midi_set_mute(true);
 	}
 
@@ -692,9 +914,29 @@ DBG_MESSAGE("simmain","loadgame file found at %s",buffer);
 		umgebung_t::fps = want_refresh < 5 ? 5 : (want_refresh > 100 ? 100 : want_refresh);
 	}
 
+	// query server stuff
+	ref_str = gimme_arg(argc, argv, "-server_id", 1);
+	if (ref_str != NULL) {
+		umgebung_t::announce_server = atoi(ref_str);
+	}
+
+	ref_str = gimme_arg(argc, argv, "-server_comment", 1);
+	if (ref_str != NULL) {
+		umgebung_t::server_name = ref_str;
+	}
+
+	ref_str = gimme_arg(argc, argv, "-server_comment", 1);
+	if (ref_str != NULL) {
+		umgebung_t::server_comment = ref_str;
+	}
+
 	chdir(umgebung_t::user_dir);
 
-	if(loadgame==""  ||  !welt->laden(loadgame)) {
+	// reset random counter to true randomness
+	setsimrand(dr_time(), dr_time());
+	clear_random_mode( 7 );	// allow all
+
+	if(loadgame==""  ||  !welt->laden(loadgame.c_str())) {
 		// create a default map
 		DBG_MESSAGE("init with default map","(failing will be a pak error!)");
 		// no autosave on initial map during the first six month ...
@@ -702,18 +944,15 @@ DBG_MESSAGE("simmain","loadgame file found at %s",buffer);
 		new_world = true;
 		sint32 old_autosave = umgebung_t::autosave;
 		umgebung_t::autosave = false;
-		einstellungen_t sets = umgebung_t::default_einstellungen;
+		einstellungen_t sets;
+		sets.copy_city_road( umgebung_t::default_einstellungen );
 		sets.set_default_climates();
 		sets.set_use_timeline( 1 );
-		sets.set_grundwasser( -2*Z_TILE_STEP );
-		sets.set_max_mountain_height( 160 );
-		sets.set_map_roughness( 0.6 );
 		sets.set_groesse(64,64);
 		sets.set_anzahl_staedte(1);
 		sets.set_land_industry_chains(1);
 		sets.set_tourist_attractions(1);
 		sets.set_verkehr_level(7);
-		sets.set_karte_nummer( 33 );
 		welt->init(&sets,0);
 		//  start in June ...
 		intr_set(welt, view);
@@ -733,7 +972,8 @@ DBG_MESSAGE("simmain","loadgame file found at %s",buffer);
 		werkzeug_t::toolbar_tool[0]->init(welt,welt->get_active_player());
 	}
 
-#ifdef DEBUG
+	welt->set_fast_forward(false);
+#if defined DEBUG || defined PROFILE
 	// do a render test?
 	if (gimme_arg(argc, argv, "-times", 0) != NULL) {
 		show_times(welt, view);
@@ -742,12 +982,24 @@ DBG_MESSAGE("simmain","loadgame file found at %s",buffer);
 	// finish after a certain month? (must be entered decimal, i.e. 12*year+month
 	if(  gimme_arg(argc, argv, "-until", 0) != NULL  ) {
 		quit_month = atoi( gimme_arg(argc, argv, "-until", 1) );
+		welt->set_fast_forward(true);
 	}
 #endif
 
-	welt->set_fast_forward(false);
-	view->display(true);
-	intr_refresh_display(true);
+	welt->reset_timer();
+	if(  !umgebung_t::networkmode  &&  !umgebung_t::server  ) {
+#ifdef display_in_main
+		DBG_MESSAGE("simmain", "calling view->display");
+		view->display(true);
+		DBG_MESSAGE("simmain", "calling intr_refresh_display");
+		intr_refresh_display(true);
+#endif
+		intr_enable();
+	}
+	else {
+		intr_disable();
+	}
+
 
 #ifdef USE_SOFTPOINTER
 	// Hajo: give user a mouse to work with
@@ -756,6 +1008,7 @@ DBG_MESSAGE("simmain","loadgame file found at %s",buffer);
 		display_set_pointer(skinverwaltung_t::mouse_cursor->get_bild_nr(0));
 	}
 #endif
+	DBG_MESSAGE("simmain", "calling display_show_pointer");
 	display_show_pointer(true);
 	show_pointer(1);
 	set_pointer(0);
@@ -764,151 +1017,202 @@ DBG_MESSAGE("simmain","loadgame file found at %s",buffer);
 
 	// Hajo: simgraph init loads default fonts, now we need to load
 	// the real fonts for the current language
+	DBG_MESSAGE("simmain", "sprachengui_t::init_font_from_lang");
 	sprachengui_t::init_font_from_lang();
 
-	welt->get_message()->clear();
-	ticker::add_msg("Welcome to Simutrans, a game created by Hj. Malthaner and the Simutrans community.", koord::invalid, PLAYER_FLAG + 1);
-
-	zeige_banner(welt);
-
-	intr_set(welt, view);
-
-	do {
+	destroy_all_win(true);
+	if(  !umgebung_t::server  ) {
+		welt->get_message()->clear();
+	}
+	while(  !umgebung_t::quit_simutrans  ) {
 		// play next tune?
 		check_midi();
 
+		if(  !umgebung_t::networkmode  &&  new_world  ) {
+			DBG_MESSAGE("simmain", "show banner");
+			printf( "Show banner ... \n" );
+			ticker::add_msg("Welcome to Simutrans, a game created by Hj. Malthaner and the Simutrans community.", koord::invalid, PLAYER_FLAG + 1);
+			modal_dialogue( new banner_t(welt), magic_none, welt, never_quit );
+			// only show new world, if no other dialoge is active ...
+			new_world = win_get_open_count()==0;
+			DBG_MESSAGE("simmain", "banner closed");
+		}
+		if(  umgebung_t::quit_simutrans  ) {
+			break;
+		}
+
 		// to purge all previous old messages
+		DBG_MESSAGE("simmain", "set_message_flags");
 		welt->get_message()->set_message_flags(umgebung_t::message_flags[0], umgebung_t::message_flags[1], umgebung_t::message_flags[2], umgebung_t::message_flags[3]);
 
+		if(  !umgebung_t::networkmode  &&  !umgebung_t::server  ) {
+			welt->set_pause( false );
+		}
+
+#if 1
+		if(  new_world  ) {
+			modal_dialogue( new welt_gui_t(welt, &umgebung_t::default_einstellungen), magic_welt_gui_t, welt, never_quit );
+			if(  umgebung_t::quit_simutrans  ) {
+				break;
+			}
+		}
+#else
 		if (new_world) {
-			welt_gui_t *wg = new welt_gui_t(welt, &umgebung_t::default_einstellungen);
-			sprachengui_t* sg = new sprachengui_t();
-			climate_gui_t* cg = new climate_gui_t(wg, &umgebung_t::default_einstellungen);
+			climate_gui_t *cg = new climate_gui_t(&umgebung_t::default_einstellungen);
 			event_t ev;
 
 			view->display(true);
 
-			// we want to center wg (width 260) between sg (width 220) and cg (176)
-
-			create_win(10, 40, sg, w_info, magic_sprachengui_t );
-			create_win((disp_width - 220 - cg->get_fenstergroesse().x -10 -10- 260)/2 + 220 + 10, (disp_height - 300) / 2, wg, w_do_not_delete, magic_welt_gui_t );
 			create_win((disp_width - cg->get_fenstergroesse().x-10), 40, cg, w_info, magic_climate );
 
-			setsimrand(dr_time(), dr_time());
+			// we want to center wg (width 260) between sg (width 220) and cg (176)
+			welt_gui_t *wg = new welt_gui_t(welt, &umgebung_t::default_einstellungen);
+			create_win((disp_width - 220 - cg->get_fenstergroesse().x -10 -10- 260)/2 + 220 + 10, (disp_height - 300) / 2, wg, w_do_not_delete, magic_welt_gui_t );
 
 			do {
+				// disable pause to allow for screen updates
+				welt->set_pause( false );
+				welt->set_fast_forward(false);
+
 				INT_CHECK("simmain 803");
+				DBG_DEBUG4("wait_for_new_world", "calling win_poll_event");
 				win_poll_event(&ev);
 				INT_CHECK("simmain 805");
+				DBG_DEBUG4("wait_for_new_world", "calling check_pos_win");
 				check_pos_win(&ev);
+				if(  ev.ev_class == EVENT_SYSTEM  &&  ev.ev_code == SYSTEM_QUIT  ) {
+					umgebung_t::quit_simutrans = true;
+				}
 				INT_CHECK("simmain 807");
+				if(  umgebung_t::networkmode  ) {
+					static int count = 0;
+					if(  ((count++)&7)==0 ) {
+						static uint32 last_step = dr_time();
+						uint32 next_step = dr_time();
+						DBG_DEBUG4("wait_for_new_world", "calling welt->sync_step");
+						welt->sync_step( next_step-last_step, true, true );
+						DBG_DEBUG4("wait_for_new_world", "calling win_poll_event");
+						welt->step();
+						DBG_DEBUG4("wait_for_new_world", "calling welt->step");
+						last_step = next_step;
+						DBG_DEBUG4("wait_for_new_world", "back from welt->step");
+					}
+				}
 				dr_sleep(5);
+				welt->reset_interaction();
+				DBG_DEBUG4("wait_for_new_world", "end of loop");
 			} while(
 				!wg->get_load() &&
 				!wg->get_scenario() &&
 				!wg->get_load_heightfield() &&
 				!wg->get_start() &&
 				!wg->get_close() &&
-				!wg->get_quit()
+				!wg->get_quit() &&
+				!umgebung_t::quit_simutrans
 			);
 
 			if (IS_LEFTCLICK(&ev)) {
 				do {
+					DBG_DEBUG4("wait_for_new_world", "calling display_get_event");
 					display_get_event(&ev);
 				} while (!IS_LEFTRELEASE(&ev));
 			}
 
+			DBG_DEBUG4("wait_for_new_world", "calling destroy_all_win");
+			destroy_all_win(true);
+			welt->get_message()->clear();
+
 			// scenario?
 			if(wg->get_scenario()) {
 				char path[1024];
-				destroy_win( magic_climate );
-				destroy_win( magic_sprachengui_t );
-				destroy_win( magic_welt_gui_t );
-				delete wg;
-				sprintf( path, "%s%sscenario/", umgebung_t::program_dir, (const char *)umgebung_t::objfilename );
+				sprintf( path, "%s%sscenario/", umgebung_t::program_dir, umgebung_t::objfilename.c_str() );
 				chdir( path );
+				delete wg;
 				create_win( new scenario_frame_t(welt), w_info, magic_load_t );
 				chdir( umgebung_t::user_dir );
 			}
 			// Neue Karte erzeugen
 			else if (wg->get_start()) {
-				destroy_win( magic_climate );
-				destroy_win( magic_sprachengui_t );
-				destroy_win( magic_welt_gui_t );
 				// since not autodelete
+				DBG_DEBUG4("wait_for_new_world", "delete wg");
 				delete wg;
 
 				create_win(200, 100, new news_img("Erzeuge neue Karte.\n", skinverwaltung_t::neueweltsymbol->get_bild_nr(0)), w_info, magic_none);
+
+				DBG_DEBUG4("wait_for_new_world", "calling intr_refresh_display");
 				intr_refresh_display(true);
 
 				umgebung_t::default_einstellungen.heightfield = "";
+				DBG_DEBUG4("wait_for_new_world", "calling welt->init");
 				welt->init(&umgebung_t::default_einstellungen,0);
 
 				// save setting ...
 				loadsave_t file;
-				if(file.wr_open("default.sve",loadsave_t::binary,"settings only")) {
+				if(file.wr_open("default.sve",loadsave_t::binary,"settings only",SAVEGAME_VER_NR)) {
 					// save default setting
 					umgebung_t::default_einstellungen.rdwr(&file);
 					file.close();
 				}
-				destroy_all_win();
+				DBG_DEBUG4("wait_for_new_world", "calling destroy_all_win");
+				destroy_all_win(true);
+				DBG_DEBUG4("wait_for_new_world", "calling welt->step_month");
 				welt->step_month( umgebung_t::default_einstellungen.get_starting_month() );
-			} else if(wg->get_load()) {
-				destroy_win( magic_climate );
-				destroy_win( magic_sprachengui_t );
-				destroy_win( magic_welt_gui_t );
+				welt->set_pause(false);
+				DBG_DEBUG4("wait_for_new_world", "new world created");
+			}
+			else if(wg->get_load()) {
 				delete wg;
 				create_win( new loadsave_frame_t(welt, true), w_info, magic_load_t);
-			} else if(wg->get_load_heightfield()) {
-				destroy_win( magic_climate );
-				destroy_win( magic_sprachengui_t );
-				destroy_win( magic_welt_gui_t );
-				welt->load_heightfield(&umgebung_t::default_einstellungen);
-				delete wg;
-			} else {
-				destroy_win( magic_climate );
-				destroy_win( magic_sprachengui_t );
-				destroy_win( magic_welt_gui_t );
-				// quit the game
-				if (wg->get_quit()) break;
-				delete wg;
 			}
+			else if(wg->get_load_heightfield()) {
+				delete wg;
+				welt->load_heightfield(&umgebung_t::default_einstellungen);
+				welt->step_month( umgebung_t::default_einstellungen.get_starting_month() );
+			}
+			else {
+				// quit the game
+				if (wg->get_quit()  ||  umgebung_t::quit_simutrans  ) {
+					delete wg;
+					break;
+				}
+			}
+			DBG_DEBUG4("wait_for_new_world", "the end");
 		}
-
+#endif
+		printf( "Running world, pause=%i, fast forward=%i ... \n", welt->is_paused(), welt->is_fast_forward() );
 		loadgame = ""; // only first time
 
 		// run the loop
-		while(welt->interactive()) {
-#ifdef DEBUG
-			if(  welt->get_current_month() >= quit_month  ) {
-				umgebung_t::quit_simutrans = true;
-				break;
-			}
-#endif
-		}
+		welt->interactive(quit_month);
 
 		new_world = true;
 		welt->get_message()->get_message_flags(&umgebung_t::message_flags[0], &umgebung_t::message_flags[1], &umgebung_t::message_flags[2], &umgebung_t::message_flags[3]);
 		welt->set_fast_forward(false);
+		welt->set_pause(false);
+		setsimrand(dr_time(), dr_time());
 
-	} while (!umgebung_t::quit_simutrans);
+		printf( "World finished ...\n" );
+	}
 
 	intr_disable();
 
 	// save setting ...
 	chdir( umgebung_t::user_dir );
-	if(file.wr_open("settings.xml",loadsave_t::xml,"settings only/")) {
+	if(file.wr_open("settings.xml",loadsave_t::xml,"settings only/",SAVEGAME_VER_NR)) {
 		umgebung_t::rdwr(&file);
 		umgebung_t::default_einstellungen.rdwr(&file);
 		file.close();
 	}
 
+	welt->destroy();	// some compiler aparently do not like accessing welt during destroy
 	delete welt;
 	welt = NULL;
 
 	delete view;
 	view = 0;
 
+	network_core_shutdown();
+
 	simgraph_exit();
 
 	close_midi();
