@@ -6,6 +6,7 @@
  */
 
 #include <string.h>
+#include "dataobj/koord.h"
 #include "simdebug.h"
 #include "simticker.h"
 #include "simgraph.h"
@@ -38,6 +39,12 @@ bool ticker::empty()
 }
 
 
+void ticker::clear_ticker()
+{
+	list.clear();
+}
+
+
 void ticker::add_msg(const char* txt, koord pos, int color)
 {
 	// don't store more than 4 messages, it's useless.
