@@ -16,12 +16,12 @@
 #define _simvehikel_h
 
 #include "../simtypes.h"
+#include "../simconvoi.h"
 #include "../simdings.h"
 #include "../halthandle_t.h"
 #include "../convoihandle_t.h"
 #include "../ifc/fahrer.h"
 #include "../boden/grund.h"
-#include "../boden/wege/weg.h"
 #include "../besch/vehikel_besch.h"
 #include "../vehicle/overtaker.h"
 #include "../tpl/slist_tpl.h"
@@ -85,7 +85,7 @@ protected:
 	// cached image
 	image_id bild;
 
-	sint16 calc_height();		// Offset Bergauf/Bergab
+	sint8 calc_height();		// Offset Bergauf/Bergab
 
 	virtual bool hop_check() = 0;
 	virtual void hop() = 0;
@@ -101,6 +101,8 @@ protected:
 public:
 	// only called during load time: set some offsets
 	static void set_diagonal_multiplier( uint32 multiplier, uint32 old_multiplier );
+	static uint16 get_diagonal_multiplier() { return diagonal_multiplier; }
+
 	static void set_overtaking_offsets( bool driving_on_the_left );
 
 	// if true, this convoi needs to restart for correct alignment
@@ -111,10 +113,11 @@ public:
 	inline void set_bild( image_id b ) { bild = b; }
 	virtual image_id get_bild() const {return bild;}
 
-	sint16 get_hoff() const {return hoff;}
+	sint8 get_hoff() const {return hoff;}
+	uint8 get_steps() const {return steps;}
 
 	// to make smaller steps than the tile granularity, we have to calculate our offsets ourselves!
-	virtual void get_screen_offset( int &xoff, int &yoff ) const;
+	virtual void get_screen_offset( int &xoff, int &yoff, const sint16 raster_width ) const;
 
 	virtual void rotate90();
 
@@ -140,6 +143,12 @@ public:
 };
 
 
+template<> inline vehikel_basis_t* ding_cast<vehikel_basis_t>(ding_t* const d)
+{
+	return d->is_moving() ? static_cast<vehikel_basis_t*>(d) : 0;
+}
+
+
 /**
  * Klasse fr alle Fahrzeuge mit Route
  *
@@ -190,7 +199,7 @@ protected:
 	virtual void hop();
 
 	// current limit (due to track etc.)
-	uint32 speed_limit;
+	sint32 speed_limit;
 
 	ribi_t::ribi alte_fahrtrichtung;
 
@@ -225,6 +234,7 @@ protected:
 	bool ist_letztes:1;				// flags auskunft ber die position
 	bool rauchen:1;
 	bool check_for_finish:1;		// true, if on the last tile
+	bool has_driven:1;
 
 	virtual void calc_bild();
 
@@ -234,8 +244,6 @@ public:
 	// the coordinates, where the vehicle was loaded the last time
 	koord last_stop_pos;
 
-	enum { SPEED_UNLIMITED=0x07FFFFFF };
-
 	convoi_t *get_convoi() const { return cnv; }
 
 	virtual void rotate90();
@@ -260,7 +268,7 @@ public:
 
 	void darf_rauchen(bool yesno ) { rauchen = yesno;}
 
-	virtual bool calc_route(koord3d start, koord3d ziel, uint32 max_speed, route_t* route);
+	virtual bool calc_route(koord3d start, koord3d ziel, sint32 max_speed, route_t* route);
 	uint16 get_route_index() const {return route_index;}
 	const koord3d get_pos_prev() const {return pos_prev;}
 
@@ -332,6 +340,10 @@ public:
 	*/
 	inline int get_gesamtgewicht() const { return sum_weight; }
 
+	// returns speedlimit of ways (and if convoi enters station etc)
+	// the convoi takes care of the max_speed of the vehicle
+	sint32 get_speed_limit() const { return speed_limit; }
+
 	const slist_tpl<ware_t> & get_fracht() const { return fracht;}   // liste der gerade transportierten gter
 
 	/**
@@ -385,12 +397,12 @@ public:
 	* fahrzeug an haltestelle entladen
 	* @author Hj. Malthaner
 	*/
-	bool entladen(koord k, halthandle_t halt);
+	bool entladen(halthandle_t halt);
 
 	/**
 	* fahrzeug an haltestelle beladen
 	*/
-	bool beladen(koord k, halthandle_t halt);
+	bool beladen(halthandle_t halt);
 
 	// sets or querey begin and end of convois
 	void set_erstes(bool janein) {ist_erstes = janein;}
@@ -399,6 +411,9 @@ public:
 	void set_letztes(bool janein) {ist_letztes = janein;}
 	bool is_last() {return ist_letztes;}
 
+	// marks the vehicle as really used
+	void set_driven() { has_driven = true; }
+
 	virtual void set_convoi(convoi_t *c);
 
 	/**
@@ -429,6 +444,12 @@ public:
 };
 
 
+template<> inline vehikel_t* ding_cast<vehikel_t>(ding_t* const d)
+{
+	return dynamic_cast<vehikel_t*>(d);
+}
+
+
 /**
  * Eine Klasse fr Strassenfahrzeuge. Verwaltet das Aussehen der
  * Fahrzeuge und die Befahrbarkeit des Untergrundes.
@@ -452,9 +473,9 @@ public:
 	virtual void set_convoi(convoi_t *c);
 
 	// how expensive to go here (for way search)
-	virtual int get_kosten(const grund_t *,const uint32 ) const;
+	virtual int get_kosten(const grund_t *, const sint32, koord) const;
 
-	virtual bool calc_route(koord3d start, koord3d ziel, uint32 max_speed, route_t* route);
+	virtual bool calc_route(koord3d start, koord3d ziel, sint32 max_speed, route_t* route);
 
 	virtual bool ist_weg_frei(int &restart_speed);
 
@@ -462,13 +483,13 @@ public:
 	virtual bool ist_ziel(const grund_t *,const grund_t *) const;
 
 	// since we must consider overtaking, we use this for offset calculation
-	virtual void get_screen_offset( int &xoff, int &yoff ) const;
+	virtual void get_screen_offset( int &xoff, int &yoff, const sint16 raster_width ) const;
 
 	ding_t::typ get_typ() const { return automobil; }
 
 	schedule_t * erzeuge_neuen_fahrplan() const;
 
-	virtual overtaker_t *get_overtaker() { return cnv ? static_cast<overtaker_t *>(cnv) : NULL; }
+	virtual overtaker_t* get_overtaker() { return cnv; }
 };
 
 
@@ -489,14 +510,23 @@ protected:
 
 	void betrete_feld();
 
+	bool is_weg_frei_signal( uint16 start_index, int &restart_speed );
+
+	bool is_weg_frei_pre_signal( signal_t *sig, uint16 start_index, int &restart_speed );
+
+	bool is_weg_frei_longblock_signal( signal_t *sig, uint16 start_index, int &restart_speed );
+
+	bool is_weg_frei_choose_signal( signal_t *sig, uint16 start_index, int &restart_speed );
+
+
 public:
 	virtual waytype_t get_waytype() const { return track_wt; }
 
 	// since we might need to unreserve previously used blocks, we must do this before calculation a new route
-	bool calc_route(koord3d start, koord3d ziel, uint32 max_speed, route_t* route);
+	bool calc_route(koord3d start, koord3d ziel, sint32 max_speed, route_t* route);
 
 	// how expensive to go here (for way search)
-	virtual int get_kosten(const grund_t *,const uint32 ) const;
+	virtual int get_kosten(const grund_t *, const sint32, koord) const;
 
 	// returns true for the way search to an unknown target.
 	virtual bool ist_ziel(const grund_t *,const grund_t *) const;
@@ -505,13 +535,12 @@ public:
 	virtual bool ist_weg_frei(int &restart_speed);
 
 	// reserves or unreserves all blocks and returns the handle to the next block (if there)
-	// if count is larger than 1, maximum 64 tiles will be checked (freeing or reserving a choose signal path)
-	// return the last checked block
-	uint16 block_reserver(const route_t *route,uint16 start_index,int count, bool reserve) const;
+	// returns true on successful reservation
+	bool block_reserver(const route_t *route, uint16 start_index, uint16 &next_signal, uint16 &next_crossing, int signal_count, bool reserve, bool force_unreserve ) const;
 
 	void verlasse_feld();
 
-	enum ding_t::typ get_typ() const { return waggon; }
+	typ get_typ() const { return waggon; }
 
 	waggon_t(karte_t *welt, loadsave_t *file, bool is_first, bool is_last);
 	waggon_t(koord3d pos, const vehikel_besch_t* besch, spieler_t* sp, convoi_t *cnv); // start und fahrplan
@@ -538,7 +567,7 @@ public:
 	monorail_waggon_t(karte_t *welt, loadsave_t *file, bool is_first, bool is_last) : waggon_t(welt, file,is_first, is_last) {}
 	monorail_waggon_t(koord3d pos, const vehikel_besch_t* besch, spieler_t* sp, convoi_t* cnv) : waggon_t(pos, besch, sp, cnv) {}
 
-	enum ding_t::typ get_typ() const { return monorailwaggon; }
+	typ get_typ() const { return monorailwaggon; }
 
 	schedule_t * erzeuge_neuen_fahrplan() const;
 };
@@ -559,7 +588,7 @@ public:
 	maglev_waggon_t(karte_t *welt, loadsave_t *file, bool is_first, bool is_last) : waggon_t(welt, file, is_first, is_last) {}
 	maglev_waggon_t(koord3d pos, const vehikel_besch_t* besch, spieler_t* sp, convoi_t* cnv) : waggon_t(pos, besch, sp, cnv) {}
 
-	enum ding_t::typ get_typ() const { return maglevwaggon; }
+	typ get_typ() const { return maglevwaggon; }
 
 	schedule_t * erzeuge_neuen_fahrplan() const;
 };
@@ -580,7 +609,7 @@ public:
 	narrowgauge_waggon_t(karte_t *welt, loadsave_t *file, bool is_first, bool is_last) : waggon_t(welt, file, is_first, is_last) {}
 	narrowgauge_waggon_t(koord3d pos, const vehikel_besch_t* besch, spieler_t* sp, convoi_t* cnv) : waggon_t(pos, besch, sp, cnv) {}
 
-	enum ding_t::typ get_typ() const { return narrowgaugewaggon; }
+	typ get_typ() const { return narrowgaugewaggon; }
 
 	schedule_t * erzeuge_neuen_fahrplan() const;
 };
@@ -598,7 +627,7 @@ class schiff_t : public vehikel_t
 {
 protected:
 	// how expensive to go here (for way search)
-	virtual int get_kosten(const grund_t*, const uint32) const { return 1; }
+	virtual int get_kosten(const grund_t *, const sint32, koord) const { return 1; }
 
 	void calc_akt_speed(const grund_t *gr);
 
@@ -646,7 +675,7 @@ private:
 
 	enum flight_state { taxiing=0, departing=1, flying=2, landing=3, looking_for_parking=4, flying2=5, taxiing_to_halt=6  };
 
-	enum flight_state state;	// functions needed for the search without destination from find_route
+	flight_state state;	// functions needed for the search without destination from find_route
 
 	sint16 flughoehe;
 	sint16 target_height;
@@ -681,15 +710,15 @@ public:
 	virtual ribi_t::ribi get_ribi(const grund_t* ) const;
 
 	// how expensive to go here (for way search)
-	virtual int get_kosten(const grund_t *,const uint32 ) const;
+	virtual int get_kosten(const grund_t *, const sint32, koord) const;
 
 	virtual bool ist_weg_frei(int &restart_speed);
 
 	virtual void set_convoi(convoi_t *c);
 
-	bool calc_route(koord3d start, koord3d ziel, uint32 max_speed, route_t* route);
+	bool calc_route(koord3d start, koord3d ziel, sint32 max_speed, route_t* route);
 
-	enum ding_t::typ get_typ() const { return aircraft; }
+	typ get_typ() const { return aircraft; }
 
 	schedule_t * erzeuge_neuen_fahrplan() const;
 
@@ -697,20 +726,22 @@ public:
 
 	int get_flyingheight() const {return flughoehe-hoff-2;}
 
-	// since our image is the shadow ...
-	virtual image_id get_bild() const {return IMG_LEER;}
+	// image: when flying empty, on ground the plane
+	virtual image_id get_bild() const {return !is_on_ground() ? IMG_LEER : bild;}
 
-	// outline is the planes shadow
-	virtual PLAYER_COLOR_VAL get_outline_bild() const {return bild;}
+	// image: when flying the shadow, on ground empty
+	virtual PLAYER_COLOR_VAL get_outline_bild() const {return !is_on_ground() ? bild : IMG_LEER;}
 
-	// shadow has black color
-	virtual PLAYER_COLOR_VAL get_outline_colour() const {return TRANSPARENT75_FLAG | OUTLINE_FLAG | COL_BLACK;}
+	// shadow has black color (when flying)
+	virtual PLAYER_COLOR_VAL get_outline_colour() const {return !is_on_ground() ? TRANSPARENT75_FLAG | OUTLINE_FLAG | COL_BLACK : 0;}
 
-	// this draws the "real" aircrafts!
+	// this draws the "real" aircrafts (when flying)
 	virtual void display_after(int xpos, int ypos, bool dirty) const;
 
 	// the speed calculation happens it calc_height
 	void calc_akt_speed(const grund_t*) {}
+
+	bool is_on_ground() const { return flughoehe==0  &&  state!=flying; }
 };
 
 #endif
